/*******************************************************************************
 * GX sample program
 *
 * Rotate and mirror image example.
 ******************************************************************************/

#include "gxsd.h"
#include "gxerror.h"
#include "gximage.h"
#include "gxsdldr.c"

/*******************************************************************************
 * If an error occurred it displays the error and terminates the program
 ******************************************************************************/
void term_if_err(int st) {
	int err_code;
	wchar_t err_str[256];
	err_str[(sizeof(err_str)/sizeof(wchar_t))-1] = 0;

	if(st) return;		/* No error occurred */

	/* Displays error code and description */
	gx_geterror(&err_code, err_str, (sizeof(err_str)/sizeof(wchar_t))-1);
	fprintf(stderr, "GX error occurred: %i %ls\n", err_code, err_str);

	/* Terminates the program */
	exit(1);
}

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main(void) {
	struct gxHANDLE handle;
	struct gxIMAGE *image, *workimg;
	int st, degree;
	char fname[128];
	
	/* Creates a new image handler with default properties */
	st = gx_openmodule(&handle, L"gximage", L"default");
	term_if_err(st);
	
	/* Creates a new image structure */
	st = gx_allocimage(handle, &image);
	term_if_err(st);
	
	/* Loads the sample image */
	st = gx_loadimage(handle, image, L"../../../data/mushroom.jpg", GX_UNDEF);
	term_if_err(st);
	
	/* Creates an image structure to work */
	st = gx_allocimage(handle, &workimg);
	term_if_err(st);
	
	/* Rotates and saves the image */
	for(degree = 90; degree <= 360; degree += 90) {
	
		/* Rotates the image */
		st = gx_rotateimage(handle, workimg, image, degree);
		term_if_err(st);
		
		/* Saves it */
		sprintf(fname, "test_rotate%i.jpg", degree);
		st = gx_saveimagea(handle, workimg, fname, GX_JPEG);
		term_if_err(st);
	}
	
	/* Mirrors the image */
	st = gx_mirrorimage(handle, workimg, image, GX_MIRROR_VERTICAL);
	term_if_err(st);
	
	/* Saves it */
	st = gx_saveimage(handle, workimg, L"test_mirror.jpg", GX_JPEG);
	term_if_err(st);

	/* Frees the image structure */
	st = gx_unrefimage(handle, image);
	term_if_err(st);

	/* Frees the workimg structure */
	st = gx_unrefimage(handle, workimg);
	term_if_err(st);
	
	/* Closes the opened image module */
	st = gx_closehandle(&handle);
	term_if_err(st);

	return 0;
}
