#ifndef FPSTYPES_INCL
#define FPSTYPES_INCL
/*******************************************************************************
 * FPS TYPES 2010-2021 (c) Adaptive Recognition
 * 2021.06.03 v2.1.10
 ******************************************************************************/
#ifdef GX_DOTNET
	#using "gxdotnet7.dll"
#endif
#include "gxtypes.h"
/******************************************************************************/
/**
  \file fpstypes.h
  <b>Fingerprint Scanner Api module constants</b>

  The Fingerprint Scanner is a software  product by <b>ADAPTIVE RECOGNITION</b>,
  which is for capturing and handling live fingerprints  from ten fingers and it
  bases on the GX system.  The fpstypes.h contains the descriptions of constants
  of the api module.
*/
/******************************************************************************/
#ifdef GX_NAMESPACES
	namespace fps {
#endif

#ifdef GX_DOTNET
	#pragma managed
#endif

/** Device connection modes in the fpsapi */

GX_ENUM FPS_USAGEMODE {
	FPS_UMODE_DISCONNECT	= 0,		/**< Disconnect the actually used device */
	FPS_UMODE_NORMAL_USE	= 1,		/**< Connect the device with restricted abilities */
	FPS_UMODE_FULL_CONTROL	= 2			/**< Connect the device with all the abilities */
};

/**
  Device freerun light controlling modes. Not all of the modes can be used at the same time.
  The system can override the control settings.
*/

GX_ENUM FPS_FREERUNMODE {
	FPS_FRMODE_NULL			= 0x0000,	/**< Disable freerun activity */
	FPS_FRMODE_PREVIEWLIGHT	= 0x0001,	/**< Direct controlled lights for real-time preview image capturing */
	FPS_FRMODE_TESTPRESENCE	= 0x0004,	/**< Lights controlled by the finger presence detection algorithm */
	FPS_FRMODE_HQPREVIEW	= 0x0010,	/**< High quality preview */
	FPS_FRMODE_ROLLPREVIEW	= 0x0020	/**< Roll mode preview */
};

/** Result of the finger presence detection module */

GX_ENUM FPS_TESTPRESENCE {
	FPS_TP_ABSENT	= 0,				/**< Fingers are out of the scan area */
	FPS_TP_DEFAULT	= 2,				/**< The test is starting */
	FPS_TP_DIRTY	= 3,				/**< Platen needs cleaning */
	FPS_TP_PRESENT	= 4					/**< Fingers are on the scan area */
};

/** Image types of the fpsapi functions (get-, save-, drawimage) */

GX_ENUM FPS_IMAGE_TYPE {
	FPS_IT_PREVIEW = 2,					/**< Low resolution real-time captured image */
	FPS_IT_FINGER = 3,					/**< Captured finger image */
};

/** The event types and the events of the fpsapi module */

GX_ENUM FPS_EVENT {
	FPS_ET_CAPTURE			= 2,		/**< Capture events */
	FPS_ET_IO				= 4,		/**< Input events */

	FPS_EV_CAPTUREFINISHED	= 101,		/**< Capture process finished (Capture event) */
	FPS_EV_IMAGECAPTURED	= 102,		/**< Image captured (Capture event) */
	FPS_EV_FGCAP			= 106,		/**< Fingers captured (Capture event) */
	FPS_EV_PREVIEWCAPTURED	= 107,		/**< Preview captured (Capture event) */

	FPS_EV_FINGERDETECTION	= 201,		/**< Finger detection state changed (Input event) */
	FPS_EV_BUTTON			= 202,		/**< Button state changed (Input event) */
	FPS_EV_POWER			= 203,		/**< Power state or battery level changed (Input event) */
	FPS_EV_CONNECTION		= 204,		/**< Device list changes (Input event), occures when no device is used and a new device is connected or the used device is disconnected */
};

#ifdef WIN32

/** The window messages of the FPS system. */

GX_ENUM FPS_WND_MESSAGE {
	FPS_WM_CAPTUREFINISHED	= 1,		/**< Window message type for capture finished event */
	FPS_WM_IMAGECAPTURED	= 2,		/**< Window message type for image captured event */
	FPS_WM_PREVIEWCAPTURED	= 5,		/**< Window message type for preview captured event */
	FPS_WM_FINGERDETECTION	= 6,		/**< Window message type for finger detection state changed event */
	FPS_WM_BUTTON			= 7,		/**< Window message type for button state changed event */
	FPS_WM_POWER			= 8,		/**< Window message type for power state or battery level changed event */
	FPS_WM_CONNECTION		= 9,		/**< Window message type for device list changes, occures when no device is used and a new device is connected or the used device is disconnected */
	FPS_WM_FGCAP			= 11,		/**< Window message type for fingers captured event */

	FPS_WM_LAST_VALUE		= 31,		/**< The end of the reserved space for future window message types */
};

#endif

/** Power state codes of devices */

GX_ENUM FPS_POWER_STATE {
	FPS_PWR_MASK			= 0x000f,	/**< Mask for power data */

	FPS_PWR_UNKNOWN			= 0x0000,	/**< The power state is unknown (cannot be tested) */
	FPS_PWR_OFF				= 0x0001,	/**< The power is off */
	FPS_PWR_ON				= 0x0002,	/**< The power is on */

	FPS_BATTERY_MASK		= 0x00f0,	/**< Mask for battery data */

	FPS_BATTERY_UNAVAILABLE	= 0x0000,	/**< The battery is unavailbe */
	FPS_BATTERY_AVAILABLE	= 0x0010,	/**< The battery is availbe */
	FPS_BATTERY_UNKNOWN		= 0x0020,	/**< The state of the battery is unavailbe */
};

/** Status led color codes of FPS devices */

GX_ENUM FPS_STATUS_LED_COLOR {
	FPS_SLC_OFF		= 0x00,				/**< The led is off */
	FPS_SLC_GREEN	= 0x81,				/**< The color of the led is green */
	FPS_SLC_RED		= 0x82,				/**< The color of the led is red */
	FPS_SLC_ORANGE	= 0x83,				/**< The color of the led is orange */
	FPS_SLC_BLUE	= 0x84,				/**< The color of the led is blue */
	FPS_SLC_AUTO	= 0xac,				/**< Automatic control */
	FPS_SLC_ANY		= 0xff,				/**< The color of the led is one of the available lights */
};

/** Id of the buzzer */

GX_ENUM FPS_BUZZER {
	FPS_SLC_BUZZER	= 0xc0,		/**< Buzzer */
};

/** Button state codes of FPS devices */

GX_ENUM FPS_KEY_STATE {
	FPS_KEY_UNKNOWN		= 0,			/**< The state or presence of the button is unknown (cannot be tested) */
	FPS_KEY_PRESSED		= 1,			/**< The state of the button is pressed */
	FPS_KEY_RELEASED	= 2,			/**< The state of the button is released */
	FPS_KEY_NOKEY		= 3				/**< The button does not exist on the device */
};

/** Button codes of FPS devices */

GX_ENUM FPS_KEY_CODE {
	FPS_KEY_POWER	= 0x10101,			/**< Code of the power button */
	FPS_KEY_START	= 0x10110,			/**< Code of the start button */
};

/** Variant IDs used. */

GX_ENUM FPS_VAR_ID {

	FPSV_ROOT			= 0,					/**< The root of the document structure */

	FPSV_PAGELIST		= FPSV_ROOT | 1,		/**< Reserved for internal use */

	FPSV_CONFIDENCE		= 0xD0000005,			/**< Confidence value */

	FPSV_IMAGE			= 0xD0100000,			/**< Image */
	FPSV_IMAGELIST		= FPSV_IMAGE | 1,		/**< List of images */
	FPSV_IMAGEFRAME		= FPSV_IMAGE | 2,		/**< Frame by image coordinates (1/16 pixel) */

	FPSV_DEVICE_INFO	= 0xD0200000,			/**< The root of the device info structure */
	FPSV_DEVICE_ID		= FPSV_DEVICE_INFO | 1,	/**< Device id string */
	FPSV_CAPTIME		= FPSV_DEVICE_INFO | 3,	/**< Date and time of capture */

	FPSV_COMMON			= 0xD0700000,			/**< RFU */
	FPSV_PROCTIME		= FPSV_COMMON | 1,		/**< Date and time of process execution */
	FPSV_PROCPERIOD		= FPSV_COMMON | 2,		/**< Duration of process execution */
	FPSV_WARNLIST		= FPSV_COMMON | 3,		/**< List of warning messages */
	FPSV_TASK			= FPSV_COMMON | 4,		/**< Call code of a GX task */

	FPSV_FINGER			= 0xD0800000,			/**< Finger position, see \ref FPS_POSITION */
	FPSV_FINGER_TYPE	= FPSV_FINGER | 1,		/**< Finger impression type, see \ref FPS_IMPRESSION_TYPE */
	FPSV_NIST_QUALITY	= FPSV_FINGER | 2,		/**< NIST image quality value */
	FPSV_DRYWET			= FPSV_FINGER | 3,		/**< Finger dry/wet level */
	FPSV_FRACTION		= FPSV_FINGER | 4,		/**< Finger fraction level */
};

/** Finger storage IDs. */

GX_ENUM FPS_POSITION {
	FPS_POS_UNKNOWN			= 0,			/**< Unknown finger */
	FPS_POS_RIGHT_THUMB		= 1,			/**< Right thumb */
	FPS_POS_RIGHT_INDEX		= 2,			/**< Right index finger */
	FPS_POS_RIGHT_MIDDLE	= 3,			/**< Right middle finger */
	FPS_POS_RIGHT_RING		= 4,			/**< Right ring finger */
	FPS_POS_RIGHT_LITTLE	= 5,			/**< Right little finger */
	FPS_POS_LEFT_THUMB		= 6,			/**< Left thumb */
	FPS_POS_LEFT_INDEX		= 7,			/**< Left index finger */
	FPS_POS_LEFT_MIDDLE		= 8,			/**< Left middle finger */
	FPS_POS_LEFT_RING		= 9,			/**< Left ring finger */
	FPS_POS_LEFT_LITTLE		= 10,			/**< Left little finger */

	FPS_POS_PLAIN_RIGHT_THUMB		= 11,	/**< Plain right thumb */
	FPS_POS_PLAIN_LEFT_THUMB		= 12,	/**< Plain left thumb */
	FPS_POS_PLAIN_RIGHT_4FINGERS	= 13,	/**< Plain right four fingers */
	FPS_POS_PLAIN_LEFT_4FINGERS		= 14,	/**< Plain left four fingers */
	FPS_POS_PLAIN_THUMBS			= 15,	/**< Plain two thumbs together */

	FPS_POS_BOTH_INDEXES			= 46,	/**< Plain two index fingers together */
};

/** The target object of fingerprint scanning. */

GX_ENUM FPS_SCAN_OBJECT {
	FPS_SCAN_LEFT_HAND		= 1,		/**< Scan left hand fingers */
	FPS_SCAN_RIGHT_HAND		= 2,		/**< Scan right hand fingers */
	FPS_SCAN_SAME_FINGER	= 3,		/**< Scan same fingers of both hands */
};

/** The capturing mode of the finger capture process. */

GX_ENUM FPS_IMPRESSION_TYPE {
	FPS_SCAN_LIVE			= 0,		/**< Live scan plain */
	FPS_SCAN_ROLLED			= 1,		/**< Live scan rolled */
	FPS_SCAN_ACCUMULATED	= 0x1000,	/**< Accumulated live scan */
};

/** Finger existence IDs. */

GX_ENUM FPS_PRESENCE {
	FPS_NO_INFO		= 0,				/**< No Information about missing finger */
	FPS_AMPUTATION	= 1,				/**< Amputation */
	FPS_NO_PRINT	= 2,				/**< Unable to scan */
	FPS_AVAILABLE	= 3,				/**< Finger is available */
};

/** Warnings about finger capture. */

GX_ENUM FPS_FAILURE {
	FPS_FAIL_LEFT		= 1,			/**< Fingers hang out at the left side of the window */
	FPS_FAIL_RIGHT		= 2,			/**< Fingers hang out at the right side of the window */
	FPS_FAIL_TOP		= 4,			/**< Fingers hang out at the top of the window */
	FPS_FAIL_BOTTOM		= 8,			/**< Fingers hang out at the bottom of the window */
	FPS_FAIL_MISSING	= 0x10,			/**< Some fingers missing */
	FPS_FAIL_EXTRA		= 0x20,			/**< Some additional fingers found */
	FPS_FAIL_HAND		= 0x40,			/**< Mismatched hand found */
	FPS_FAIL_UPRIGHT	= 0x80,			/**< The fingers don't stand in upright position */
};

/******************************************************************************/
#ifdef GX_NAMESPACES
	}
#endif
/******************************************************************************/
#endif//FPSTYPES_INCL
