//------------------------------------------------------------------------------
// GX Sample 1 program
//------------------------------------------------------------------------------
// It is a sample program for the GxImage OCX ActiveX control.
// It shows the image viewing functionalities.

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtDlgs, StdCtrls, OleCtrls, GXIMAGE_OCXLib_TLB, Menus, ExtCtrls, Spin,
  ComCtrls, Math, Grids, AxCtrls;

type
  TFSample1 = class(TForm)
	OpenPictureDialog: TOpenPictureDialog;
	MainMenu: TMainMenu;
	FileMenu: TMenuItem;
	LoadImage: TMenuItem;
	HelpMenu: TMenuItem;
	ImageInfo: TStatusBar;
	PImage: TPanel;
	SbVert: TScrollBar;
	SbHoriz: TScrollBar;
	PView: TPanel;
	ViewMode: TRadioGroup;
	PartView: TGroupBox;
	LWndLeft: TLabel;
	LWndRight: TLabel;
	LWndTop: TLabel;
	LWndBottom: TLabel;
	WndTop: TSpinEdit;
	PWnd: TPanel;
	WndRight: TSpinEdit;
	WndBottom: TSpinEdit;
	WndLeft: TSpinEdit;
	StretchImage: TCheckBox;
	StretchMode: TRadioGroup;
	ModifyView: TGroupBox;
	RotateViewCCW: TButton;
	HMirror: TButton;
	VMirror: TButton;
	RotateViewCW: TButton;
	ZoomView: TGroupBox;
	LHZoomPos: TLabel;
	LVZoomPos: TLabel;
	LZoom: TLabel;
	Zoom: TTrackBar;
	HZoomPos: TSpinEdit;
	VZoomPos: TSpinEdit;
	PLoadImage: TPanel;
	Quit: TMenuItem;
	About: TMenuItem;
	img: TGxImageOCX;
	procedure LoadImageClick(Sender: TObject);
	procedure RotateViewCWClick(Sender: TObject);
	procedure RotateViewCCWClick(Sender: TObject);
	procedure StretchModeClick(Sender: TObject);
	procedure ViewModeClick(Sender: TObject);
	procedure WndTopChange(Sender: TObject);
	procedure WndLeftChange(Sender: TObject);
	procedure WndRightChange(Sender: TObject);
	procedure WndBottomChange(Sender: TObject);
	procedure SbHorizChange(Sender: TObject);
	procedure SbVertChange(Sender: TObject);
	procedure StretchImageClick(Sender: TObject);
	procedure ZoomChange(Sender: TObject);
	procedure HZoomPosChange(Sender: TObject);
	procedure VZoomPosChange(Sender: TObject);
	procedure QuitClick(Sender: TObject);
	procedure HMirrorClick(Sender: TObject);
	procedure VMirrorClick(Sender: TObject);
	procedure AboutClick(Sender: TObject);
  private
	{ Private declarations }
  public
	{ Public declarations }
	procedure InfoRefresh;
	procedure InitControls;
	procedure ShowError;
  end;

var
  FSample1: TFSample1;

implementation

{$R *.DFM}

procedure TFSample1.LoadImageClick(Sender: TObject);
begin
  // Loaging the image
  if OpenPictureDialog.Execute then begin
	// The Load Image function returns non 0 (true) value on success, 0 on error.
	if (img.LoadImage(OpenPictureDialog.FileName) = 0) then begin
	  // Loading failed...
	  ShowError;
	  ImageInfo.SimpleText := 'Loading image ' + OpenPictureDialog.FileName + ' failed!';
	end else begin
	  // Image loading succeeded.
	  InitControls;
	end;
  end;
end;

procedure TFSample1.InitControls;
begin
	// Setting the defaults
	img.ViewMode := vmNormal;
	img.StretchImage := true;
	img.StretchMode := smBilinear;

	img.wndLeft := 0;
	img.wndRight := img.XSize;
	img.wndTop := 0;
	img.wndBottom := img.YSize;

	img.Zoom := 1;
	img.HZoomPos := img.XSize div 2;
	img.VZoomPos := img.YSize div 2;

	ViewMode.Enabled := false;
	StretchImage.Enabled := false;
	StretchMode.Enabled := false;
	ModifyView.Enabled := false;
	PartView.Enabled := false;
	ZoomView.Enabled := false;

	// Stretch Modes
	StretchImage.Enabled := true;
	StretchImage.Checked := img.StretchImage;
	StretchMode.Enabled := StretchImage.Checked;
	StretchMode.ItemIndex := img.StretchMode - 1;

	// View manipulating panels
	ModifyView.Enabled := true;
	ZoomView.Enabled := (img.ViewMode = vmZoom);
	PartView.Enabled := (img.ViewMode = vmPart);

	// Scrollbars init
	SbHoriz.Enabled := not (img.ViewMode = vmNormal);
	SbHoriz.Position := 0;
	SbVert.Enabled := not (img.ViewMode = vmNormal);
	SbVert.Position := 0;

	// View Modes
	ViewMode.Enabled := true;
	ViewMode.ItemIndex := img.ViewMode - 1;

	// Zoom View Mode
	HZoomPos.MaxValue := img.XSize;
	VZoomPos.MaxValue := img.YSize;
	HZoomPos.Value := img.HZoomPos;
	VZoomPos.Value := img.VZoomPos;
	Zoom.Position := Round(Log2(img.Zoom))*2 + 10;
	LZoom.Caption := 'Zoom: ' + FloatToStrF(img.Zoom, ffFixed, 2, 2);

	// Part View Mode
	WndLeft.MaxValue := img.XSize;
	WndLeft.Value := img.WndLeft;

	WndTop.MaxValue := img.YSize;
	WndTop.Value := img.WndTop;

	WndRight.MaxValue := img.XSize;
	WndRight.Value := img.WndRight;

	WndBottom.MaxValue := img.YSize;
	WndBottom.Value := img.WndBottom;

	// Refreshing information bar (status bar)
	InfoRefresh;
end;

procedure TFSample1.RotateViewCWClick(Sender: TObject);
begin
  // Rotating 90 Clockwise
  img.Rotation := (img.Rotation+90) mod 360;
  InfoRefresh;
end;

procedure TFSample1.RotateViewCCWClick(Sender: TObject);
begin
  // Rotating 90 Counter Clockwise
  img.Rotation := (img.Rotation-90) mod 360;
  InfoRefresh;
end;

procedure TFSample1.StretchModeClick(Sender: TObject);
begin
  // Changing the stretch mode
  img.StretchMode := StretchMode.ItemIndex + 1;
end;

procedure TFSample1.ViewModeClick(Sender: TObject);
begin
  // changing the view mode
  img.ViewMode := ViewMode.ItemIndex + 1;

  ZoomView.Enabled := (img.ViewMode = vmZoom);
  PartView.Enabled := (img.ViewMode = vmPart);
  StretchImage.Enabled := (img.ViewMode <> vmZoom);

  SbHoriz.Enabled := not (img.ViewMode = vmNormal);
  SbVert.Enabled := not (img.ViewMode = vmNormal);

  if (img.ViewMode = vmPart) then begin
	SbHoriz.Max := img.XSize - (wndRight.Value - wndLeft.Value);
	SbHoriz.Position := WndLeft.Value;
	SbVert.Max := img.YSize - (wndBottom.Value - wndTop.Value);
	SbVert.Position := WndTop.Value;
  end;
  if (img.ViewMode = vmZoom) then begin
	SbHoriz.Max := img.XSize;
	SbHoriz.Position := img.HZoomPos;
	SbVert.Max := img.YSize;
	SbVert.Position := img.VZoomPos;
  end;
end;

procedure TFSample1.WndTopChange(Sender: TObject);
begin
  img.WndTop := WndTop.Value;
  WndBottom.MinValue := img.WndTop;
  SbVert.Max := img.YSize - (wndBottom.Value - wndTop.Value);
  SbVert.Position := SbVert.Max - (img.YSize - WndBottom.Value);
end;

procedure TFSample1.WndLeftChange(Sender: TObject);
begin
  img.WndLeft := WndLeft.Value;
  WndRight.MinValue := img.WndLeft;
  SbHoriz.Max := img.XSize - (wndRight.Value - wndLeft.Value);
  SbHoriz.Position := SbHoriz.Max - (img.XSize - WndRight.Value);
end;

procedure TFSample1.WndRightChange(Sender: TObject);
begin
  img.WndRight := WndRight.Value;
  WndLeft.MaxValue := img.WndRight;
  SbHoriz.Max := img.XSize - (wndRight.Value - wndLeft.Value);
  SbHoriz.Position := SbHoriz.Max - (img.XSize - WndRight.Value);
end;

procedure TFSample1.WndBottomChange(Sender: TObject);
begin
  img.WndBottom := WndBottom.Value;
  WndTop.MaxValue := img.WndBottom;
  SbVert.Max := img.YSize - (wndBottom.Value - wndTop.Value);
  SbVert.Position := SbVert.Max - (img.YSize - WndBottom.Value);
end;

procedure TFSample1.SbHorizChange(Sender: TObject);
begin
  if (img.ViewMode = vmPart) then begin
	img.WndRight := img.WndRight - (img.WndLeft - SbHoriz.Position);
	img.WndLeft := SbHoriz.Position;
	WndLeft.Value := img.WndLeft;
	WndRight.Value := img.WndRight;
  end;
  if (img.ViewMode = vmZoom) then begin
	img.HZoomPos := SbHoriz.Position;
	HZoomPos.Value := img.HZoomPos;
  end;
end;

procedure TFSample1.SbVertChange(Sender: TObject);
begin
  if (img.ViewMode = vmPart) then begin
	img.WndBottom := img.WndBottom - (img.WndTop - SbVert.Position);
	img.WndTop := SbVert.Position;
	WndTop.Value := img.WndTop;
	WndBottom.Value := img.WndBottom;
  end;
  if (img.ViewMode = vmZoom) then begin
	img.VZoomPos := SbVert.Position;
	VZoomPos.Value := img.VZoomPos;
  end;
end;

procedure TFSample1.StretchImageClick(Sender: TObject);
begin
  img.StretchImage := StretchImage.Checked;
end;

procedure TFSample1.ZoomChange(Sender: TObject);
begin
  // Zoom rate
  //
  // Control value varies between 0 and 20, 1:1 zoom rate corresponds to Zoom.Position = 10
  // The zoom level is changed as a power of 2, with 0.5 stepping:
  // ...  2^(-1)  2^(-0.5)  2^0  2^0.5  2^1  ...

  img.Zoom := Power(2, (Zoom.Position - 10)/2);
  LZoom.Caption := 'Zoom: ' + FloatToStrF(img.Zoom, ffFixed, 2, 2);
end;

procedure TFSample1.HZoomPosChange(Sender: TObject);
begin
  img.HZoomPos := HZoomPos.Value;
  SbHoriz.Position := HZoomPos.Value;
end;

procedure TFSample1.VZoomPosChange(Sender: TObject);
begin
  img.VZoomPos := VZoomPos.Value;
  SbVert.Position := VZoomPos.Value;
end;

procedure TFSample1.QuitClick(Sender: TObject);
begin
  FSample1.Close;
end;

procedure TFSample1.InfoRefresh;
begin
  // Refreshing information on status bar.
  ImageInfo.SimpleText :=
	'Image size: ' + IntToStr(img.XSize) + ' x ' + IntToStr(img.YSize) + ' pixels, ' +
	'Resolution: ' + IntToStr(img.HRes) + ' x ' + IntToStr(img.VRes) + ' pix/m, ' +
	'Line size: '  + IntToStr(img.SLine) + ' bytes ' +
	'Rotation: '   + IntToStr(img.Rotation) + ' ';
end;

procedure TFSample1.HMirrorClick(Sender: TObject);
begin
  // Mirroring horizontally: the MIRROR_HORIZONTAL flag/bit is set/unset
  img.Mirror := img.Mirror xor GX_MIRROR_HORIZONTAL;
end;

procedure TFSample1.VMirrorClick(Sender: TObject);
begin
  // Mirroring vertically: the MIRROR_VERTICAL flag/bit is set/unset
  img.Mirror := img.Mirror xor GX_MIRROR_VERTICAL;
end;

procedure TFSample1.AboutClick(Sender: TObject);
begin
  // About dialog box
  MessageDlg(
	'It is a sample program for the GxImage OCX ActiveX control.' + #13 +
	'It shows the image viewing functionalities.',
	mtInformation, [mbOk], 0);
end;

procedure TFSample1.ShowError;
var
  errcode: integer;
  errmsg: WideString;
begin
  // Getting error code and message
  img.GetError(errcode, errmsg);
  MessageDlg('Error No.: ' + IntToStr(errcode) + #13 + 'Error message: ' + errmsg, mtError, [mbOK], 0);
end;

end.

