//------------------------------------------------------------------------------
// GX Sample 2 program
//------------------------------------------------------------------------------
// It is a sample program for the GxImage OCX ActiveX control.
// It shows the image manipulating functionalities.

unit Unit2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, OleCtrls, GXIMAGE_OCXLib_TLB, ExtDlgs, ComCtrls, StdCtrls, Spin,
  ExtCtrls, Math, AxCtrls;

type
  TFSample2 = class(TForm)
	MainMenu1: TMainMenu;
	FileMenu: TMenuItem;
	HelpMenu: TMenuItem;
	Quit: TMenuItem;
	About: TMenuItem;
	LoadImage: TMenuItem;
	SaveImage: TMenuItem;
	SaveView: TMenuItem;
	N1: TMenuItem;
	N2: TMenuItem;
	New: TMenuItem;
	N3: TMenuItem;
	ConvertImage: TMenuItem;
	ImageInfo: TStatusBar;
	SavePictureDialog: TSavePictureDialog;
	OpenPictureDialog: TOpenPictureDialog;
	PImage: TPanel;
	SbVert: TScrollBar;
	SbHoriz: TScrollBar;
	Panel1: TPanel;
	ModifyView: TGroupBox;
	RotateImageCCW: TButton;
	HMirror: TButton;
	VMirror: TButton;
	RotateImageCW: TButton;
	CreateImage: TButton;
	ClearImage: TButton;
	ZoomView: TGroupBox;
	LHZoomPos: TLabel;
	LVZoomPos: TLabel;
	LZoom: TLabel;
	Zoom: TTrackBar;
	HZoomPos: TSpinEdit;
	VZoomPos: TSpinEdit;
	ResizeImage: TButton;
	NewWidth: TSpinEdit;
	NewHeight: TSpinEdit;
	LNewWidth: TLabel;
	LNewHeight: TLabel;
	img: TGxImageOCX;
	procedure AboutClick(Sender: TObject);
	procedure QuitClick(Sender: TObject);
	procedure FormCreate(Sender: TObject);
	procedure ConvertClick(Sender: TObject);
	procedure LoadImageClick(Sender: TObject);
	procedure NewClick(Sender: TObject);
	procedure RotateImageCWClick(Sender: TObject);
	procedure RotateImageCCWClick(Sender: TObject);
	procedure HMirrorClick(Sender: TObject);
	procedure VMirrorClick(Sender: TObject);
	procedure SaveImageClick(Sender: TObject);
	procedure SaveViewClick(Sender: TObject);
	procedure CreateImageClick(Sender: TObject);
	procedure ZoomChange(Sender: TObject);
	procedure SbVertChange(Sender: TObject);
	procedure SbHorizChange(Sender: TObject);
	procedure HZoomPosChange(Sender: TObject);
	procedure VZoomPosChange(Sender: TObject);
	procedure ResizeImageClick(Sender: TObject);
  private
	{ Private declarations }
  public
	{ Public declarations }
	procedure InitControls;
	procedure InfoRefresh;
	procedure ShowError;
  end;

var
  FSample2: TFSample2;

const
  FormatsNo = 13;
  Formats: array[1..FormatsNo] of string = (
	'GX_GRAY',  'GX_RGB555',  'GX_BGR555',  'GX_RGB565',  'GX_BGR565',
	'GX_RGB',   'GX_BGR',     'GX_RGBA',    'GX_BGRA',    'GX_YUV422',
	'GX_GRAY12','GX_RGB12',   'GX_BGR12'
  );
implementation

{$R *.DFM}

procedure TFSample2.AboutClick(Sender: TObject);
begin
  // About dialog box
  MessageDlg(
	'It is a sample program for the GxImage OCX ActiveX control.' + #13 +
	'It shows the image manipulating functionalities.',
	mtInformation, [mbOk], 0);
end;

procedure TFSample2.QuitClick(Sender: TObject);
begin
  FSample2.Close;
end;

procedure TFSample2.FormCreate(Sender: TObject);
var
  NewItem: TMenuItem;
  i: integer;
begin
  // Creating the Convert Image menu entries
  for i := 1 to FormatsNo do begin
	NewItem := TMenuItem.Create(Self);
	NewItem.Caption := Formats[i];
	NewItem.HelpContext := i;
	NewItem.OnClick := ConvertClick;
	ConvertImage.Add(NewItem);
  end;
  InitControls;
end;

procedure TFSample2.ConvertClick(Sender: TObject);
begin
  // Converting the image in the selected format
  if (((Sender as TMenuItem).HelpContext) = GX_GRAY) then
	MessageDlg('This will discard color infomation!', mtInformation, [mbOK], 0);

  img.ConvertImage(((Sender as TMenuItem).HelpContext), 0, 0, 0);
end;

procedure TFSample2.LoadImageClick(Sender: TObject);
begin
  // Loaging the image
  if OpenPictureDialog.Execute then begin
	// The Load Image function returns non 0 (true) value on success, 0 on error.
	if (img.LoadImage(OpenPictureDialog.FileName) = 0) then begin
	  // Loading failed...
	  ShowError;
	  ImageInfo.SimpleText := 'Loading image ' + OpenPictureDialog.FileName + ' failed!';
	end else begin
	  // Image loading succeeded.
	  InitControls;
	end;
  end;
end;

procedure TFSample2.NewClick(Sender: TObject);
begin
  // Discarding the image
  img.ClearImage;
  InitControls;
end;

procedure TFSample2.RotateImageCWClick(Sender: TObject);
begin
  // Rotating the image. Attention: this function changes the original image!
  img.RotateImage(90);
end;

procedure TFSample2.RotateImageCCWClick(Sender: TObject);
begin
  // Rotating the image. Attention: this function changes the original image!
  img.RotateImage(-90);
end;

procedure TFSample2.HMirrorClick(Sender: TObject);
begin
  // Mirroring the image. Attention: this function changes the original image!
  img.HMirrorImage;
end;

procedure TFSample2.VMirrorClick(Sender: TObject);
begin
  // Mirroring the image. Attention: this function changes the original image!
  img.VMirrorImage;
end;

procedure TFSample2.SaveImageClick(Sender: TObject);
begin
  if (SavePictureDialog.Execute) then begin
	// Saving the original image in the selected format
	if (img.SaveImage(SavePictureDialog.FileName, SavePictureDialog.FilterIndex) = 0) then begin
	  // Saving image failed...
	  ShowError;
	end;
  end;
end;

procedure TFSample2.SaveViewClick(Sender: TObject);
begin
  if (SavePictureDialog.Execute) then begin
	// Saving the actual view of the image in the selected format
	if (img.SaveView(SavePictureDialog.FileName, SavePictureDialog.FilterIndex) = 0) then begin
	  // Saving image failed...
	  ShowError;
	end;
  end;
end;

procedure TFSample2.CreateImageClick(Sender: TObject);
const
  XSize = 64;
  YSize = 64;
  SLine = XSize;
var
  image: array[1..XSize] of array[1..YSize] of byte;
  i,j: integer;
begin
  // Creating the chess board
  for i := 1 to XSize do
	for j := 1 to YSize do
	  image[j,i] := ((i + j) mod 2) * 255;

  // Creating an image from a buffer
  img.CreateImage(integer(@image), GX_GRAY, XSize, YSize,SLine);
  InitControls;
end;

procedure TFSample2.InitControls;
begin
	// Setting the defaults
	img.ViewMode := vmZoom;

	img.Zoom := 1;
	img.HZoomPos := img.XSize div 2;
	img.VZoomPos := img.YSize div 2;

	// Zoom View Mode
	HZoomPos.MaxValue := img.XSize;
	VZoomPos.MaxValue := img.YSize;
	HZoomPos.Value := img.HZoomPos;
	VZoomPos.Value := img.VZoomPos;
	Zoom.Position := Round(Log2(img.Zoom))*2 + 10;
	LZoom.Caption := 'Zoom: ' + FloatToStrF(img.Zoom, ffFixed, 2, 2);

	// Scrollbars init
	SbHoriz.Max := img.XSize;
	SbHoriz.Position := img.HZoomPos;
	SbVert.Max := img.YSize;
	SbVert.Position := img.VZoomPos;

	// Resize controls
	NewWidth.Value := img.XSize;
	NewHeight.Value := img.YSize;

	// Refreshing information bar (status bar)
	InfoRefresh;
end;

procedure TFSample2.InfoRefresh;
begin
  // Refreshing information on status bar.
  ImageInfo.SimpleText :=
	'Image size: ' + IntToStr(img.XSize) + ' x ' + IntToStr(img.YSize) + ' pixels, ' +
	'Resolution: ' + IntToStr(img.HRes) + ' x ' + IntToStr(img.VRes) + ' pix/m, ' +
	'Line size: '  + IntToStr(img.SLine) + ' bytes ';
end;

procedure TFSample2.ZoomChange(Sender: TObject);
begin
  // Zoom rate
  //
  // Control value varies between 0 and 20, 1:1 zoom rate corresponds to Zoom.Position = 10
  // The zoom level is changed as a power of 2, with 0.5 stepping:
  // ...  2^(-1)  2^(-0.5)  2^0  2^0.5  2^1  ...

  img.Zoom := Power(2, (Zoom.Position - 10)/2);
  LZoom.Caption := 'Zoom: ' + FloatToStrF(img.Zoom, ffFixed, 2, 2);
end;

procedure TFSample2.SbVertChange(Sender: TObject);
begin
  img.VZoomPos := SbVert.Position;
  VZoomPos.Value := img.VZoomPos;
end;

procedure TFSample2.SbHorizChange(Sender: TObject);
begin
  img.HZoomPos := SbHoriz.Position;
  HZoomPos.Value := img.HZoomPos;
end;

procedure TFSample2.HZoomPosChange(Sender: TObject);
begin
  img.HZoomPos := HZoomPos.Value;
  SbHoriz.Position := HZoomPos.Value;
end;

procedure TFSample2.VZoomPosChange(Sender: TObject);
begin
  img.VZoomPos := VZoomPos.Value;
  SbVert.Position := VZoomPos.Value;
end;

procedure TFSample2.ShowError;
var
  errcode: integer;
  errmsg: WideString;
begin
  // Getting error code and message
  img.GetError(errcode, errmsg);
  MessageDlg('Error No.: ' + IntToStr(errcode) + #13 + 'Error message: ' + errmsg, mtError, [mbOK], 0);
end;

procedure TFSample2.ResizeImageClick(Sender: TObject);
begin
  // Resizing the image without changing the image format.
  //
  // The 0 value for the SLine parameter means that we let the GX system
  // calculate the proper value for it.
  img.ConvertImage(img.Format, NewWidth.Value, NewHeight.Value, 0);

  // Zoom View Mode
  HZoomPos.MaxValue := img.XSize;
  VZoomPos.MaxValue := img.YSize;
  HZoomPos.Value := img.HZoomPos;
  VZoomPos.Value := img.VZoomPos;
  Zoom.Position := Round(Log2(img.Zoom))*2 + 10;
  LZoom.Caption := 'Zoom: ' + FloatToStrF(img.Zoom, ffFixed, 2, 2);

  // Scrollbars init
  SbHoriz.Max := img.XSize;
  SbHoriz.Position := img.HZoomPos;
  SbVert.Max := img.YSize;
  SbVert.Position := img.VZoomPos;

  InfoRefresh;
end;

end.
