//------------------------------------------------------------------------------
// GX Sample 3 program
//------------------------------------------------------------------------------
// It is a sample program for the GxImage OCX ActiveX control.
// It shows the connection and zoom functionalities and shows the usage of the ScreenPosToImagePos function through an
// OnMouseMove event handling example (try moving the mouse over the source image when the images are connected).

unit Unit3;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OleCtrls, GXIMAGE_OCXLib_TLB, Menus, StdCtrls, ComCtrls, ExtDlgs, Math,
  Spin, ExtCtrls, Buttons, AxCtrls;

type
  TFSample3 = class(TForm)
    ImageInfo: TStatusBar;
    GroupBox1: TGroupBox;
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    LoadImage: TMenuItem;
    Quit: TMenuItem;
    HelpMenu: TMenuItem;
    About: TMenuItem;
    SourceImage: TGxImageOCX;
    OpenPictureDialog: TOpenPictureDialog;
    GroupBox2: TGroupBox;
    TargetImage: TGxImageOCX;
    SourceView: TGroupBox;
    HMirror: TCheckBox;
    VMirror: TCheckBox;
    TargetView: TGroupBox;
    LZoom: TLabel;
    LImgSrcParam: TLabel;
    Zoom: TTrackBar;
    ImgSrcParam: TComboBox;
    AutoUpdate: TCheckBox;
    UpdateImage: TButton;
    PasteImage: TButton;
    CopyImageToClipboard: TButton;
    CopyViewToClipboard: TButton;
    N1: TMenuItem;
    N2: TMenuItem;
    SaveSourceView: TMenuItem;
    SaveTargetImage: TMenuItem;
    SavePictureDialog: TSavePictureDialog;
    SaveSourceImage: TMenuItem;
    SaveTargetView: TMenuItem;
    N3: TMenuItem;
    SourceClear: TPanel;
    TargetClear: TPanel;
    SbVert: TScrollBar;
    SbHoriz: TScrollBar;
    PBitmap: TPanel;
    Image: TImage;
    CopyHBitmap: TButton;
    ConnectImages: TButton;
    CopyTImage: TButton;
    LoadImageBox: TMenuItem;
    procedure AboutClick(Sender: TObject);
    procedure QuitClick(Sender: TObject);
    procedure LoadImageClick(Sender: TObject);
    procedure ZoomChange(Sender: TObject);
    procedure ConnectImagesClick(Sender: TObject);
    procedure UpdateImageClick(Sender: TObject);
    procedure AutoUpdateClick(Sender: TObject);
    procedure SourceImageMouseMove(Sender: TObject; Button,
      Shift: Smallint; x, y: Integer);
    procedure ImgSrcParamChange(Sender: TObject);
    procedure HMirrorClick(Sender: TObject);
    procedure VMirrorClick(Sender: TObject);
    procedure CopyImageToClipboardClick(Sender: TObject);
    procedure CopyViewToClipboardClick(Sender: TObject);
    procedure PasteImageClick(Sender: TObject);
    procedure SaveSourceViewClick(Sender: TObject);
    procedure SaveTargetImageClick(Sender: TObject);
    procedure SaveSourceImageClick(Sender: TObject);
    procedure SaveTargetViewClick(Sender: TObject);
    procedure SourceClearClick(Sender: TObject);
    procedure TargetClearClick(Sender: TObject);
    procedure TargetImageDblClick(Sender: TObject);
    procedure SbHorizChange(Sender: TObject);
    procedure SbVertChange(Sender: TObject);
    procedure CopyHBitmapClick(Sender: TObject);
    procedure CopyTImageClick(Sender: TObject);
    procedure LoadImageBoxClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ShowError(const img: TGxImageOCX);
    procedure InitControls;
    procedure UpdateTargetControls;
    procedure InfoRefresh(const img: TGxImageOCX);
  end;

var
  FSample3: TFSample3;

implementation

{$R *.DFM}

procedure TFSample3.AboutClick(Sender: TObject);
begin
  // About dialog box
  MessageDlg(
    'It is a sample program for the GxImage OCX ActiveX control.' + #13 +
    '' + #13 +
    'It shows the connection and zoom functionalities and' + #13 +
    'shows the usage of the ScreenPosToImagePos function through an' + #13 +
    'OnMouseMove event handling example (try moving the mouse over the' + #13 +
    'source image when the images are connected).' + #13 +
    '', mtInformation, [mbOk], 0);
end;

procedure TFSample3.QuitClick(Sender: TObject);
begin
  FSample3.Close;
end;

procedure TFSample3.LoadImageClick(Sender: TObject);
begin
  // Loaging the image
  if OpenPictureDialog.Execute then begin
    // The Load Image function returns non 0 (true) value on success, 0 on error.
    if (SourceImage.LoadImage(OpenPictureDialog.FileName) = 0) then begin
      // Loading failed...
      ShowError(SourceImage);
      ImageInfo.SimpleText := 'Loading image ' + OpenPictureDialog.FileName + ' failed!';
    end else begin
      // Image loading succeeded.
      InitControls;
    end;
  end;
end;

procedure TFSample3.ShowError(const img: TGxImageOCX);
var
  errcode: integer;
  errmsg: WideString;
begin
  // Getting error code and message
  img.GetError(errcode, errmsg);
  MessageDlg('Error No.: ' + IntToStr(errcode) + #13 + 'Error message: ' + errmsg, mtError, [mbOK], 0);
end;

procedure TFSample3.InitControls;
begin
  // Setting defaults
  TargetImage.Zoom := 4;
  // Source View
  HMirror.Checked := false;
  VMirror.Checked := false;

  // Target View
  TargetView.Enabled := true;
  Zoom.Position := Round(Log2(TargetImage.Zoom))*2 + 10;
  LZoom.Caption := 'Zoom: ' + FloatToStrF(TargetImage.Zoom, ffFixed, 2, 2);
end;

procedure TFSample3.InfoRefresh(const img: TGxImageOCX);
begin
  // Refreshing information on status bar.
  ImageInfo.SimpleText :=
    'Image size: ' + IntToStr(img.XSize) + ' x ' + IntToStr(img.YSize) + ' pixels, ' +
    'Resolution: ' + IntToStr(img.HRes) + ' x ' + IntToStr(img.VRes) + ' pix/m, ' +
    'Line size: '  + IntToStr(img.SLine) + ' bytes ' +
    'Rotation: '   + IntToStr(img.Rotation) + ' ';
end;

procedure TFSample3.ZoomChange(Sender: TObject);
begin
  // Zoom rate
  //
  // Control value varies between 0 and 20, 1:1 zoom rate corresponds to Zoom.Position = 10
  // The zoom level is changed as a power of 2, with 0.5 stepping:
  // ...  2^(-1)  2^(-0.5)  2^0  2^0.5  2^1  ...

  TargetImage.Zoom := Power(2, (Zoom.Position - 10)/2);
  LZoom.Caption := 'Zoom: ' + FloatToStrF(TargetImage.Zoom, ffFixed, 2, 2);
end;

procedure TFSample3.ConnectImagesClick(Sender: TObject);
begin
  // Conneccting to the image source
  if (TargetImage.Connect(SourceImage.GetImageSource, SourceImage.GetSourceHandle) = 0) then begin
    ShowError(TargetImage);
  end;
  UpdateTargetControls;
  ConnectImages.Enabled:= false;
end;

procedure TFSample3.UpdateImageClick(Sender: TObject);
begin
  // Updating image on demand (restoring connection, see documentation)
  TargetImage.UpdateImage;
  // The update does not refresh automatically
  //TargetImage.Refresh;
end;

procedure TFSample3.AutoUpdateClick(Sender: TObject);
begin
  TargetImage.AutoUpdateImage := AutoUpdate.Checked;
end;

procedure TFSample3.SourceImageMouseMove(Sender: TObject; Button,
  Shift: Smallint; x, y: Integer);
var
  XPos, YPos : integer;
  r : integer;
begin
  if not ConnectImages.Enabled then begin
    if (TargetImage.ImgSrcParam = 'View') then
       r := SourceImage.ScreenPosToViewPos(x, y, XPos, YPos)
    else r := SourceImage.ScreenPosToImagePos(x, y, XPos, YPos);
    if not (r = 0) then begin
      TargetImage.HZoomPos := XPos;
      SbHoriz.Position := XPos;
      TargetImage.VZoomPos := YPos;
      SbVert.Position := YPos;
    end;
  end;
end;

procedure TFSample3.ImgSrcParamChange(Sender: TObject);
begin
  TargetImage.ImgSrcParam := ImgSrcParam.Text;
end;

procedure TFSample3.HMirrorClick(Sender: TObject);
begin
  SourceImage.Mirror := SourceImage.Mirror xor GX_MIRROR_HORIZONTAL;
end;

procedure TFSample3.VMirrorClick(Sender: TObject);
begin
  SourceImage.Mirror := SourceImage.Mirror xor GX_MIRROR_VERTICAL;
end;

procedure TFSample3.CopyImageToClipboardClick(Sender: TObject);
begin
  if (SourceImage.CopyImage = 0) then begin
    ShowError(SourceImage);
  end else begin
    PasteImage.Enabled := true;
  end;
end;

procedure TFSample3.CopyViewToClipboardClick(Sender: TObject);
begin
  if (SourceImage.CopyView = 0) then begin
    ShowError(SourceImage);
  end else begin
    PasteImage.Enabled := true;
  end;
end;

procedure TFSample3.PasteImageClick(Sender: TObject);
begin
  TargetImage.PasteImage;
  UpdateTargetControls;
end;

procedure TFSample3.SaveSourceImageClick(Sender: TObject);
begin
  if (SavePictureDialog.Execute) then begin
    // Saving the original image in the selected format
    if (SourceImage.SaveImage(SavePictureDialog.FileName, SavePictureDialog.FilterIndex) = 0) then begin
      // Saving image failed...
      ShowError(SourceImage);
    end;
  end;
end;

procedure TFSample3.SaveSourceViewClick(Sender: TObject);
begin
  if (SavePictureDialog.Execute) then begin
    // Saving the original image in the selected format
    if (SourceImage.SaveView(SavePictureDialog.FileName, SavePictureDialog.FilterIndex) = 0) then begin
      // Saving image failed...
      ShowError(SourceImage);
    end;
  end;
end;

procedure TFSample3.SaveTargetImageClick(Sender: TObject);
begin
  if (SavePictureDialog.Execute) then begin
    // Saving the original image in the selected format
    if (TargetImage.SaveImage(SavePictureDialog.FileName, SavePictureDialog.FilterIndex) = 0) then begin
      // Saving image failed...
      ShowError(TargetImage);
    end;
  end;
end;

procedure TFSample3.SaveTargetViewClick(Sender: TObject);
begin
  if (SavePictureDialog.Execute) then begin
    // Saving the original image in the selected format
    if (TargetImage.SaveView(SavePictureDialog.FileName, SavePictureDialog.FilterIndex) = 0) then begin
      // Saving image failed...
      ShowError(TargetImage);
    end;
  end;
end;

procedure TFSample3.SourceClearClick(Sender: TObject);
begin
  SourceImage.ClearImage;
end;

procedure TFSample3.TargetClearClick(Sender: TObject);
begin
  TargetImage.ClearImage;
end;

procedure TFSample3.TargetImageDblClick(Sender: TObject);
  function BoolToStr(value: boolean): string;
  begin
    if (value) then BoolToStr := 'True' else BoolToStr := 'False';
  end;
  function ColorToStr(value: TColor): string;
  begin
    ColorToStr := '#' + IntToHex(value,6);
  end;
begin
  MessageDlg(
    'AutoRefresh: ' + BoolToStr((Sender as TGxImageOCX).AutoRefresh) + #13 +
    'AutoSize: ' + BoolToStr((Sender as TGxImageOCX).AutoSize) + #13 +
    'AutoUpdateImage: ' + BoolToStr((Sender as TGxImageOCX).AutoUpdateImage) + #13 +
    'BackColor: ' + ColorToStr((Sender as TGxImageOCX).BackColor) + #13 +
    'Format: ' + IntToStr((Sender as TGxImageOCX).Format) + #13 +
    'HasWindow: ' + BoolToStr((Sender as TGxImageOCX).HasWindow) + #13 +
    'HRes: ' + IntToStr((Sender as TGxImageOCX).HRes) + #13 +
    'HZoomPos: ' + IntToStr((Sender as TGxImageOCX).HZoomPos) + #13 +
    'ImagePtr: ' + IntToStr((Sender as TGxImageOCX).ImagePtr) + #13 +
    'ImgSrcParam: ' + ((Sender as TGxImageOCX).ImgSrcParam) + #13 +
    'Mirror: ' + IntToStr((Sender as TGxImageOCX).Mirror) + #13 +
    'Rotation: ' + IntToStr((Sender as TGxImageOCX).Rotation) + #13 +
    'SaveQuality: ' + IntToStr((Sender as TGxImageOCX).SaveQuality) + #13 +
    'SLine: ' + IntToStr((Sender as TGxImageOCX).SLine) + #13 +
    'StretchImage: ' + BoolToStr((Sender as TGxImageOCX).StretchImage) + #13 +
    'StretchMode: ' + IntToStr((Sender as TGxImageOCX).StretchMode) + #13 +
    'ViewMode: ' + IntToStr((Sender as TGxImageOCX).ViewMode) + #13 +
    'ViewPtr: ' + IntToStr((Sender as TGxImageOCX).ViewPtr) + #13 +
    'ViewSLine: ' + IntToStr((Sender as TGxImageOCX).ViewSLine) + #13 +
    'ViewXSize: ' + IntToStr((Sender as TGxImageOCX).ViewXSize) + #13 +
    'ViewYSize: ' + IntToStr((Sender as TGxImageOCX).ViewYSize) + #13 +
    'VRes: ' + IntToStr((Sender as TGxImageOCX).VRes) + #13 +
    'VZoomPos: ' + IntToStr((Sender as TGxImageOCX).VZoomPos) + #13 +
    'WndBottom: ' + IntToStr((Sender as TGxImageOCX).WndBottom) + #13 +
    'WndLeft: ' + IntToStr((Sender as TGxImageOCX).WndLeft) + #13 +
    'WndRight: ' + IntToStr((Sender as TGxImageOCX).WndRight) + #13 +
    'WndTop: ' + IntToStr((Sender as TGxImageOCX).WndTop) + #13 +
    'XSize: ' + IntToStr((Sender as TGxImageOCX).XSize) + #13 +
    'YSize: ' + IntToStr((Sender as TGxImageOCX).YSize) + #13 +
    'Zoom: ' + FloatToStrF((Sender as TGxImageOCX).Zoom, ffFixed, 2, 2) + #13 +
    //': ' + ((Sender as TGxImageOCX).) + #13 +
    '', mtInformation, [mbOk], 0);
end;

procedure TFSample3.SbHorizChange(Sender: TObject);
begin
  TargetImage.HZoomPos := SbHoriz.Position;
end;

procedure TFSample3.SbVertChange(Sender: TObject);
begin
  TargetImage.VZoomPos := SbVert.Position;
end;

procedure TFSample3.UpdateTargetControls;
begin
  SbHoriz.Max:= TargetImage.XSize;
  SbHoriz.Position := TargetImage.HZoomPos;
  SbVert.Max:= TargetImage.YSize;
  SbVert.Position := TargetImage.VZoomPos;
  AutoUpdate.Checked := TargetImage.AutoUpdateImage;
  ImgSrcParam.Text := TargetImage.ImgSrcParam;
  Zoom.Position := Round(Log2(TargetImage.Zoom))*2 + 10;
  LZoom.Caption := 'Zoom: ' + FloatToStrF(TargetImage.Zoom, ffFixed, 2, 2);
end;

procedure TFSample3.CopyHBitmapClick(Sender: TObject);
begin
  // Copying image as a HBitmap
  Image.Picture.Bitmap.Handle := SourceImage.ImageBitmap;
end;


procedure TFSample3.CopyTImageClick(Sender: TObject);
begin
  // Copying TImage to the GxImage control
  SourceImage.ImageBitmap := Image.Picture.Bitmap.Handle;
end;

procedure TFSample3.LoadImageBoxClick(Sender: TObject);
begin
  // Loaging the image
  if OpenPictureDialog.Execute then begin
    // The Load Image function returns non 0 (true) value on success, 0 on error.
    Image.Picture.LoadFromFile(OpenPictureDialog.FileName);
    // Image loading succeeded.
    InitControls;
  end;
end;

end.
