unit fpstypes;
//------------------------------------------------------------------------------
// FPS Types file - V2.1.10
//
// 2010-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
const

// FPS_USAGEMODE - Device connection modes
	FPS_UMODE_DISCONNECT		= 0;
	FPS_UMODE_NORMAL_USE		= 1;
	FPS_UMODE_FULL_CONTROL	= 2;

// FPS_FREERUNMODE - Device freerun light controlling modes
	FPS_FRMODE_NULL			= $0000;
	FPS_FRMODE_PREVIEWLIGHT	= $0001;
	FPS_FRMODE_TESTPRESENCE	= $0004;
	FPS_FRMODE_HQPREVIEW	= $0010;
	FPS_FRMODE_ROLLPREVIEW	= $0020;

// FPS_TESTPRESENCE - finger presence detection modes
	FPS_TP_ABSENT	= 0;
	FPS_TP_DEFAULT	= 2;
	FPS_TP_DIRTY	= 3;
	FPS_TP_PRESENT	= 4;

// FPS_IMAGE_TYPE - Image types
	FPS_IT_PREVIEW			= 2;
	FPS_IT_FINGER			= 3;

// FPS_EVENT - Event types
	FPS_ET_CAPTURE			= 2;

	FPS_EV_CAPTUREFINISHED	= 101;
	FPS_EV_IMAGECAPTURED		= 102;
	FPS_EV_FGCAP				= 106;
	FPS_EV_PREVIEWCAPTURED	= 107;

// FPS_POWER_STATE - Power state codes
	FPS_PWR_MASK				= $000f;

	FPS_PWR_UNKNOWN			= $0000;
	FPS_PWR_OFF				= $0001;
	FPS_PWR_ON				= $0002;

	FPS_BATTERY_MASK			= $00f0;

	FPS_BATTERY_UNAVAILABLE	= $0000;
	FPS_BATTERY_AVAILABLE	= $0010;
	FPS_BATTERY_UNKNOWN		= $0020;

// FPS_STATUS_LED_COLOR - Status led color codes
	FPS_SLC_OFF		= $00;
	FPS_SLC_GREEN	= $81;
	FPS_SLC_RED		= $82;
	FPS_SLC_ORANGE	= $83;
	FPS_SLC_BLUE		= $84;
	FPS_SLC_AUTO	= $ac;
	FPS_SLC_ANY		= $ff;

// FPS_BUZZER - Id of the buzzer
	FPS_SLC_BUZZER	= $c0;

// FPS_KEY_STATE - Button state codes
	FPS_KEY_UNKNOWN		= 0;
	FPS_KEY_PRESSED		= 1;
	FPS_KEY_RELEASED		= 2;
	FPS_KEY_NOKEY		= 3;

// FPS_KEY_CODE - Button codes
	FPS_KEY_POWER	= $10101;
	FPS_KEY_START	= $10110;

// FPS_VAR_ID - Variant IDs used

	FPSV_ROOT			= 0;

	FPSV_PAGELIST		= FPSV_ROOT or 1;

	FPSV_CONFIDENCE		= $D0000005;

	FPSV_IMAGE			= $D0100000;
	FPSV_IMAGELIST		= FPSV_IMAGE or 1;
	FPSV_IMAGEFRAME		= FPSV_IMAGE or 2;

	FPSV_DEVICE_INFO		= $D0200000;
	FPSV_DEVICE_ID		= FPSV_DEVICE_INFO or 1;
	FPSV_CAPTIME			= FPSV_DEVICE_INFO or 3;

	FPSV_COMMON			= $D0700000;
	FPSV_PROCTIME		= FPSV_COMMON or 1;
	FPSV_PROCPERIOD		= FPSV_COMMON or 2;
	FPSV_WARNLIST		= FPSV_COMMON or 3;
	FPSV_TASK			= FPSV_COMMON or 4;

	FPSV_FINGER			= $D0800000;
	FPSV_FINGER_TYPE		= FPSV_FINGER or 1;
	FPSV_NIST_QUALITY		= FPSV_FINGER or 2;
	FPSV_DRYWET			= FPSV_FINGER or 3;
	FPSV_FRACTION		= FPSV_FINGER or 4;

// FPS_POSITION - Finger storage IDs.
	FPS_POS_UNKNOWN		= 0;
	FPS_POS_RIGHT_THUMB	= 1;
	FPS_POS_RIGHT_INDEX	= 2;
	FPS_POS_RIGHT_MIDDLE	= 3;
	FPS_POS_RIGHT_RING	= 4;
	FPS_POS_RIGHT_LITTLE	= 5;
	FPS_POS_LEFT_THUMB	= 6;
	FPS_POS_LEFT_INDEX	= 7;
	FPS_POS_LEFT_MIDDLE	= 8;
	FPS_POS_LEFT_RING		= 9;
	FPS_POS_LEFT_LITTLE	= 10;

	FPS_POS_PLAIN_RIGHT_THUMB		= 11;
	FPS_POS_PLAIN_LEFT_THUMB		= 12;
	FPS_POS_PLAIN_RIGHT_4FINGERS	= 13;
	FPS_POS_PLAIN_LEFT_4FINGERS	= 14;
	FPS_POS_PLAIN_THUMBS			= 15;

	FPS_POS_BOTH_INDEXES			= 46;

//FPS_SCAN_OBJECT - The target object of fingerprint scanning.
	FPS_SCAN_LEFT_HAND	= 1;
	FPS_SCAN_RIGHT_HAND	= 2;
	FPS_SCAN_SAME_FINGER	= 3;

//FPS_IMPRESSION_TYPE - The capturing mode of the finger capture process.
	FPS_SCAN_LIVE			= 0;
	FPS_SCAN_ROLLED		= 1;
	FPS_SCAN_ACCUMULATED	= $1000;

//FPS_PRESENCE - Finger existence IDs.
	FPS_NO_INFO		= 0;
	FPS_AMPUTATION	= 1;
	FPS_NO_PRINT		= 2;
	FPS_AVAILABLE		= 3;

// FPS_FAILURE - Warnings about finger capture.
	FPS_FAIL_LEFT		= 1;
	FPS_FAIL_RIGHT	= 2;
	FPS_FAIL_TOP		= 4;
	FPS_FAIL_BOTTOM	= 8;
	FPS_FAIL_MISSING	= $10;
	FPS_FAIL_EXTRA	= $20;
	FPS_FAIL_HAND		= $40;
	FPS_FAIL_UPRIGHT	= $80;

//------------------------------------------------------------------------------
implementation
end.
