function sleep(milliseconds) {
  let start = new Date().getTime();
  for (let i = 0; i < 1e7; i++) {
    if (new Date().getTime() - start > milliseconds) {
      break;
    }
  }
}

function JSON_RPC(host) {
  this.rpc_id_counter = 0;
  this.rpc_mgr = null;
  this.rpc_calls = [];

  if (!host.startsWith("https://") && !host.startsWith("http://")) {
    console.error("Unhandled protocol");
    return;
  }
  this.ws = new WebSocket('ws' + host.substring(4));

  let rpc_this = this;

  // --- external callback functions ---
  this.ext_callbacks = {};

  this.CallExt = function(name, param) {
    if (!("ext_callbacks" in this)) return;
    if (this.ext_callbacks == null) return;
    if (!(name in this.ext_callbacks)) return;
    let func = this.ext_callbacks[name];
    if (func == null) return;
    return func(param);
  };
  // -----------------------------------
  this.close = function() {
    this.ws.close();
  };

  this.ws.onopen = function(ev) {};
  this.ws.onclose = function(ev) {};
  this.ws.onmessage = function(ev) {
    rpc_this.handle_json_rpc_answer(ev);
  };
  this.ws.onerror = function(ev) {};

  this.send_json_rpc_request = function(
    rpc_method_name,
    rpc_method_params,
    rpc_callback,
    waitbinary
  ) {
    if (!Array.isArray(rpc_callback)) rpc_callback = [rpc_callback, null];
    else if (rpc_callback.length == 1)
      rpc_callback = [rpc_callback[0], rpc_callback[0]];

    let obj = new Object();
    obj.m_name = rpc_method_name;
    obj.id = this.rpc_id_counter;
    obj.params = null;
    obj.callback = null;
    obj.callback_err = null;
    obj.waitbinary = 0;
    if (typeof waitbinary != "undefined") {
      obj.waitbinary = 1;
    }
    if (typeof rpc_callback[1] != "undefined")
      obj.callback_err = rpc_callback[1];
    if (typeof rpc_callback[0] != "undefined") obj.callback = rpc_callback[0];
    if (typeof rpc_method_params != "undefined")
      obj.params = JSON.stringify(rpc_method_params);
    this.rpc_calls.push(obj);
    let request = '{"jsonrpc": "2.0", "method": "' + rpc_method_name + '"';
    if (obj.params != null) {
      request += ', "params": ';
      request += obj.params;
    }
    request += ' ,"id":"';
    request += this.rpc_id_counter + '"}';
    this.ws.send(request);
    this.rpc_id_counter++;
  };

  this.handle_json_rpc_answer = function(ev) {
    if (ev.data instanceof Blob) {
      let found = false;
      let i;
      for (i = 0; i < this.rpc_calls.length; i++) {
        if (this.rpc_calls[i].waitbinary == 1) {
          found = true;
          break;
        }
      }
      if (found) {
        if (this.rpc_calls[i].callback)
          this.rpc_calls[i].callback(ev.data);
      }
      return;
    }
    let cb_data = JSON.parse(ev.data);
    if ("id" in cb_data && cb_data.id != null /*&& cb_data.id != ""*/) {
      let found = false;

      // find matching request:
      for (let i = 0; i < this.rpc_calls.length; i++) {
        if (this.rpc_calls[i].id == cb_data.id) {
          found = true;
          cb_data.m_name = this.rpc_calls[i].m_name;
          cb_data.callback = this.rpc_calls[i].callback;
          cb_data.callback_err = this.rpc_calls[i].callback_err;
          this.rpc_calls.splice(i, 1).m_name;
          break;
        }
      }
      if (!found) {
        console.error("error: unknown id for rpc answer, id:" + cb_data.id);
      }
      else if ("error" in cb_data) {
        console.error(ev);
        if (cb_data.callback_err) cb_data.callback_err(cb_data);
      } else {
        if (cb_data.callback) cb_data.callback(cb_data);
      }
    } else {
      if ("method" in cb_data) rpc_this.CallExt(cb_data.method, cb_data.params);
    }
  };
}

/**************************************************************************
 *
 *
 *
 * Passport Reader functions
 *
 *
 *
 *************************************************************************/

let Pr22 = (function() {
  let rpc_handle = null;
  let proxyhost = null;
  let device;

  return {
    setProxy(_host) {
      proxyhost = _host;
    },

    listServers: function(responseHandler) {
      if (!proxyhost) return responseHandler([]);
      const devlistUrl = "http://" + proxyhost + "/devicelist";
      const req = new XMLHttpRequest();
      req.addEventListener("load", function(response) {
        console.log("response" + response);
        responseHandler(JSON.parse(req.responseText));
      });
      req.timeout = 2000;
      req.open("GET", devlistUrl);
      req.send();
    },

    connect: function(
      host,
      onConnectFinishedHandler,
      onConnectFailedHandler,
      onConnectionClosedHandler
    ) {
      if (rpc_handle) return;

      if (proxyhost) {
        device = host;

        const req = new XMLHttpRequest();
        req.open("POST", "http://" + proxyhost + "/login/" + device, false);
        try {
          req.send();
        } catch (exception) {
          if (exception.name == "NetworkError") {
            console.error("Can not connect to the device proxy");
            return;
          }
        }
        if (req.status != 200) {
          if (onConnectFailedHandler) {
            onConnectFailedHandler();
            return;
          }
        }

        sleep(8000);

        host = "http://" + proxyhost + "/" + device;
      }

      rpc_handle = new JSON_RPC(host);
      if (onConnectFinishedHandler) {
        rpc_handle.ws.onopen = onConnectFinishedHandler;
      }
      if (onConnectFailedHandler) {
        rpc_handle.ws.onerror = onConnectFailedHandler;
      }
      if (onConnectionClosedHandler) {
        rpc_handle.ws.onclose = onConnectionClosedHandler;
      }
    },

    disconnect: function() {
      if (!rpc_handle) return;
      let rpc_tmp = rpc_handle;
      rpc_handle = null;
      rpc_tmp.close();
      if (proxyhost) {
        const req = new XMLHttpRequest();
        req.open("POST", "http://" + proxyhost + "/logout/" + device, false);
        try {
          req.send();
        } catch (exception) {
          if (exception.name == "NetworkError") {
            console.error("Can not connect to the device proxy");
          }
        }
      }
    },

    DocumentReader: function() {
      return {
        useDevice: function(deviceName, responseHandler, mode) {
          let params = {};
          if (typeof deviceName == "string") params.Device = deviceName;
          if (typeof deviceName == "number") params.Index = deviceName;
          params.Mode = mode;

          rpc_handle.send_json_rpc_request(
            "UseDevice",
            params,
            responseHandler
          );
        },

        closeDevice: function(responseHandler) {
          rpc_handle.send_json_rpc_request(
            "CloseDevice",
            null,
            responseHandler
          );
        },

        setProperty: function(prop, value, responseHandler) {
          let params = [prop, value];
          rpc_handle.send_json_rpc_request(
            "SetProperty",
            params,
            responseHandler
          );
        },

        getProperty: function(prop, responseHandler) {
          rpc_handle.send_json_rpc_request(
            "GetProperty",
            prop,
            responseHandler
          );
        },

        setEventHandler: function(eventType, eventHandler, responseHandler) {
          rpc_handle.ext_callbacks[eventType] = eventHandler;
          let params = {
            EventType: eventType,
          };
          rpc_handle.send_json_rpc_request("SetEvent", params, responseHandler);
        },

        clearEventHandler: function(eventType, eventHandler, responseHandler) {
          rpc_handle.ext_callbacks[eventType] = eventHandler;
          let params = {
            EventType: eventType,
          };
          rpc_handle.send_json_rpc_request(
            "ClearEvent",
            params,
            responseHandler
          );
        },

        triggerEvent: function(eventType, responseHandler) {
          let params = {
            EventType: eventType,
          };
          rpc_handle.send_json_rpc_request(
            "TriggerEvent",
            params,
            responseHandler
          );
        },

        getScanner: function() {
          return {
            getInfo: function(responseHandler) {
              rpc_handle.send_json_rpc_request(
                "GetScannerInfo",
                null,
                responseHandler
              );
            },

            scan: function(task, responseHandler, pagePosition) {
              let params = {};
              if (typeof task == "string") params.Lights = [task];
              else if (Array.isArray(task)) params.Lights = task;
              else {
                params.Lights = task.get();
                params.Window = task.objectWindow;
              }
              params.PagePos = pagePosition;

              rpc_handle.send_json_rpc_request("Scan", params, responseHandler);
            },

            cleanUpData: function(responseHandler) {
              let params = {
                LastPage: false,
              };
              rpc_handle.send_json_rpc_request(
                "CleanUp",
                params,
                responseHandler
              );
            },

            cleanUpLastPage: function(responseHandler) {
              let params = {
                LastPage: true,
              };
              rpc_handle.send_json_rpc_request(
                "CleanUp",
                params,
                responseHandler
              );
            },

            getImage: function(selection, responseHandler, options) {
              let params = {};
              if (Array.isArray(selection)) {
                for (let it in selection) {
                  if (typeof selection[it] == "number")
                    params.Page = selection[it];
                  else if (selection[it] == "DocView") params.DocView = 1;
                  else if (selection[it] == "PreferDocView") params.DocView = 2;
                  else params.Light = selection[it];
                }
              } else params.Light = selection;
              let base64;
              if (typeof options != "undefined") {
                if (options.Base64) base64 = options.Base64;
                for (let val in options) params[val] = options[val];
              }
              rpc_handle.send_json_rpc_request(
                "GetImage",
                params,
                responseHandler,
                !base64
              );
            },

            getImageInfo: function(selection, responseHandler) {
              let params = {};
              if (Array.isArray(selection)) {
                for (let it in selection) {
                  if (typeof selection[it] == "number")
                    params.Page = selection[it];
                  else if (selection[it] == "DocView") params.DocView = 1;
                  else if (selection[it] == "PreferDocView") params.DocView = 2;
                  else params.Light = selection[it];
                }
              } else params.Light = selection;

              rpc_handle.send_json_rpc_request(
                "GetImageInfo",
                params,
                responseHandler
              );
            },

            startTask: function(task, responseHandler) {
              let params = {};
              if (typeof task == "string") params.Task = task;
              else params.Task = task.get();

              rpc_handle.send_json_rpc_request(
                "StartTask",
                params,
                responseHandler
              );
            },

            stopTask: function(task, responseHandler) {
              let params = {};
              if (typeof task == "string") params.Task = task;
              else params.Task = task.get();

              rpc_handle.send_json_rpc_request(
                "StopTask",
                params,
                responseHandler
              );
            },
          };
        },

        getEngine: function() {
          return {
            getInfo: function(responseHandler) {
              rpc_handle.send_json_rpc_request(
                "GetEngineInfo",
                null,
                responseHandler
              );
            },

            listLicenses: function(responseHandler, options) {
              rpc_handle.send_json_rpc_request(
                "ListLicenses",
                options,
                responseHandler
              );
            },

            listEngines: function(responseHandler) {
              rpc_handle.send_json_rpc_request(
                "ListEngines",
                null,
                responseHandler
              );
            },

            analyze: function(page, task, responseHandler, imgFilter) {
              let params = {
                Page: page,
                Task: task.get(),
              };
              if (typeof imgFilter == "string") params.Lights = [imgFilter];
              else if (Array.isArray(imgFilter)) params.Lights = imgFilter;
              else if (imgFilter != null) params.Lights = imgFilter.get();

              rpc_handle.send_json_rpc_request(
                "Analyze",
                params,
                responseHandler
              );
            },

            merge: function(one, other, responseHandler) {
              let params = {
                One: one,
                Other: other,
              };

              rpc_handle.send_json_rpc_request(
                "MergeDocuments",
                params,
                responseHandler
              );
            },

            getRootDocument: function(format, responseHandler, options) {
              let params = {
                Format: format,
              };
              let base64;
              if (typeof options != "undefined") {
                if (options.Base64) base64 = options.Base64;
                for (let val in options) params[val] = options[val];
              }

              rpc_handle.send_json_rpc_request(
                "GetRootDocument",
                params,
                responseHandler,
                !base64
              );
            },

            getFieldImage: function(doc, field, responseHandler, options) {
              let params = {
                Doc: doc,
                Field: field,
              };
              let base64;
              if (typeof options != "undefined") {
                if (options.Base64) base64 = options.Base64;
                for (let val in options) params[val] = options[val];
              }

              rpc_handle.send_json_rpc_request(
                "GetFieldImage",
                params,
                responseHandler,
                !base64
              );
            },
          };
        },

        listCardReaders: function(responseHandler) {
          rpc_handle.send_json_rpc_request(
            "ListCardReaders",
            null,
            responseHandler
          );
        },

        getCardReader: function(index) {
          let rdindex = index;

          return {
            getInfo: function(responseHandler) {
              let params = {
                Reader: rdindex,
              };
              rpc_handle.send_json_rpc_request(
                "GetReaderInfo",
                params,
                responseHandler
              );
            },

            listCards: function(responseHandler) {
              let params = {
                Reader: rdindex,
              };
              rpc_handle.send_json_rpc_request(
                "ListCards",
                params,
                responseHandler
              );
            },

            connectCard: function(cardNo, responseHandler) {
              let params = {
                Reader: rdindex,
                Card: cardNo,
              };
              rpc_handle.send_json_rpc_request(
                "ConnectCard",
                params,
                responseHandler
              );
            },

            getCard: function(connectResponse) {
              let serial = "";

              if (connectResponse.result != undefined)
                serial = connectResponse.result.CardId;

              return {
                getInfo: function(responseHandler) {
                  let params = {
                    Card: serial,
                  };
                  rpc_handle.send_json_rpc_request(
                    "GetCardInfo",
                    params,
                    responseHandler
                  );
                },

                startRead: function(task, responseHandler, options) {
                  let params = {
                    Card: serial,
                  };
                  if (typeof task == "string") params.Files = [task];
                  else if (Array.isArray(task)) params.Files = task;
                  else {
                    params.Files = task.get();
                    params.AuthLevel = task.authLevel;
                  }
                  if (typeof options != "undefined") {
                    for (let val in options) params[val] = options[val];
                  }

                  rpc_handle.send_json_rpc_request(
                    "StartRead",
                    params,
                    responseHandler
                  );
                },

                getNextAuthentication: function(forceNext, responseHandler) {
                  let params = {
                    Card: serial,
                    ForceNext: forceNext,
                  };
                  rpc_handle.send_json_rpc_request(
                    "GetNextAuthentication",
                    params,
                    responseHandler
                  );
                },

                authenticate: function(auth, data, responseHandler) {
                  let params = {
                    Card: serial,
                    Auth: auth,
                  };
                  if (data != null) params.Data = data;
                  rpc_handle.send_json_rpc_request(
                    "Authenticate",
                    params,
                    responseHandler
                  );
                },

                listApplications: function(responseHandler) {
                  let params = {
                    Card: serial,
                  };
                  rpc_handle.send_json_rpc_request(
                    "ListApplications",
                    params,
                    responseHandler
                  );
                },

                listFiles: function(responseHandler) {
                  let params = {
                    Card: serial,
                  };
                  rpc_handle.send_json_rpc_request(
                    "ListFiles",
                    params,
                    responseHandler
                  );
                },

                getFile: function(file, responseHandler, format) {
                  let params = {
                    Card: serial,
                    File: file,
                    Format: format,
                  };
                  let binary;
                  if (format == "Binary") binary = true;

                  rpc_handle.send_json_rpc_request(
                    "GetFile",
                    params,
                    responseHandler,
                    binary
                  );
                },

                checkHash: function(file, responseHandler) {
                  let params = {
                    Card: serial,
                    File: file,
                  };
                  rpc_handle.send_json_rpc_request(
                    "CheckHash",
                    params,
                    responseHandler
                  );
                },

                convertFileId: function(file, responseHandler) {
                  let params = {
                    Card: serial,
                    File: file,
                  };
                  rpc_handle.send_json_rpc_request(
                    "ConvertFileId",
                    params,
                    responseHandler
                  );
                },

                getAuthResult: function(responseHandler) {
                  let params = {
                    Card: serial,
                  };
                  rpc_handle.send_json_rpc_request(
                    "GetAuthResult",
                    params,
                    responseHandler
                  );
                },

                disconnect: function(responseHandler) {
                  let params = {
                    Card: serial,
                  };
                  rpc_handle.send_json_rpc_request(
                    "DisconnectCard",
                    params,
                    responseHandler
                  );
                },
              };
            },
          };
        },
      };
    },

    DocScannerTask: function() {
      let lightlist = new Array();

      return {
        objectWindow: undefined,

        add: function(light) {
          lightlist.push(light);
          return this;
        },

        del: function(light) {
          lightlist.push("-" + light);
          return this;
        },

        get: function() {
          return lightlist;
        },
      };
    },

    EngineTask: function() {
      let reflist = new Array();

      return {
        add: function(field) {
          reflist.push(field);
          return this;
        },

        del: function(field) {
          reflist.push("-" + field);
          return this;
        },

        get: function() {
          return reflist;
        },
      };
    },

    FreerunTask: function() {
      let task;

      return {
        Detection: function() {
          task = "Detection";
          return this;
        },

        get: function() {
          return task;
        },
      };
    },

    ECardTask: function() {
      let filelist = new Array();

      return {
        authLevel: undefined,

        add: function(file) {
          filelist.push(file);
          return this;
        },

        del: function(file) {
          filelist.push("-" + file);
          return this;
        },

        get: function() {
          return filelist;
        },
      };
    },

    listDevices: function(responseHandler) {
      rpc_handle.send_json_rpc_request("ListDevices", null, responseHandler);
    },

    getVersion: function(responseHandler) {
      let params = {
        Interface: "2.2.0.21",
      };
      rpc_handle.send_json_rpc_request(
        "GetApiVersion",
        params,
        responseHandler
      );
    },

    listMethods: function(responseHandler) {
      rpc_handle.send_json_rpc_request("ListMethods", null, responseHandler);
    },

    listEnumValues: function(enumType, responseHandler) {
      let params = {
        EnumType: enumType,
      };
      rpc_handle.send_json_rpc_request(
        "ListEnumValues",
        params,
        responseHandler
      );
    },

    keepAlive: function(responseHandler) {
      rpc_handle.send_json_rpc_request("KeepAlive", null, responseHandler);
    },

    login: function(user, passw, responseHandler) {
      let params = {
        User: user,
        Password: passw,
      };
      rpc_handle.send_json_rpc_request("Login", params, responseHandler);
    },

    Admin: function() {
      return {
        uploadFile: function(fileName, fileData, responseHandler) {
          let params = {
            FileName: fileName,
            FileData: fileData,
          };
          rpc_handle.send_json_rpc_request(
            "UploadFile",
            params,
            responseHandler
          );
        },

        getUsageInfo: function(responseHandler) {
          rpc_handle.send_json_rpc_request(
            "GetUsageInfo",
            null,
            responseHandler
          );
        },

        forceLogout: function(address, responseHandler) {
          let params = {
            Address: address,
          };
          rpc_handle.send_json_rpc_request(
            "ForceLogout",
            params,
            responseHandler
          );
        },

        userMessage: function(address, message, responseHandler) {
          let params = {
            Address: address,
            Message: message,
          };
          rpc_handle.send_json_rpc_request(
            "UserMessage",
            params,
            responseHandler
          );
        },
      };
    },
  };
})();

export default {
  Pr22,
};
//module.exports = Pr22
