/*******************************************************************************
 * GX sample program
 *
 * Load, save and convert image example.
 ******************************************************************************/

#include "gxsd.h"
#include "gxerror.h"
#include "gximage.h"
#include "gxsdldr.c"

/*******************************************************************************
 * If an error occurred it displays the error and terminates the program
 ******************************************************************************/
void term_if_err(int st) {
	int err_code;
	wchar_t err_str[256];
	err_str[(sizeof(err_str)/sizeof(wchar_t))-1] = 0;

	if(st) return;		/* No error occurred */

	/* Displays error code and description */
	gx_geterror(&err_code, err_str, (sizeof(err_str)/sizeof(wchar_t))-1);
	fprintf(stderr, "GX error occurred: %i %ls\n", err_code, err_str);

	/* Terminates the program */
	exit(1);
}

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main(void) {
	struct gxIMAGE *image;
	int st;
	
	/* Creates a new image handle with default properties */
	struct gxHANDLE handle;
	st = gx_openmodule(&handle, L"gximage", L"default");
	term_if_err(st);
	
	/* Creates a new image structure */
	st = gx_allocimage(handle, &image);
	term_if_err(st);
	
	/* Loads an image */
	st = gx_loadimage(handle, image, L"../../../data/mushroom.jpg", GX_UNDEF);
	term_if_err(st);
	
	/* Converts the image */
	st = gx_convertimage(handle, image, image, GX_GRAY,
				image->xsize, image->ysize, image->sline);
	term_if_err(st);
	
	/* Saves to BMP format */
	st = gx_saveimage(handle, image, L"test.bmp", GX_BMP);
	term_if_err(st);
	
	/* Frees the image structure */
	st = gx_unrefimage(handle, image);
	term_if_err(st);
	
	/* Closes the opened image module */
	st = gx_closehandle(&handle);
	term_if_err(st);
	
	return 0;
}
