/*******************************************************************************
 * GX sample program
 *
 * JPEG quality handling example.
 ******************************************************************************/

#include "gxsd.h"
#include "gxerror.h"
#include "gximage.h"
#include "gxsdldr.c"

/*******************************************************************************
 * If an error occurred it displays the error and terminates the program
 ******************************************************************************/
void term_if_err(int st) {
	int err_code;
	wchar_t err_str[256];
	err_str[(sizeof(err_str)/sizeof(wchar_t))-1] = 0;

	if(st) return;		/* No error occurred */

	/* Displays error code and description */
	gx_geterror(&err_code, err_str, (sizeof(err_str)/sizeof(wchar_t))-1);
	fprintf(stderr, "GX error occurred: %i %ls\n", err_code, err_str);

	/* Terminates the program */
	exit(1);
}

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main(void) {
	struct gxHANDLE handle_jpeg10, handle_jpeg80;
	struct gxIMAGE *image1, *image2;
	int st;
	
	/* Creates an image handler with default properties */
	st = gx_openmodule(&handle_jpeg10, L"gximage", L"default");
	term_if_err(st);

	/* Updates the JPEG quality to 10 */
	st = gx_setmodulepropertyint(handle_jpeg10, L"jpeg/compress/quality", 10);
	term_if_err(st);

	/* Creates an image handler with default properties */
	st = gx_openmodule(&handle_jpeg80, L"gximage", L"default");
	term_if_err(st);

	/* Updates the JPEG quality to 80 */
	st = gx_setmodulepropertyint(handle_jpeg80, L"jpeg/compress/quality", 80);
	term_if_err(st);

	/* Creates an image with the first image handler (where the JPEG quality is 10) */
	st = gx_allocimage(handle_jpeg10, &image1);
	term_if_err(st);

	/* Creates an image with the second image handler (where the JPEG quality is 80) */
	st = gx_allocimage(handle_jpeg80, &image2);
	term_if_err(st);
	
	/* Loads the sample image */
	st = gx_loadimage(handle_jpeg10, image1, L"../../../data/mushroom.jpg", GX_UNDEF);
	term_if_err(st);
	
	/* Saves it to JPEG format (where the quality is 10) */
	st = gx_saveimage(handle_jpeg10, image1, L"test_jpeg10.jpg", GX_JPEG);
	term_if_err(st);

	/* Loads the sample image */
	st = gx_loadimage(handle_jpeg80, image2, L"../../../data/mushroom.jpg", GX_UNDEF);
	term_if_err(st);

	/* Saves it to JPEG format (where the quality is 80) */
	st = gx_saveimage(handle_jpeg80, image2, L"test_jpeg80.jpg", GX_JPEG);
	term_if_err(st);

	/* Frees the image1 structure */
	st = gx_unrefimage(handle_jpeg10, image1);
	term_if_err(st);
	
	/* Frees the image2 structure */
	st = gx_unrefimage(handle_jpeg80, image2);
	term_if_err(st);
	
	/* Closes the first handle */
	st = gx_closehandle(&handle_jpeg10);
	term_if_err(st);

	/* Closes the second handle */
	st = gx_closehandle(&handle_jpeg80);
	term_if_err(st);
			
	return 0;
}
