/*******************************************************************************
 * GX sample program
 *
 * Load image from memory example example.
 ******************************************************************************/

#include "gxsd.h"
#include "gxerror.h"
#include "gximage.h"
#include "gxsdldr.c"

/*******************************************************************************
 * If an error occurred it displays the error and terminates the program
 ******************************************************************************/
void term_if_err(int st) {
	int err_code;
	wchar_t err_str[256];
	err_str[(sizeof(err_str)/sizeof(wchar_t))-1] = 0;

	if(st) return;		/* No error occurred */

	/* Displays error code and description */
	gx_geterror(&err_code, err_str, (sizeof(err_str)/sizeof(wchar_t))-1);
	fprintf(stderr, "GX error occurred: %i %ls\n", err_code, err_str);

	/* Terminates the program */
	exit(1);
}

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main(void) {
	struct gxHANDLE handle;
	struct gxIMAGE *image;
	int st;
	
	/* Pixels of the 4x4 chessboard (0x00 = black pixel, 0xFF = white pixel) */
	const unsigned char pixels[4*4] = {
		0x00, 0xFF, 0x00, 0xFF,
		0xFF, 0x00, 0xFF, 0x00,
		0x00, 0xFF, 0x00, 0xFF,
		0xFF, 0x00, 0xFF, 0x00,
	};
	
	/* Opens an image module with default properties */
	st = gx_openmodule(&handle, L"gximage", L"default");
	term_if_err(st);
	
	/* Creates a new image struct */
	st = gx_allocimage(handle, &image);
	term_if_err(st);
	
	/***************************************************************************
	 * Creates a pixel area for the chessboard
	 *  pixelformat:  grayscale (GX_GRAY: 1 byte per pixel)
	 *  width,height: 4x4 pixel
	 *  sline:        4 byte (size of a row = 4 pixel * 1 byte /grayscale/ )
	 **************************************************************************/
	st = gx_createimage(handle, image, GX_GRAY, 4, 4, 4);
	term_if_err(st);
	
	/* Loads the pixel data from the memory (-1 = RAW format) */
	st = gx_loadfrommem(handle, image, (const char *)pixels, sizeof(pixels), -1);
	term_if_err(st);

	/* Saves the chessboard image */
	st = gx_saveimage(handle, image, L"test_chessboard.png", GX_PNG);
	term_if_err(st);

	/* Frees the image structure */
	st = gx_unrefimage(handle, image);
	term_if_err(st);
	
	/* Closes the opened image module */
	st = gx_closehandle(&handle);
	term_if_err(st);
	
	return 0;
}
