/*******************************************************************************
 * GX sample program
 *
 * Ref, unref image example
 ******************************************************************************/

#include "gxsd.h"
#include "gxerror.h"
#include "gximage.h"
#include "gxsdldr.c"

struct gxIMAGE *workimg = 0;	/* The image to work with */

/*******************************************************************************
 * If an error occurred it displays the error and terminates the program
 ******************************************************************************/
void term_if_err(int st) {
	int err_code;
	wchar_t err_str[256];
	err_str[(sizeof(err_str)/sizeof(wchar_t))-1] = 0;

	if(st) return;		/* No error occurred */

	/* Displays error code and description */
	gx_geterror(&err_code, err_str, (sizeof(err_str)/sizeof(wchar_t))-1);
	fprintf(stderr, "GX error occurred: %i %ls\n", err_code, err_str);

	/* Terminates the program */
	exit(1);
}

/*******************************************************************************
 * First process
 ******************************************************************************/
void my_process_begin(struct gxIMAGE *image) {
	/* Opens an image module */
	struct gxHANDLE handle;
	int st = gx_openmodule(&handle, L"gximage", L"default");
	term_if_err(st);

	/* Stores the pointer */
	workimg = image;

	/* Increases the reference counter (the main::gx_unrefimage() will not free) */
	st = gx_refimage(handle, image);
	term_if_err(st);

	/* Closes the opened module */
	st = gx_closehandle(&handle);
	term_if_err(st);
}

/*******************************************************************************
 * Second process
 ******************************************************************************/
void my_process_end(void) {
	/* Opens an image module */
	struct gxHANDLE handle;
	int st = gx_openmodule(&handle, L"gximage", L"default");
	term_if_err(st);

	/* Saves the image */
	st = gx_saveimage(handle, workimg, L"test_attach.jpg", GX_JPEG);
	term_if_err(st);
	
	/* Decreases the reference counter and if it is zero, the function frees the pixel area. */
	st = gx_unrefimage(handle, workimg);
	term_if_err(st);
	
	/* Closes the opened module */
	st = gx_closehandle(&handle);
	term_if_err(st);
}

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main(void) {
	int st;
	struct gxHANDLE handle;
	struct gxIMAGE *image;
	
	/* Opens an image module with default properties */
	st = gx_openmodule(&handle, L"gximage", L"default");
	term_if_err(st);
	
	/* Creates a new image structure */
	st = gx_allocimage(handle, &image);
	term_if_err(st);

	/* Loads the sample image */
	st = gx_loadimage(handle, image, L"../../../data/mushroom.jpg", GX_UNDEF);
	term_if_err(st);
	
	/* Calls the first process which attaches this image */
	my_process_begin(image);
	/* Decreases the reference counter of the image */
	gx_unrefimage(handle, image);
	
	/* Calls the second process which saves and removes the work image */
	my_process_end();
	
	/* Closes the opened module */
	st = gx_closehandle(&handle);
	term_if_err(st);

	return 0;
}
