#ifndef GXIMAGE_C_IFC_H
#define GXIMAGE_C_IFC_H

#include "gximage_types.h"

#if !defined(GX_SWIGIF) && !defined(GX_MANUAL)	/*(*/

#ifdef GX_NAMESPACES
	namespace gx {
#endif


/******************************************************************************/
/* Parameter structure for the gx_getpixelsize() function. */
struct GX_PARM_GETPIXELSIZE {
	gxi32 format;	/**< Pixel format. */
	gxi32 size;		/**< Size of the pixel format. */
};

/* Parameter structure for the gx_convertpixels() function. */
struct GX_PARM_CONVERTPIXELS {
	void *target;			/**< Pointer to a pixel data to where the function stores the data. */
	gxi32 tformat;			/**< Pixel format of the target area. */
	gxi32 tsline;			/**< Size of a row of the target area in bytes. */
	const void *source;		/**< Pointer to a pixel data. */
	gxi32 sformat;			/**< Pixel format of the source area */
	gxi32 ssline;			/**< Size of a row of the source area in bytes. */
	gxi32 xsize;			/**< Horizontal size of the pixel area. */
	gxi32 ysize;			/**< Vertical size of the pixel area. */
};

/** Parameter structure for the gx_fastreconvert() function.
 *
 * The fast reconvert function can be used for converting:
 * - RGB555,BGR555,RGB565,BGR565 \<-\> RGB555,BGR555,RGB565,BGR565
 * - RGB,BGR \<-\> RGB,BGR
 * - RGBA,BGRA \<-\> RGBA,BGRA
 * - RGB12,BGR12 \<-\> RGB12,BGR12 */
struct GX_PARM_FASTRECONVERT {
	void *pixels;		/**< Pointer to a pixel data. */
	gxi32 tformat;		/**< Pixel format of the destination image. */
	gxi32 sformat;		/**< Pixel format of the source image. */
	gxi32 sline;		/**< Size of a row of the pixel area in bytes. */
	gxi32 xsize;		/**< Horizontal size of the pixel area. */
	gxi32 ysize;		/**< Vertical size of the pixel area. */
};

/** Parameter structure for the gx_zoompixels() function. */
struct GX_PARM_ZOOMPIXELS {
	void *target;		/**< Pointer to an allocated area where the zoom function places the converted image. */
	gxi32 tsline;		/**< Size of a row of the destination area. */
	gxi32 txsize;		/**< Horizontal size of the destination area. */
	gxi32 tysize;		/**< Vertical size of the destination area. */
	const void *source;	/**< Pointer to pixels of source area. */
	gxi32 ssline;		/**< Size of a row of the source area. */
	gxi32 sxsize;		/**< Horizontal size of the source area. */
	gxi32 sysize;		/**< Vertical size of the source area. */
	gxi32 format;		/**< Pixel format. */
	gxi32 sx1_8;		/**< Left coordinate of zoom area (shifted left with 8 bit = 256) */
	gxi32 sy1_8;		/**< Top coordinate of zoom area (shifted left with 8 bit = 256) */
	gxi32 sx2_8;		/**< Right coordinate of zoom area (shifted left with 8 bit = 256) */
	gxi32 sy2_8;		/**< Bottom coordinate of zoom area (shifted left with 8 bit = 256) */
};

/** Parameter structure for the gx_rotatepixels() function. */
struct GX_PARM_ROTATEPIXELS {
	void *target;		/**< Pointer to an allocated area where the rotate function places the rotated image. */
	gxi32 tsline;		/**< Size of a row of the destination area. */
	gxi32 txsize;		/**< Horizontal size of the destination area. */
	gxi32 tysize;		/**< Vertical size of the destination area. */
	const void *source;	/**< Pointer to pixels of source area. */
	gxi32 ssline;		/**< Size of a row of the source area. */
	gxi32 sxsize;		/**< Horizontal size of the source area. */
	gxi32 sysize;		/**< Vertical size of the source area. */
	gxi32 format;		/**< Pixel format. */
	gxi32 degree;		/**< Degree (Must be 0, 90, 180, 270). */
};

/** Parameter structure for the gx_mirrorpixels() function. */
struct GX_PARM_MIRRORPIXELS {
	void *target;		/**< Pointer to an allocated area where the mirror function places the mirror image. */
	gxi32 tsline;		/**< Size of a row of the destination area. */
	gxi32 txsize;		/**< Horizontal size of the destination area. */
	gxi32 tysize;		/**< Vertical size of the destination area. */
	const void *source;	/**< Pointer to pixels of source area. */
	gxi32 ssline;		/**< Size of a row of the source area. */
	gxi32 sxsize;		/**< Horizontal size of the source area. */
	gxi32 sysize;		/**< Vertical size of the source area. */
	gxi32 format;		/**< Pixel format. */
	gxi32 flags;		/**< Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and GX_MIRROR_VERTICAL. */
};

/** Parameter structure for the gx_createimage() function. */
struct GX_PARM_CREATEIMAGE {
	gxIMAGE *pimage;	/**< Pointer to an initialized image structure. */
	gxi32	 format;	/**< Pixel format. */
	gxi32	 xsize;		/**< Horizontal size of the image in pixels. */
	gxi32	 ysize;		/**< Vertical size of the image in pixels. */
	gxi32	 sline;		/**< Length of a row in bytes (min.: xsize * size of a pixel in bytes). If zero the system computes the correct value. */
};

/** Parameter structure for the gx_convertimage() function. */
struct GX_PARM_CONVERTIMAGE {
	gxIMAGE *pdest;		/**< Pointer to an image where the function writes the converted image. */
	gxIMAGE *psrc;		/**< Pointer to the source image. If it is equal to pdest the function attempts to fast reconvert. */
	gxi32	 format;	/**< Pixel format. */
	gxi32	 xsize;		/**< Horizontal size of the image in pixels. */
	gxi32	 ysize;		/**< Vertical size of the image in pixels. */
	gxi32	 sline;		/**< Length of a row of the destination image in bytes. If zero the system computes the correct value. */
};

/** Parameter structure for the gx_zoomimage() function. */
struct GX_PARM_ZOOMIMAGE {
	gxIMAGE	 *pdest;	/**< Pointer to an image where the zoom function places the converted image. */
	gxi32	  xsize;	/**< Horizontal size of the destination image. */
	gxi32	  ysize;	/**< Vertical size of the destination image. */
	gxi32	  sline;	/**< Length of a row of the destination image in bytes. It can be zero. */
	gxIMAGE	 *psrc;		/**< Pointer to the source image. */
	gxi32	  sx1_8;	/**< Left coordinate of zoom area (shifted left with 8 bit = 256) */
	gxi32	  sy1_8;	/**< Top coordinate of zoom area (shifted left with 8 bit = 256) */
	gxi32	  sx2_8;	/**< Right coordinate of zoom area (shifted left with 8 bit = 256) */
	gxi32	  sy2_8;	/**< Bottom coordinate of zoom area (shifted left with 8 bit = 256) */
};

/** Parameter structure for the gx_rotateimage() function. */
struct GX_PARM_ROTATEIMAGE {
	gxIMAGE	*pdest;		/**< Pointer to an image where the rotate function places the converted image. */
	gxIMAGE *psrc;		/**< Pointer to the source image. */
	gxi32	 degree;	/**< Degree (Must be 0, 90, 180, 270). */
};

/** Parameter structure for the gx_mirrorimage() function. */
struct GX_PARM_MIRRORIMAGE {
	gxIMAGE	*pdest;		/**< Pointer to an image where the mirror function places the converted image. */
	gxIMAGE *psrc;		/**< Pointer to the source image. */
	gxi32	 flags;		/**< Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and GX_MIRROR_VERTICAL. */
};

/** Parameter structure for the gx_loadimage() function (Unicode version). */
struct GX_PARM_LOADIMAGE {
	gxIMAGE *pimage;		/**< Pointer to an image where the function loads the data to. */
	const wchar_t *filename;/**< Name of the image file. */
	gxi32	 format;		/**< The specific pixel format (giving GX_GRAY converts a color image to grayscale). */
};

/** Parameter structure for the gx_loadimage() function (ASCII version). */
struct GX_PARM_LOADIMAGEA {
	gxIMAGE *pimage;		/**< Pointer to an image where the function loads the data to. */
	const char *filename;	/**< Name of the image file. */
	gxi32	 format;		/**< The specific pixel format (giving GX_GRAY converts a color image to grayscale). */
};

/** Parameter structure for the gx_saveimage() function (Unicode version). */
struct GX_PARM_SAVEIMAGE {
	gxIMAGE *pimage;		/**< Pointer to the image. */
	const wchar_t *filename;/**< Name of the image file. */
	gxi32	 fileformat;	/**< Format of the image file (giving GX_BMP writes the file in BMP format). */
};

/** Parameter structure for the gx_saveimage() function (ASCII version). */
struct GX_PARM_SAVEIMAGEA {
	gxIMAGE *pimage;		/**< Pointer to the image. */
	const char *filename;	/**< Name of the image file. */
	gxi32	 fileformat;	/**< Format of the image file (giving GX_BMP writes the file in BMP format). */
};

/** Parameter structure for the gx_loadfrommem() function. */
struct GX_PARM_LOADFROMMEM {
	gxIMAGE	*pimage;		/**< Pointer to an image where the function loads the data to. */
	const void *buffer;		/**< Pointer to the image data. */
	gxi32	 buflen;		/**< Length of image data in bytes. */
	gxi32	 format;		/**< The specific pixel format (giving GX_GRAY converts a color image to grayscale). */
};

/** Parameter structure for the gx_savetomem() function. */
struct GX_PARM_SAVETOMEM {
	gxIMAGE *pimage;		/**< Pointer to the image. */
	void	**buffer;		/**< Pointer to a memory area pointer. */
	gxi32	*buflen;		/**< Size of memory area. */
	gxi32	 fileformat;	/**< Format of the image file (giving GX_BMP writes the file in BMP format). */
};

#if defined(WIN32)
/** Parameter structure for the gx_displayimage() function. */
struct GX_PARM_DISPLAYIMAGE {
	HDC		 hdc;			/**< Device handle. */
	gxi32	 x;				/**< X1 coordinate where the function writes the first pixel of the image. */
	gxi32	 y;				/**< Y1 coordinate where the function writes the first pixel of the image. */
	gxi32	 xsize;			/**< Horizontal size of displayed area. */
	gxi32	 ysize;			/**< Vertical size of displayed area. */
	gxIMAGE *pimage;		/**< Pointer to the image. */
};
#endif

/** Parameter structure for the gx_getpixelptr() function. */
struct GX_PARM_GETPIXELPTR {
	gxIMAGE *pimage;		/**< Pointer to the image. */
	gxi32	 x;				/**< X coordinate of the pixel. */
	gxi32	 y;				/**< Y coordinate of the pixel. */
	void	*ptr;			/**< Returned pointer of the pixel. */
};

/** Parameter structure for the gx_loadimagefromfile() function (Unicode version). */
struct GX_PARM_LOADIMAGEFROMFILE {
	gxIMAGE			*pimage;		/**< Pointer to an image where the function loads the data to. */
	const wchar_t	*filename;		/**< Name of the image file. */
	gxi32			 format;		/**< The specific pixel format (giving GX_GRAY converts a color image to grayscale) .*/
	gxVARIANT		 properties;	/**< Properties for the image loader. It can be NULL. */
	gxVARIANT		 comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_loadimagefromfilea() function (ASCII version). */
struct GX_PARM_LOADIMAGEFROMFILEA {
	gxIMAGE			*pimage;		/**< Pointer to an image where the function loads the data to. */
	const char		*filename;		/**< Name of the image file. */
	gxi32	 		 format;		/**< The specific pixel format (giving GX_GRAY converts a color image to grayscale). */
	gxVARIANT		 properties;	/**< Properties for the image loader. It can be NULL. */
	gxVARIANT		 comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_saveimagetofile() function (Unicode version). */
struct GX_PARM_SAVEIMAGETOFILE {
	gxIMAGE			*pimage;		/**< Pointer to the image. */
	const wchar_t	*filename;		/**< Name of the image file. */
	gxi32	 		 fileformat;	/**< Format of the image file (giving GX_BMP writes the file in BMP format). */
	gxVARIANT		 properties;	/**< Properties for the image creator. It can be NULL. */
	gxVARIANT		 comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_saveimagetofilea() function (ASCII version). */
struct GX_PARM_SAVEIMAGETOFILEA {
	gxIMAGE			*pimage;		/**< Pointer to the image. */
	const char		*filename;		/**< Name of the image file. */
	gxi32	 		 fileformat;	/**< Format of the image file (giving GX_BMP writes the file in BMP format). */
	gxVARIANT		 properties;	/**< Properties for the image creator. It can be NULL. */
	gxVARIANT		 comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_saveimagetofile2() function (Unicode version). */
struct GX_PARM_SAVEIMAGETOFILE2 {
	gxIMAGE			*pimage;		/**< Pointer to the image. */
	const wchar_t	*filename;		/**< Name of the image file. */
	const wchar_t	*fileformat;	/**< Format of the image file (giving "bmp" writes the file in BMP format). */
	gxVARIANT		 properties;	/**< Properties for the image creator. It can be NULL. */
	gxVARIANT		 comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_loadimagefrommem() function. */
struct GX_PARM_LOADIMAGEFROMMEM {
	gxIMAGE			*pimage;		/**< Pointer to an image where the function loads the data to. */
	const void		*buffer;		/**< Pointer to the image data. */
	gxi32	 		 buflen;		/**< Length of image data in bytes. */
	gxi32	 		 format;		/**< The specific pixel format (giving GX_GRAY converts a color image to grayscale). */
	gxVARIANT		 properties;	/**< Properties for the image loader. It can be NULL. */
	gxVARIANT		 comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_saveimagetomem() function. */
struct GX_PARM_SAVEIMAGETOMEM {
	gxIMAGE			 *pimage;		/**< Pointer to the image. */
	void			**buffer;		/**< Pointer to a memory area pointer. */
	gxi32			 *buflen;		/**< Size of memory area. */
	gxi32	 		  fileformat;	/**< Format of the image file (giving GX_BMP writes the file in BMP format). */
	gxVARIANT		  properties;	/**< Properties for the image creator. It can be NULL. */
	gxVARIANT		  comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_saveimagetomem2() function. */
struct GX_PARM_SAVEIMAGETOMEM2 {
	gxIMAGE			 *pimage;		/**< Pointer to the image. */
	void			**buffer;		/**< Pointer to a memory area pointer. */
	gxi32			 *buflen;		/**< Size of memory area. */
	const wchar_t	 *fileformat;	/**< Format of the image file (giving "bmp" writes the file in BMP format). */
	gxVARIANT		  properties;	/**< Properties for the image creator. It can be NULL. */
	gxVARIANT		  comments;		/**< Comments of the image. It can be NULL. */
};

/** Parameter structure for the gx_copyimage() function. */
struct GX_PARM_COPYIMAGE {
	gxIMAGE		*target;		/**< Pointer to an initialized destination structure. */
	gxIMAGE		*source;		/**< Pointer to the source. */
};

/** Parameter structure for the gx_imagetovariant() function. */
struct GX_PARM_IMAGETOVARIANT {
	gxIMAGE		*pimage;		/**< Pointer to the image. */
	int			 varid;			/**< The new variant id. */
	gxVARIANT	 variant;		/**< Variant structure for result. */
};

/** Parameter structure for the gx_imagefromvariant() function. */
struct GX_PARM_IMAGEFROMVARIANT {
	gxIMAGE		*pimage;		/**< Pointer to an initialized destination structure. */
	gxVARIANT	 variant;		/**< Source variant structure. */
};

/** Parameter structure for the gx_testimagefileformat() function. */
struct GX_PARM_TESTIMAGEFILEFORMAT {
	const wchar_t	*filename;	/**< Name of the image file. */
	gxi32			 fileformat;	/**< Format of the image file. */
};

/** Parameter structure for the gx_testimagefileformata() function. */
struct GX_PARM_TESTIMAGEFILEFORMATA {
	const char	*filename;		/**< Name of the image file. */
	gxi32		 fileformat;	/**< Format of the image file. */
};

/** Parameter structure for the gx_testimagefileformatmem() function. */
struct GX_PARM_TESTIMAGEFILEFORMATMEM {
	const void	*buffer;		/**< Pointer to a memory area pointer. */
	gxi32		 buflen;		/**< Size of memory area. */
	gxi32		 fileformat;	/**< Format of the image file. */
};

/** Parameter structure for the gx_testimagefileformatid() function. */
struct GX_PARM_TESTIMAGEFILEFORMATID {
	const wchar_t	*filename;	/**< Name of the image file. */
	wchar_t*		 formatid;	/**< Format Id of the image file. */
	gxi32			 slen;		/**< Size of memory area. */
};

/** Parameter structure for the gx_testimagefileformatidmem() function. */
struct GX_PARM_TESTIMAGEFILEFORMATIDMEM {
	const void	*buffer;		/**< Pointer to a memory area pointer. */
	gxi32		 buflen;		/**< Size of memory area. */
	wchar_t*		 formatid;	/**< Format Id of the image file. */
	gxi32			 slen;		/**< Size of memory area. */
};

#endif	/* GX_SWIGIF GX_MANUAL )*/

/******************************************************************************/
#if !defined(NO_GX_FUNCTIONS) && !defined(GX_SWIGIF)	/*(*/
/******************************************************************************/

/******************************************************************************/
/** Returns the size of a pixel in bytes.
 * @see GX_PARM_GETPIXELSIZE
 * @param handle Handle of the module.
 * @param size Size of the pixel format.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @return	On error false is returned and the error code/string in the GX system
 *			is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_getpixelsize(gxHANDLE handle, gxi32 *size, gxi32 format) {
	struct GX_PARM_GETPIXELSIZE ps;
	gxi32 ret;
	assert(size);
	ps.format = format;
	ret = gx_call(handle, GX_CALL_GETPIXELSIZE, &ps);
	if(ret) *size = ps.size;
	return ret;
}

/******************************************************************************/
/** Converts pixels to a specified format and size.
 * @see GX_PARM_CONVERTPIXELS
 * @param handle Handle of the module.
 * @param target Pointer to a pixel data to where the function stores the data.
 * @param tformat Pixel format of the target area (see \ref GX_PIXELFORMATS).
 * @param tsline Size of a row of the target area in bytes.
 * @param source Pointer to a pixel data.
 * @param sformat Pixel format of the source area (see \ref GX_PIXELFORMATS).
 * @param ssline Size of a row of the source area in bytes.
 * @param xsize Horizontal size of the pixel area.
 * @param ysize Vertical size of the pixel area.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_convertpixels(gxHANDLE handle,
					void *target, gxi32 tformat, gxi32 tsline,
					const void *source, gxi32 sformat, gxi32 ssline,
					gxi32 xsize, gxi32 ysize) {

	struct GX_PARM_CONVERTPIXELS cp;
	cp.target = target;
	cp.tformat = tformat;
	cp.tsline = tsline;
	cp.source = source;
	cp.sformat = sformat;
	cp.ssline = ssline;
	cp.xsize = xsize;
	cp.ysize = ysize;
	return gx_call(handle, GX_CALL_CONVERTPIXELS, &cp);
}

/******************************************************************************/
/** Fast converts pixels to a specified format and size.
 *
 * The function is the same as gx_convertpixels(), only it works on the same memory area.
 * \par
 * The fast reconvert function can be used for converting:
 * - RGB555,BGR555,RGB565,BGR565 \<-\> RGB555,BGR555,RGB565,BGR565
 * - RGB,BGR \<-\> RGB,BGR
 * - RGBA,BGRA \<-\> RGBA,BGRA
 * - RGB12,BGR12 \<-\> RGB12,BGR12
 * @param handle Handle of the module.
 * @param pixels Pointer to a pixel data.
 * @param tformat Pixel format of the destination image (see \ref GX_PIXELFORMATS).
 * @param sformat Pixel format of the source image (see \ref GX_PIXELFORMATS).
 * @param sline Size of a row of the pixel area in bytes.
 * @param xsize Horizontal size of the pixel area.
 * @param ysize Vertical size of the pixel area.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_fastreconvert(gxHANDLE handle,
						void *pixels, gxi32 tformat, gxi32 sformat,
						gxi32 sline, gxi32 xsize, gxi32 ysize) {

	struct GX_PARM_FASTRECONVERT fr;
	fr.pixels = pixels;
	fr.tformat = tformat;
	fr.sformat = sformat;
	fr.sline = sline;
	fr.xsize = xsize;
	fr.ysize = ysize;
	return gx_call(handle, GX_CALL_FASTRECONVERT, &fr);
}

/******************************************************************************/
/** Zooms pixels to a specified size.
 * @param handle Handle of the module.
 * @param target Pointer to an allocated area where the zoom function places the
 *               converted image.
 * @param tsline Size of a row of the destination area.
 * @param txsize Horizontal size of the destination area.
 * @param tysize Vertical size of the destination area.
 * @param source Pointer to pixels of source area.
 * @param ssline Size of a row of the source area.
 * @param sxsize Horizontal size of the source area.
 * @param sysize Vertical size of the source area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param sx1_8  Left coordinate of zoom area (shifted left with 8 bit = 256)
 * @param sy1_8  Top coordinate of zoom area (shifted left with 8 bit = 256)
 * @param sx2_8  Right coordinate of zoom area (shifted left with 8 bit = 256)
 * @param sy2_8  Bottom coordinate of zoom area (shifted left with 8 bit = 256)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_zoompixels(gxHANDLE handle,
						void *target, gxi32 tsline, gxi32 txsize, gxi32 tysize,
						const void *source, gxi32 ssline, gxi32 sxsize, gxi32 sysize,
						gxi32 format,
						gxi32 sx1_8 GX_DEFARG(0), gxi32 sy1_8 GX_DEFARG(0),
						gxi32 sx2_8 GX_DEFARG(0), gxi32 sy2_8 GX_DEFARG(0))
						{

	struct GX_PARM_ZOOMPIXELS zp;
	assert(target);
	assert(source);
	zp.target = target;
	zp.tsline = tsline;
	zp.txsize = txsize;
	zp.tysize = tysize;
	zp.source = (void *)source;
	zp.ssline = ssline;
	zp.sxsize = sxsize;
	zp.sysize = sysize;
	zp.format = format;
	zp.sx1_8  = sx1_8;
	zp.sy1_8  = sy1_8;
	zp.sx2_8  = sx2_8;
	zp.sy2_8  = sy2_8;
	return gx_call(handle, GX_CALL_ZOOMPIXELS, &zp);
}

/******************************************************************************/
/** Rotates pixels to a specified degree.
 * @param handle Handle of the module.
 * @param target Pointer to an allocated area where the rotate function places
 *               the rotated image.
 * @param tsline Size of a row of the destination area.
 * @param txsize Horizontal size of the destination area.
 * @param tysize Vertical size of the destination area.
 * @param source Pointer to pixels of source area.
 * @param ssline Size of a row of the source area.
 * @param sxsize Horizontal size of the source area.
 * @param sysize Vertical size of the source area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param degree Degree (Must be 0, 90, 180, 270).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_rotatepixels(gxHANDLE handle,
						void *target, gxi32 tsline, gxi32 txsize, gxi32 tysize,
						const void *source, gxi32 ssline, gxi32 sxsize, gxi32 sysize,
						gxi32 format, gxi32 degree) {

	struct GX_PARM_ROTATEPIXELS rp;
	assert(target);
	assert(source);
	rp.target = target;
	rp.tsline = tsline;
	rp.txsize = txsize;
	rp.tysize = tysize;
	rp.source = source;
	rp.ssline = ssline;
	rp.sxsize = sxsize;
	rp.sysize = sysize;
	rp.format = format;
	rp.degree = degree;
	return gx_call(handle, GX_CALL_ROTATEPIXELS, &rp);
}

/******************************************************************************/
/** Mirrors pixels to a specified mode.
 * @param handle Handle of the module.
 * @param target Pointer to an allocated area where the mirror function places
 *               the mirrored image.
 * @param tsline Size of a row of the destination area.
 * @param txsize Horizontal size of the destination area.
 * @param tysize Vertical size of the destination area.
 * @param source Pointer to pixels of source area.
 * @param ssline Size of a row of the source area.
 * @param sxsize Horizontal size of the source area.
 * @param sysize Vertical size of the source area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param flags  Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and GX_MIRROR_VERTICAL.
 *               (see \ref GX_MIRRORFLAGS)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorpixels(gxHANDLE handle,
						void *target, gxi32 tsline, gxi32 txsize, gxi32 tysize,
						const void *source, gxi32 ssline, gxi32 sxsize, gxi32 sysize,
						gxi32 format, gxi32 flags) {

	struct GX_PARM_MIRRORPIXELS mp;
	assert(target);
	assert(source);
	mp.target = target;
	mp.tsline = tsline;
	mp.txsize = txsize;
	mp.tysize = tysize;
	mp.source = source;
	mp.ssline = ssline;
	mp.sxsize = sxsize;
	mp.sysize = sysize;
	mp.format = format;
	mp.flags = flags;
	return gx_call(handle, GX_CALL_MIRRORPIXELS, &mp);
}

#ifdef __cplusplus
/******************************************************************************/
/** Mirrors pixels to a specified mode.
 * @param handle Handle of the module.
 * @param pixels Pointer to pixels of the pixel area.
 * @param sline  Size of a row of the pixel area.
 * @param xsize  Horizontal size of the pixel area.
 * @param ysize  Vertical size of the pixel area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param flags  Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and GX_MIRROR_VERTICAL.
 *               (see \ref GX_MIRRORFLAGS)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorpixels(gxHANDLE handle,
						void *pixels, gxi32 sline, gxi32 xsize, gxi32 ysize,
						gxi32 format, gxi32 flags) {
	return gx_mirrorpixels(handle, pixels, sline, xsize, ysize,
						pixels, sline, xsize, ysize,
						format, flags);
}
#endif

/******************************************************************************/
/** \addtogroup gx_group_image
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Initialization of an image located on the heap or stack.
 *
 * This function fills the structure identifier and the reference counter
 * properly. The other data members are set to zero value.
 * \warning Do not call it for a dynamic allocated image (gx_allocimage)!
 * @param handle Handle of the module.
 * @param pimage Pointer to the not initialized image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_initimage(gxHANDLE handle, gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(handle, GX_CALL_INITIMAGE, (void *)pimage);
}

/******************************************************************************/
/** This function allocates a blank image.
 *
 * The structure identifier and the reference counter will be properly filled.
 * The other data members are set to zero value.
 * @param handle Handle of the module.
 * @param pimage Pointer to a \a gxIMAGE structure pointer where the function
 *               returns the pointer of the allocated image area.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_allocimage(gxHANDLE handle, gxIMAGE **pimage) {
	assert(pimage);
	return gx_call(handle, GX_CALL_ALLOCIMAGE, (void *)pimage);
}

/******************************************************************************/
/** Frees an image structure including the allocated pixel data.
 *
 * If the value of the reference counter is not properly set the function returns an
 * error. In case of dynamically allocated images it is advised to use
 * \a gx_unrefimage.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_freeimage(gxHANDLE handle, gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(handle, GX_CALL_FREEIMAGE, (void *)pimage);
}

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_reference
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Increases the image reference counter.
 *
 * The counter indicates the number of processes using the image. The counter
 * can rise to positive or negative direction as well depending on whether the image
 * is static or dynamic.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_refimage(gxHANDLE handle, gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(handle, GX_CALL_REFIMAGE, (void *)pimage);
}

/******************************************************************************/
/** Decreases the image reference counter.
 *
 * The counter indicates the number of processes using the image. If the
 * counter value becomes zero after the function call the structure will be
 * freed depending on whether the image was allocated statically or dynamically.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_unrefimage(gxHANDLE handle, gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(handle, GX_CALL_UNREFIMAGE, (void *)pimage);
}

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_image
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Makes an image with proper size and format running on an initialized image
 * structure.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image structure.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param xsize Horizontal size of image in pixels.
 * @param ysize Vertical size of image in pixels.
 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
 *         bytes). If zero the system computes the correct value.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_createimage(gxHANDLE handle, gxIMAGE* pimage, gxi32 format,
							gxi32 xsize, gxi32 ysize, gxi32 sline) {
	struct GX_PARM_CREATEIMAGE ci;
	assert(pimage);
	assert(xsize > 0);
	assert(ysize > 0);
	ci.pimage = pimage;
	ci.format = format;
	ci.xsize = xsize;
	ci.ysize = ysize;
	ci.sline = sline;
	return gx_call(handle, GX_CALL_CREATEIMAGE, (void *)&ci);
}

/******************************************************************************/
/** Converts an image to a specific format and size.
 *
 * The destination image must be initialized.
 * @param handle Handle of the module.
 * @param pdest Pointer to an initialized image where the function places the
 *              converted image.
 * @param psrc Pointer to the source image.
 * @param format Pixel format of the destination image (see \ref GX_PIXELFORMATS).
 * @param xsize Horizontal size of the destination image in pixels.
 * @param ysize Vertical size of the destination image in pixels.
 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
 *         bytes). If zero the system computes the correct value.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_convertimage(gxHANDLE handle, gxIMAGE* pdest,
						gxIMAGE* psrc, gxi32 format, gxi32 xsize, gxi32 ysize, gxi32 sline) {

	struct GX_PARM_CONVERTIMAGE ci;
	assert(pdest);
	assert(psrc);
	assert(xsize > 0);
	assert(ysize > 0);
	ci.pdest = pdest;
	ci.psrc = psrc;
	ci.format = format;
	ci.xsize = xsize;
	ci.ysize = ysize;
	ci.sline = sline;
	return gx_call(handle, GX_CALL_CONVERTIMAGE, (void *)&ci);
}

/******************************************************************************/
/** Zooms an image to a specified size.
 * @param handle Handle of the module.
 * @param pdest Pointer to an initialized image where the function places the
 *              zoomed image.
 * @param xsize Horizontal size of the destination area.
 * @param ysize Vertical size of the destination area.
 * @param sline Size of a row of the destination area.
 * @param psrc  Pointer to the source image.
 * @param sx1_8 Left coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @param sy1_8 Top coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @param sx2_8 Right coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @param sy2_8 Bottom coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_zoomimage(gxHANDLE handle, gxIMAGE *pdest,
					gxi32 xsize, gxi32 ysize, gxi32 sline,
					gxIMAGE *psrc,
					gxi32 sx1_8 GX_DEFARG(0), gxi32 sy1_8 GX_DEFARG(0),
					gxi32 sx2_8 GX_DEFARG(0), gxi32 sy2_8 GX_DEFARG(0)) {

	struct GX_PARM_ZOOMIMAGE zi;
	assert(pdest);
	assert(psrc);
	assert(xsize > 0);
	assert(ysize > 0);
	zi.pdest = pdest;
	zi.xsize = xsize;
	zi.ysize = ysize;
	zi.sline = sline;
	zi.psrc  = psrc;
	zi.sx1_8 = sx1_8;
	zi.sy1_8 = sy1_8;
	zi.sx2_8 = sx2_8;
	zi.sy2_8 = sy2_8;
	return gx_call(handle, GX_CALL_ZOOMIMAGE, (void *)&zi);
}

/******************************************************************************/
/** Rotates an image to a specified degree.
 * @param handle Handle of the module.
 * @param pdest Pointer to an initialized image where the function places the
 *              zoomed image.
 * @param psrc  Pointer to the source image.
 * @param degree Degree (Must be 0, 90, 180, 270).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_rotateimage(gxHANDLE handle, gxIMAGE *pdest,
						gxIMAGE *psrc, gxi32 degree) {

	struct GX_PARM_ROTATEIMAGE ri;
	assert(pdest);
	assert(psrc);
	ri.pdest = pdest;
	ri.psrc = psrc;
	ri.degree = degree;
	return gx_call(handle, GX_CALL_ROTATEIMAGE, &ri);
}

/******************************************************************************/
/** Mirrors an image to a specified mode.
 * @param handle Handle of the module.
 * @param pdest Pointer to an initialized image where the function places the
 *              zoomed image.
 * @param psrc  Pointer to the source image.
 * @param flags Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and
 *              GX_MIRROR_VERTICAL (see \ref GX_MIRRORFLAGS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorimage(gxHANDLE handle, gxIMAGE *pdest,
						gxIMAGE *psrc, gxi32 flags) {

	struct GX_PARM_MIRRORIMAGE mi;
	assert(pdest);
	assert(psrc);
	mi.pdest = pdest;
	mi.psrc = psrc;
	mi.flags = flags;
	return gx_call(handle, GX_CALL_MIRRORIMAGE, &mi);
}

#ifdef __cplusplus
/******************************************************************************/
/** Mirrors an image to a specified mode.
 * @param handle Handle of the module.
 * @param pimage Pointer to an image.
 * @param flags Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and
 *              GX_MIRROR_VERTICAL (see \ref GX_MIRRORFLAGS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorimage(gxHANDLE handle, gxIMAGE *pimage, gxi32 flags) {
	return gx_mirrorimage(handle, pimage, pimage, flags);
}
#endif

/******************************************************************************/
/** Loads an image from a mass storage device and converts it to a specific
 * pixel format (Unicode version).
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included). The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 *
 * \code
   gxIMAGE image;
   gx_initimage(&image);
   gx_createimage(&image, GX_GRAY, 1024, 768, 1024);
   gx_loadimage(&image, "test.raw", -1);
   ...
   \endcode
 *
 * @param handle Handle of the module.
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param filename Name of the image file.
 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimage(gxHANDLE handle, gxIMAGE* pimage,
					const wchar_t* filename, gxi32 pixelformat GX_DEFARG(GX_UNDEF)) {

	struct GX_PARM_LOADIMAGE li;
	assert(pimage);
	assert(filename);
	li.pimage = pimage;
	li.filename = filename;
	li.format = pixelformat;
	return gx_call(handle, GX_CALL_LOADIMAGE, (void *)&li);
}

/******************************************************************************/
/** Loads an image from a mass storage device and converts it to a specific
 * pixel format (ASCII version).
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included). The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 *
 * \code
   gxIMAGE image;
   gx_initimage(&image);
   gx_createimage(&image, GX_GRAY, 1024, 768, 1024);
   gx_loadimagea(&image, "test.raw", -1);
   ...
   \endcode
 *
 * @param handle Handle of the module.
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param filename Name of the image file.
 * @param pixelformat The specific pixel format.  Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimagea(gxHANDLE handle, gxIMAGE* pimage,
						const char* filename, gxi32 pixelformat GX_DEFARG(GX_UNDEF)) {
	struct GX_PARM_LOADIMAGEA li;
	assert(pimage);
	assert(filename);
	li.pimage = pimage;
	li.filename = filename;
	li.format = pixelformat;
	return gx_call(handle, GX_CALL_LOADIMAGEA, (void *)&li);
}

#ifdef __cplusplus
/******************************************************************************/
/** Loads an image from a mass storage device and converts it to a specific
 * pixel format (ASCII version).
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included). The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 *
 * \code
   gxIMAGE image;
   gx_initimage(&image);
   gx_createimage(&image, GX_GRAY, 1024, 768, 1024);
   gx_loadimage(&image, "test.raw", -1);
   ...
   \endcode
 *
 * @param handle Handle of the module.
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param filename Name of the image file.
 * @param pixelformat The specific pixel format.  Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimage(gxHANDLE handle, gxIMAGE* pimage,
					const char* filename, gxi32 pixelformat GX_DEFARG(GX_UNDEF)) {

	return gx_loadimagea(handle, pimage, filename, pixelformat);
}
#endif

/******************************************************************************/
/** Saves an image to a mass storage device in the specific file format
 * (Unicode version).
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimage(gxHANDLE handle, gxIMAGE* pimage,
						const wchar_t* filename, gxi32 fileformat) {

	struct GX_PARM_SAVEIMAGE si;
	assert(pimage);
	assert(filename);
	si.pimage = pimage;
	si.filename = filename;
	si.fileformat = fileformat;
	return gx_call(handle, GX_CALL_SAVEIMAGE, (void *)&si);
}

/******************************************************************************/
/** Saves an image to a mass storage device in the specific file format
 * (ASCII version).
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimagea(gxHANDLE handle, gxIMAGE* pimage,
						const char* filename, gxi32 fileformat) {

	struct GX_PARM_SAVEIMAGEA si;
	assert(pimage);
	assert(filename);
	si.pimage = pimage;
	si.filename = filename;
	si.fileformat = fileformat;
	return gx_call(handle, GX_CALL_SAVEIMAGEA, (void *)&si);
}

#ifdef __cplusplus
/******************************************************************************/
/** Saves an image to a mass storage device in the specific file format
 * (ASCII version).
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimage(gxHANDLE handle, gxIMAGE *pimage,
						const char *filename, gxi32 fileformat) {

	return gx_saveimagea(handle, pimage, filename, fileformat);
}
#endif

/******************************************************************************/
/** Loads an image from the memory and converts it to a specific pixel format.
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included). The format parameter of the
 * gx_loadfrommem() must be -1 for RAW images.
 *
 * @param handle Handle of the module.
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param buffer Pointer to the image data.
 * @param buflen Length of image data in bytes.
 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadfrommem(gxHANDLE handle, gxIMAGE* pimage,
					const void *buffer,	gxi32 buflen, gxi32 pixelformat GX_DEFARG(GX_UNDEF)) {
	struct GX_PARM_LOADFROMMEM lm;
	assert(pimage);
	assert(buffer);
	assert(buflen > 0);
	lm.pimage = pimage;
	lm.buffer = buffer;
	lm.buflen = buflen;
	lm.format = pixelformat;
	return gx_call(handle, GX_CALL_LOADFROMMEM, (void *)&lm);
}

/******************************************************************************/
/** Saves an image in a specific file format to a specific memory area.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @param buffer Pointer to the memory area pointer that is large enough or NULL.
 * @param buflen Size of the memory area.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_savetomem(gxHANDLE handle, gxIMAGE* pimage,
					void **buffer, gxi32 *buflen, gxi32 fileformat) {
	struct GX_PARM_SAVETOMEM sm;
	assert(pimage);
	assert(buffer);
	assert(buflen);
	assert(*buflen >= 0);
	sm.pimage = pimage;
	sm.buffer = buffer;
	sm.buflen = buflen;
	sm.fileformat = fileformat;
	return gx_call(handle, GX_CALL_SAVETOMEM, (void *)&sm);
}

/******************************************************************************/
/** Checks if image structure is valid.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @return In case of valid image the return value is non-zero. */
inline gxi32 gx_isvalidimage(gxHANDLE handle, const gxIMAGE *pimage) {
	return gx_call(handle, GX_CALL_ISVALIDIMAGE, (void *)pimage);
}

/******************************************************************************/
#ifdef WIN32	/*(*/
/** Displays an image.
 * @param handle Handle of the module.
 * @param hdc Handle of device context.
 * @param x   X1 coordinate of display.
 * @param y   Y1 coordinate of display.
 * @param xsize Displayed horizontal size of image.
 * @param ysize Displayed vertical size of image.
 * @param pimage Pointer to the image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_displayimage(gxHANDLE handle, HDC hdc, gxi32 x, gxi32 y,
							gxi32 xsize, gxi32 ysize, gxIMAGE *pimage) {

	struct GX_PARM_DISPLAYIMAGE di;
	assert(pimage);
	di.hdc = hdc;
	di.x = x;
	di.y = y;
	di.xsize = xsize;
	di.ysize = ysize;
	di.pimage = pimage;
	return gx_call(handle, GX_CALL_DISPLAYIMAGE, (void *)&di);
}

/******************************************************************************/
/** Copies an image to the clipboard.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_imgtoclipboard(gxHANDLE handle, gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(handle, GX_CALL_IMGTOCLIPBOARD, (void *)pimage);
}

/******************************************************************************/
/** Inserts an image from the clipboard.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_pasteimage(gxHANDLE handle, gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(handle, GX_CALL_PASTEIMAGE, (void *)pimage);
}
#endif	/*)*/

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_image
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Copies an image to another.
 * @param handle Handle of the module.
 * @param target Pointer to the initialized destination image.
 * @param source Pointer to the source image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_copyimage(gxHANDLE handle, gxIMAGE *target, gxIMAGE *source) {
	struct GX_PARM_COPYIMAGE cp;
	assert(target);
	assert(source);
	cp.target = target;
	cp.source = source;
	return gx_call(handle, GX_CALL_COPYIMAGE, (void *)&cp);
}

/******************************************************************************/
/** Tests if a file is an image file.
 *
 * @param handle Handle of the module.
 * @param filename Name of the file to test.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformat(gxHANDLE handle, const wchar_t *filename,
									gxi32 *fileformat) {

	struct GX_PARM_TESTIMAGEFILEFORMAT ti;
	assert(fileformat);
	ti.filename = filename;
	if(!gx_call(handle, GX_CALL_TESTIMAGEFILEFORMAT, (void *)&ti)) return false;
	*fileformat = ti.fileformat;
	return true;
}

/******************************************************************************/
/** Tests if a file is an image file.
 *
 * @param handle Handle of the module.
 * @param filename Name of the file to test.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformata(gxHANDLE handle, const char *filename,
									gxi32 *fileformat) {

	struct GX_PARM_TESTIMAGEFILEFORMATA ti;
	assert(fileformat);
	ti.filename = filename;
	if(!gx_call(handle, GX_CALL_TESTIMAGEFILEFORMATA, (void *)&ti)) return false;
	*fileformat = ti.fileformat;
	return true;
}

#ifdef __cplusplus
/******************************************************************************/
/** Tests if a file is an image file.
 *
 * @param handle Handle of the module.
 * @param filename Name of the file to test.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformat(gxHANDLE handle, const char *filename,
									gxi32 *fileformat) {
	return gx_testimagefileformata(handle, filename, fileformat);
}
#endif

/******************************************************************************/
/** Tests if a memory area contains an image file.
 *
 * @param handle Handle of the module.
 * @param buffer Pointer to the memory area.
 * @param buflen Size of the memory area.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformatmem(gxHANDLE handle, const void *buffer, gxi32 buflen,
									gxi32 *fileformat) {

	struct GX_PARM_TESTIMAGEFILEFORMATMEM ti;
	assert(fileformat);
	ti.buffer = buffer;
	ti.buflen = buflen;
	if(!gx_call(handle, GX_CALL_TESTIMAGEFILEFORMATMEM, (void *)&ti)) return false;
	*fileformat = ti.fileformat;
	return true;
}

/******************************************************************************/
/** Tests if a file is an image file.
 *
 * @param handle Handle of the module.
 * @param filename Name of the file to test.
 * @param formatid Buffer for the formatid. "" is returned for unknown file type.
 * @param slen Buffer length for formatid.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformatid(gxHANDLE handle, const wchar_t *filename,
									wchar_t *formatid, gxi32 slen) {

	struct GX_PARM_TESTIMAGEFILEFORMATID ti;
	assert(formatid);
	ti.filename = filename;
	ti.formatid = formatid;
	ti.slen = slen;
	if(!gx_call(handle, GX_CALL_TESTIMAGEFILEFORMATID, (void *)&ti)) return false;
	return true;
}

/******************************************************************************/
/** Tests if a memory area contains an image file.
 *
 * @param handle Handle of the module.
 * @param buffer Pointer to the memory area.
 * @param buflen Size of the memory area.
 * @param formatid Buffer for the formatid. "" is returned for unknown file type.
 * @param slen Buffer length for formatid.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformatidmem(gxHANDLE handle, const void *buffer, gxi32 buflen,
									wchar_t *formatid, gxi32 slen) {

	struct GX_PARM_TESTIMAGEFILEFORMATIDMEM ti;
	assert(formatid);
	ti.buffer = buffer;
	ti.buflen = buflen;
	ti.formatid = formatid;
	ti.slen = slen;
	if(!gx_call(handle, GX_CALL_TESTIMAGEFILEFORMATIDMEM, (void *)&ti)) return false;
	return true;
}

#ifdef __cplusplus
/******************************************************************************/
/** Tests if a memory area contains an image file.
 *
 * @param handle Handle of the module.
 * @param buffer Pointer to the memory area.
 * @param buflen Size of the memory area.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformat(gxHANDLE handle, const void *buffer, gxi32 buflen,
									gxi32 *fileformat) {
	return gx_testimagefileformatmem(handle, buffer, buflen, fileformat);
}
#endif

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_variant
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Embedds an image into a variant.
 * @param handle	Handle of the module.
 * @param source	Pointer to the source image.
 * @param varid		The id for the created variant.
 * @param target	Pointer to the initialized destination variant.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_imagetovariant(gxHANDLE handle, gxIMAGE *source, gxi32 varid, gxVARIANT *target) {
	struct GX_PARM_IMAGETOVARIANT itv;
	gxi32 ret;
	assert(target);
	assert(source);
	itv.pimage = source;
	itv.varid = varid;
	ret = gx_call(handle, GX_CALL_IMAGETOVARIANT, &itv);
	if(ret && target) *target = itv.variant;
	return ret;
}

/******************************************************************************/
/** Loads an embedded image from a variant.
 * @param handle	Handle of the module.
 * @param target	Pointer to the initialized destination image.
 * @param source	Variant with an embedded image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_imagefromvariant(gxHANDLE handle, gxIMAGE **target, gxVARIANT source) {
	struct GX_PARM_IMAGEFROMVARIANT ifv;
	gxi32 ret;
	assert(target);
	assert(source);
	ifv.variant = source;
	ret = gx_call(handle, GX_CALL_IMAGEFROMVARIANT, &ifv);
	if(ret && target) *target = ifv.pimage;
	return ret;
}

/******************************************************************************/
/** @} */
/******************************************************************************/

/******************************************************************************/
/** Returns the pointer of the pixel.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @param x Horizontal position of the pixel.
 * @param y Vertical position of the pixel.
 * @return The pointer of the pixel or NULL if an error occurred
 * or the pixel is not found. */
inline void *gx_getpixelptr(gxHANDLE handle, gxIMAGE *pimage, gxi32 x, gxi32 y) {
	struct GX_PARM_GETPIXELPTR gp;
	assert(pimage);
	gp.pimage = pimage;
	gp.x = x;
	gp.y = y;
	if(!gx_call(handle, GX_CALL_GETPIXELPTR, (void *)&gp)) return (void *)0;
	return gp.ptr;
}

/******************************************************************************/
#ifdef __cplusplus	/*(*/
/******************************************************************************/
/** Gets size of a pixel in bytes.
 * @param size Size of the pixel format.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_getpixelsize(gxi32 *size, gxi32 format) {
	return gx_getpixelsize(gx_direct(GX_CALL_GROUP_GX_IMAGE), size, format);
}

/******************************************************************************/
/** Converts pixels to a specified format and size.
 * @param target Pointer to a pixel data to where the function stores the data.
 * @param tformat Pixel format of the target area (see \ref GX_PIXELFORMATS).
 * @param tsline Size of a row of the target area in bytes.
 * @param source Pointer to a pixel data.
 * @param sformat Pixel format of the source area (see \ref GX_PIXELFORMATS).
 * @param ssline Size of a row of the source area in bytes.
 * @param xsize Horizontal size of the pixel area.
 * @param ysize Vertical size of the pixel area.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_convertpixels(void *target, gxi32 tformat, gxi32 tsline,
					const void *source, gxi32 sformat, gxi32 ssline,
					gxi32 xsize, gxi32 ysize) {

	return gx_convertpixels(gx_direct(GX_CALL_GROUP_GX_IMAGE),
					target, tformat, tsline,
					source, sformat, ssline, xsize, ysize);
}

/******************************************************************************/
/** Fast converts pixels to a specified format and size.
 *
 * The function is the same as gx_convertpixels(), only it works same memory area.
 * \par
 * The fast reconvert function can be used for converting:
 * - RGB555,BGR555,RGB565,BGR565 \<-\> RGB555,BGR555,RGB565,BGR565
 * - RGB,BGR \<-\> RGB,BGR
 * - RGBA,BGRA \<-\> RGBA,BGRA
 * - RGB12,BGR12 \<-\> RGB12,BGR12
 * @param pixels Pointer to a pixel data.
 * @param tformat Pixel format of the destination image (see \ref GX_PIXELFORMATS).
 * @param sformat Pixel format of the source image (see \ref GX_PIXELFORMATS).
 * @param sline Size of a row of the pixel area in bytes.
 * @param xsize Horizontal size of the pixel area.
 * @param ysize Vertical size of the pixel area.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_fastreconvert(void *pixels, gxi32 tformat, gxi32 sformat,
						gxi32 sline, gxi32 xsize, gxi32 ysize) {

	return gx_fastreconvert(gx_direct(GX_CALL_GROUP_GX_IMAGE),
					pixels, tformat, sformat,
					sline, xsize, ysize);
}

/******************************************************************************/
/** Zooms pixels to a specified size.
 * @param target Pointer to an allocated area where the zoom function places the
 *               converted image.
 * @param tsline Size of a row of the destination area.
 * @param txsize Horizontal size of the destination area.
 * @param tysize Vertical size of the destination area.
 * @param source Pointer to pixels of source area.
 * @param ssline Size of a row of the source area.
 * @param sxsize Horizontal size of the source area.
 * @param sysize Vertical size of the source area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param sx1_8  Left coordinate of zoom area (shifted left with 8 bit = 256)
 * @param sy1_8  Top coordinate of zoom area (shifted left with 8 bit = 256)
 * @param sx2_8  Right coordinate of zoom area (shifted left with 8 bit = 256)
 * @param sy2_8  Bottom coordinate of zoom area (shifted left with 8 bit = 256)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_zoompixels(void *target, gxi32 tsline, gxi32 txsize, gxi32 tysize,
						const void *source, gxi32 ssline, gxi32 sxsize, gxi32 sysize,
						gxi32 format,
						gxi32 sx1_8 = 0, gxi32 sy1_8 = 0,
						gxi32 sx2_8 = 0, gxi32 sy2_8 = 0) {

	return gx_zoompixels(gx_direct(GX_CALL_GROUP_GX_IMAGE),
					target, tsline, txsize, tysize,
					source, ssline, sxsize, sysize, format,
					sx1_8, sy1_8, sx2_8, sy2_8);
}

/******************************************************************************/
/** Rotates pixels to a specified degree.
 * @param target Pointer to an allocated area where the rotate function places
 *               the rotated image.
 * @param tsline Size of a row of the destination area.
 * @param txsize Horizontal size of the destination area.
 * @param tysize Vertical size of the destination area.
 * @param source Pointer to pixels of source area.
 * @param ssline Size of a row of the source area.
 * @param sxsize Horizontal size of the source area.
 * @param sysize Vertical size of the source area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param degree Degree (Must be 0, 90, 180, 270).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_rotatepixels(void *target, gxi32 tsline, gxi32 txsize, gxi32 tysize,
						const void *source, gxi32 ssline, gxi32 sxsize, gxi32 sysize,
						gxi32 format, gxi32 degree) {

	return gx_rotatepixels(gx_direct(GX_CALL_GROUP_GX_IMAGE),
						target, tsline, txsize, tysize,
						source, ssline, sxsize, sysize,
						format, degree);
}

/******************************************************************************/
/** Mirrors pixels to a specified mode.
 * @param target Pointer to an allocated area where the mirror function places
 *               the mirrored image.
 * @param tsline Size of a row of the destination area.
 * @param txsize Horizontal size of the destination area.
 * @param tysize Vertical size of the destination area.
 * @param source Pointer to pixels of source area.
 * @param ssline Size of a row of the source area.
 * @param sxsize Horizontal size of the source area.
 * @param sysize Vertical size of the source area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param flags  Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and GX_MIRROR_VERTICAL.
 *               (see \ref GX_MIRRORFLAGS)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorpixels(void *target, gxi32 tsline, gxi32 txsize, gxi32 tysize,
						const void *source, gxi32 ssline, gxi32 sxsize, gxi32 sysize,
						gxi32 format, gxi32 flags) {

	return gx_mirrorpixels(gx_direct(GX_CALL_GROUP_GX_IMAGE),
						target, tsline, txsize, tysize,
						source, ssline, sxsize, sysize,
						format, flags);
}

/******************************************************************************/
/** Mirrors pixels to a specified mode.
 * @param pixels Pointer to pixels of the pixel area.
 * @param sline  Size of a row of the pixel area.
 * @param xsize  Horizontal size of the pixel area.
 * @param ysize  Vertical size of the pixel area.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param flags  Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and GX_MIRROR_VERTICAL.
 *               (see \ref GX_MIRRORFLAGS)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorpixels(void *pixels, gxi32 sline, gxi32 xsize, gxi32 ysize,
						gxi32 format, gxi32 flags) {

	return gx_mirrorpixels(pixels, sline, xsize, ysize,
						pixels, sline, xsize, ysize,
						format, flags);
}

/******************************************************************************/
/** \addtogroup gx_group_image
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Initialization of an image located on a heap or a stack.
 *
 * This function fills the structure identifier and the reference
 * counter properly. The other data members are set to zero value.
 * \warning Do not call it for a dynamic allocated image (gx_allocimage)!
 * @param pimage Pointer to the ininitialized image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_initimage(gxIMAGE *pimage) {
	return gx_initimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage);
}

/******************************************************************************/
/** This function allocates a blank image.
 *
 * The structure identifier and the reference counter will be properly filled.
 * The other data members are set zero value.
 * @param pimage Pointer to a \a gxIMAGE structure pointer where the function
 *               returns the pointer of the allocated image area.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_allocimage(gxIMAGE **pimage) {
	return gx_allocimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage);
}

/******************************************************************************/
/** Frees an image structure including the allocated pixel data.
 *
 * If the value of the reference counter is not properly set the function returns an
 * error. In case of dynamically allocated images it is advised to use
 * \a gx_unrefimage.
 * @param pimage Pointer to the image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_freeimage(gxIMAGE *pimage) {
	return gx_freeimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage);
}

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_reference
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Increases the image reference counter.
 *
 * The counter indicates the number of processes using the image. The counter
 * can rise to positive or negative direction as well depending on whether the image
 * is static or dynamic.
 * @param pimage Pointer to the image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_refimage(gxIMAGE *pimage) {
	return gx_refimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage);
}

/******************************************************************************/
/** Decreases the image reference counter.
 *
 * The counter indicates the number of processes using the image. In case of
 * the counter value becomes zero after the function call the structure will be
 * freed depending on whether the image was allocated statically or dynamically.
 * @param pimage Pointer to the image structure.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_unrefimage(gxIMAGE *pimage) {
	return gx_unrefimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage);
}

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_image
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Makes an image with proper size and format running on an initialized image
 * structure.
 * @param pimage Pointer to the image structure.
 * @param format Pixel format (see \ref GX_PIXELFORMATS).
 * @param xsize Horizontal size of image in pixels.
 * @param ysize Vertical size of image in pixels.
 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
 *         bytes). If zero the system computes the correct value.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_createimage(gxIMAGE* pimage, gxi32 format,
							gxi32 xsize, gxi32 ysize, gxi32 sline) {
	return gx_createimage(gx_direct(GX_CALL_GROUP_GX_IMAGE),
						  pimage, format, xsize, ysize, sline);
}

/******************************************************************************/
/** Converts an image to a specific format and size. The destination image must
 * be initialized.
 * @param pdest Pointer to an initialized image where the function places the
 *              converted image.
 * @param psrc Pointer to the source image.
 * @param format Pixel format of the destination image (see \ref GX_PIXELFORMATS).
 * @param xsize Horizontal size of the destination image in pixels.
 * @param ysize Vertical size of the destination image in pixels.
 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
 *         bytes). If zero the system computes the correct value.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_convertimage(gxIMAGE* pdest, gxIMAGE* psrc,
						gxi32 format, gxi32 xsize, gxi32 ysize, gxi32 sline) {

	return gx_convertimage(gx_direct(GX_CALL_GROUP_GX_IMAGE),
						pdest, psrc, format, xsize, ysize, sline);
}

/******************************************************************************/
/** Zooms an image to a specified size.
 * @param pdest Pointer to an initialized image where the function places the
 *              zoomed image.
 * @param xsize Horizontal size of the destination area.
 * @param ysize Vertical size of the destination area.
 * @param sline Size of a row of the destination area.
 * @param psrc  Pointer to the source image.
 * @param sx1_8 Left coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @param sy1_8 Top coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @param sx2_8 Right coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @param sy2_8 Bottom coordinate of the zoomed area (shifted left with 8 bit = 256)
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_zoomimage(gxIMAGE *pdest,
					gxi32 xsize, gxi32 ysize, gxi32 sline,
					gxIMAGE *psrc,
					gxi32 sx1_8 = 0, gxi32 sy1_8 = 0,
					gxi32 sx2_8 = 0, gxi32 sy2_8 = 0) {

	return gx_zoomimage(gx_direct(GX_CALL_GROUP_GX_IMAGE),
					pdest, xsize, ysize, sline,
					psrc, sx1_8, sy1_8, sx2_8, sy2_8);
}

/******************************************************************************/
/** Rotates an image to a specified degree.
 * @param pdest Pointer to an initialized image where the function places the
 *              zoomed image.
 * @param psrc  Pointer to the source image.
 * @param degree Degree (Must be 0, 90, 180, 270).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_rotateimage(gxIMAGE *pdest, gxIMAGE *psrc, gxi32 degree) {
	return gx_rotateimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pdest, psrc, degree);
}

/******************************************************************************/
/** Mirrors an image to a specified mode.
 * @param pdest Pointer to an initialized image where the function places the
 *              zoomed image.
 * @param psrc  Pointer to the source image.
 * @param flags Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and
 *              GX_MIRROR_VERTICAL (see \ref GX_MIRRORFLAGS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorimage(gxIMAGE *pdest, gxIMAGE *psrc, gxi32 flags) {
	return gx_mirrorimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pdest, psrc, flags);
}

/******************************************************************************/
/** Mirrors an image to a specified mode.
 * @param pimage Pointer to an image.
 * @param flags Mirror flags. It can be GX_MIRROR_HORIZONTAL or/and
 *              GX_MIRROR_VERTICAL (see \ref GX_MIRRORFLAGS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_mirrorimage(gxIMAGE *pimage, gxi32 flags) {
	return gx_mirrorimage(pimage, pimage, flags);
}

/******************************************************************************/
/** Loads an image from a mass storage device and converts it to a specific
 * pixel format (Unicode version).
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included).  The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 *
 * \code
   gxIMAGE image;
   gx_initimage(&image);
   gx_createimage(&image, GX_GRAY, 1024, 768, 1024);
   gx_loadimage(&image, "test.raw", -1);
   ...
   \endcode
 *
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param filename Name of the image file.
 * @param pixelformat The specific pixel format.  Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimage(gxIMAGE* pimage, const wchar_t* filename,
						gxi32 pixelformat GX_DEFARG(GX_UNDEF)) {

	return gx_loadimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage,
					filename, pixelformat);
}

/******************************************************************************/
/** Loads an image from a mass storage device and converts it to a specific
 * pixel format (ASCII version).
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included).  The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 *
 * \code
   gxIMAGE image;
   gx_initimage(&image);
   gx_createimage(&image, GX_GRAY, 1024, 768, 1024);
   gx_loadimage(&image, "test.raw", -1);
   ...
   \endcode
 *
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param filename Name of the image file.
 * @param pixelformat The specific pixel format.  Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimage(gxIMAGE* pimage, const char* filename,
						gxi32 pixelformat GX_DEFARG(GX_UNDEF)) {

	return gx_loadimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage,
					filename, pixelformat);
}

/******************************************************************************/
/** Saves an image to a mass storage device in the specific file format
 * (Unicode version).
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimage(gxIMAGE* pimage,
						const wchar_t* filename, gxi32 fileformat) {

	return gx_saveimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage, filename, fileformat);
}

/******************************************************************************/
/** Saves an image to a mass storage device in the specific file format
 * (ASCII version).
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimage(gxIMAGE* pimage,
						const char* filename, gxi32 fileformat) {

	return gx_saveimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage, filename, fileformat);
}

/******************************************************************************/
/** Loads an image from the memory and converts it to a specific pixel format.
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included).  The format parameter of the
 * gx_loadfrommem() must be -1 for RAW images.
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param buffer Pointer to the image data.
 * @param buflen Length of image data in bytes.
 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadfrommem(gxIMAGE* pimage,
					const void *buffer,	gxi32 buflen, gxi32 pixelformat GX_DEFARG(GX_UNDEF)) {

	return gx_loadfrommem(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage, buffer,
						buflen, pixelformat);
}

/******************************************************************************/
/** Saves an image in a specific file format to a specific memory area.
 * @param pimage Pointer to the image.
 * @param buffer Pointer to the memory area pointer that is large enough or NULL.
 * @param buflen Size of the memory area.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_savetomem(gxIMAGE* pimage,
					void **buffer, gxi32 *buflen, gxi32 fileformat) {

	return gx_savetomem(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage, buffer, buflen, fileformat);
}

/******************************************************************************/
/** Checks if image structure is valid.
 * @param pimage Pointer to the image.
 * @return In case of valid image the return value is non-zero. */
inline gxi32 gx_isvalidimage(const gxIMAGE *pimage) {

	return gx_isvalidimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage);
}

/******************************************************************************/
#ifdef WIN32	/*(*/
/** Displays an image.
 * @param hdc Handle of device context.
 * @param x   X1 coordinate of display.
 * @param y   Y1 coordinate of display.
 * @param xsize Displayed horizontal size of image.
 * @param ysize Displayed vertical size of image.
 * @param pimage Pointer to the image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_displayimage(HDC hdc, gxi32 x, gxi32 y,
							gxi32 xsize, gxi32 ysize, gxIMAGE *pimage) {

	return gx_displayimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), hdc, x, y, xsize, ysize, pimage);
}

/******************************************************************************/
/** Copies an image to the clipboard.
 * @param pimage Pointer to the image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_imgtoclipboard(gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_IMGTOCLIPBOARD, (void *)pimage);
}

/******************************************************************************/
/** Inserts an image from the clipboard.
 * @param pimage Pointer to the image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_pasteimage(gxIMAGE *pimage) {
	assert(pimage);
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_PASTEIMAGE, (void *)pimage);
}
#endif	/*)*/

/******************************************************************************/
/** Copies an image to another.
 * @param target Pointer to the initialized destination image.
 * @param source Pointer to the source image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_copyimage(gxIMAGE *target, gxIMAGE *source) {
	return gx_copyimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), target, source);
}

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_variant
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Embedds an image into a variant.
 * @param source	Pointer to the source image.
 * @param varid		The id for the created variant.
 * @param target	Pointer to the initialized destination variant.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_imagetovariant(gxIMAGE *source, gxi32 varid, gxVARIANT *target) {
	return gx_imagetovariant(gx_direct(GX_CALL_GROUP_GX_IMAGE), source, varid, target);
}

/******************************************************************************/
/** Loads an embedded image from a variant.
 * @param target	Pointer to the initialized destination image.
 * @param source	Variant with an embedded image.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_imagefromvariant(gxIMAGE **target, gxVARIANT source) {
	return gx_imagefromvariant(gx_direct(GX_CALL_GROUP_GX_IMAGE), target, source);
}

/******************************************************************************/
/** @} */
/******************************************************************************/
/** \addtogroup gx_group_image
 * @{
 */
/******************************************************************************/

/******************************************************************************/
/** Loads an image with comments from a mass storage device and converts it to
 * a specific pixel format (Unicode version).
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included).  The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 *
 * \code
   gxIMAGE image;
   gx_initimage(&image);
   gx_createimage(&image, GX_GRAY, 1024, 768, 1024);
   gx_loadimage(&image, "test.raw", -1);
   ...
   \endcode
 *
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param filename Name of the image file.
 * @param pixelformat The specific pixel format.  Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @param properties Properties for the image loader. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimagefromfile(gxIMAGE* pimage, const wchar_t* filename,
						gxi32 pixelformat GX_DEFARG(GX_UNDEF),
						gxVARIANT properties GX_DEFARG(0),
						gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_LOADIMAGEFROMFILE li;
	assert(pimage);
	assert(filename);
	li.pimage = pimage;
	li.filename = filename;
	li.format = pixelformat;
	li.properties = properties;
	li.comments = comments;
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_LOADIMAGEFROMFILE, (void *)&li);
}

/******************************************************************************/
/** Loads an image with comments from a mass storage device and converts it to
 * a specific pixel format (ASCII version).
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included).  The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 *
 * \code
   gxIMAGE image;
   gx_initimage(&image);
   gx_createimage(&image, GX_GRAY, 1024, 768, 1024);
   gx_loadimage(&image, "test.raw", -1);
   ...
   \endcode
 *
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param filename Name of the image file.
 * @param pixelformat The specific pixel format.  Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @param properties Properties for the image loader. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimagefromfile(gxIMAGE* pimage, const char* filename,
						gxi32 pixelformat GX_DEFARG(GX_UNDEF),
						gxVARIANT properties GX_DEFARG(0),
						gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_LOADIMAGEFROMFILEA li;
	assert(pimage);
	assert(filename);
	li.pimage = pimage;
	li.filename = filename;
	li.format = pixelformat;
	li.properties = properties;
	li.comments = comments;
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_LOADIMAGEFROMFILEA, (void *)&li);
}

/******************************************************************************/
/** Saves an image with comments to a mass storage device in the specific file
 * format (Unicode version).
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @param properties Properties for the image writer. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimagetofile(gxIMAGE* pimage,
						const wchar_t* filename, gxi32 fileformat,
						gxVARIANT properties GX_DEFARG(0),
						gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_SAVEIMAGETOFILE si;
	assert(pimage);
	assert(filename);
	si.pimage = pimage;
	si.filename = filename;
	si.fileformat = fileformat;
	si.properties = properties;
	si.comments = comments;
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_SAVEIMAGETOFILE, (void *)&si);
}

/******************************************************************************/
/** Saves an image with comments to a mass storage device in the specific file
 * format (ASCII version).
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @param properties Properties for the image writer. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimagetofile(gxIMAGE* pimage,
						const char* filename, gxi32 fileformat,
						gxVARIANT properties GX_DEFARG(0),
						gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_SAVEIMAGETOFILEA si;
	assert(pimage);
	assert(filename);
	si.pimage = pimage;
	si.filename = filename;
	si.fileformat = fileformat;
	si.properties = properties;
	si.comments = comments;
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_SAVEIMAGETOFILEA, (void *)&si);
}

/******************************************************************************/
/** Saves an image with comments to a mass storage device in the specific file
 * format (Unicode version).
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @param filename Name of the image file.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @param properties Properties for the image writer. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimagetofile2(gxHANDLE handle, gxIMAGE* pimage,
						const wchar_t* filename, const wchar_t* fileformat,
						gxVARIANT properties GX_DEFARG(0),
						gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_SAVEIMAGETOFILE2 si;
	assert(pimage);
	assert(filename);
	si.pimage = pimage;
	si.filename = filename;
	si.fileformat = fileformat;
	si.properties = properties;
	si.comments = comments;
	return gx_call(handle, GX_CALL_SAVEIMAGETOFILE2, (void *)&si);
}

/******************************************************************************/
/** Loads an image with comments from the memory and converts it to a specific
 * pixel format.
 *
 * On loading a RAW image, the gxIMAGE must be created with correct
 * parameters (sline parameter also included).  The format parameter of the
 * gx_loadimage() must be -1 for RAW images.
 * @param pimage Pointer to an initialized image structure where the function
 *               loads the data.
 * @param buffer Pointer to the image data.
 * @param buflen Length of image data in bytes.
 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
 *               image to grayscale (see \ref GX_PIXELFORMATS).
 * @param properties Properties for the image loader. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_loadimagefrommem(gxIMAGE* pimage,
					const void *buffer,	gxi32 buflen, gxi32 pixelformat GX_DEFARG(GX_UNDEF),
					gxVARIANT properties GX_DEFARG(0),
					gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_LOADIMAGEFROMMEM li;
	assert(pimage);
	li.pimage = pimage;
	li.buffer = buffer;
	li.buflen = buflen;
	li.format = pixelformat;
	li.properties = properties;
	li.comments = comments;
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_LOADIMAGEFROMMEM, (void *)&li);
}

/******************************************************************************/
/** Saves an image with comments in a specific file format to a specific memory
 * area.
 * @param pimage Pointer to the image.
 * @param buffer Pointer to the memory area pointer that is large enough or NULL.
 * @param buflen Size of the memory area.
 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
 * @param properties Properties for the image writer. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimagetomem(gxIMAGE* pimage,
					void **buffer, gxi32 *buflen, gxi32 fileformat,
					gxVARIANT properties GX_DEFARG(0),
					gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_SAVEIMAGETOMEM si;
	assert(pimage);
	si.pimage = pimage;
	si.buffer = buffer;
	si.buflen = buflen;
	si.fileformat = fileformat;
	si.properties = properties;
	si.comments = comments;
	return gx_call(gx_direct(GX_CALL_GROUP_GX_IMAGE), GX_CALL_SAVEIMAGETOMEM, (void *)&si);
}

/******************************************************************************/
/** Saves an image with comments in a specific file format to a specific memory
 * area.
 * @param handle Handle of the module.
 * @param pimage Pointer to the image.
 * @param buffer Pointer to the memory area pointer that is large enough or NULL.
 * @param buflen Size of the memory area.
 * @param fileformat Format of the image file.
 * @param properties Properties for the image writer. It can be NULL.
 * @param comments Comments of the image. It can be NULL.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_saveimagetomem2(gxHANDLE handle, gxIMAGE* pimage,
					void **buffer, gxi32 *buflen, wchar_t* fileformat,
					gxVARIANT properties GX_DEFARG(0),
					gxVARIANT comments GX_DEFARG(0)) {

	struct GX_PARM_SAVEIMAGETOMEM2 si;
	assert(pimage);
	si.pimage = pimage;
	si.buffer = buffer;
	si.buflen = buflen;
	si.fileformat = fileformat;
	si.properties = properties;
	si.comments = comments;
	return gx_call(handle, GX_CALL_SAVEIMAGETOMEM2, (void *)&si);
}

/******************************************************************************/
/** Tests if a file is an image file.
 *
 * @param filename Name of the file to test.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformat(const wchar_t *filename, gxi32 *fileformat) {

	return gx_testimagefileformat(gx_direct(GX_CALL_GROUP_GX_IMAGE), filename, fileformat);
}

/******************************************************************************/
/** Tests if a file is an image file.
 *
 * @param filename Name of the file to test.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformat(const char *filename, gxi32 *fileformat) {

	return gx_testimagefileformat(gx_direct(GX_CALL_GROUP_GX_IMAGE), filename, fileformat);
}

/******************************************************************************/
/** Tests if a memory area contains an image file.
 *
 * @param buffer Pointer to the memory area.
 * @param buflen Size of the memory area.
 * @param fileformat Buffer for the fileformat. -1 is returned for unknown file type.
 * @return On error false is returned and the error code/string in the GX system
 *         is set appropriately ( see gx_geterror() ). */
inline gxi32 gx_testimagefileformat(const void *buffer, gxi32 buflen, gxi32 *fileformat) {

	return gx_testimagefileformat(gx_direct(GX_CALL_GROUP_GX_IMAGE), buffer, buflen, fileformat);
}

/******************************************************************************/
/** @} */
/******************************************************************************/

/******************************************************************************/
/** Returns the pointer of the pixel.
 * @param pimage Pointer to the image.
 * @param x Horizontal position of the pixel.
 * @param y Vertical position of the pixel.
 * @return The pointer of the pixel or NULL if an error occurred
 * or the pixel is not found. */
inline void *gx_getpixelptr(gxIMAGE *pimage, gxi32 x, gxi32 y) {
	return gx_getpixelptr(gx_direct(GX_CALL_GROUP_GX_IMAGE), pimage, x, y);
}

#ifdef GX_NAMESPACES
	}
#endif

/******************************************************************************/
#endif	/*)*/
#endif	/*)*/

#endif
