#ifndef GXIMAGE_CPP_IFC_H
#define GXIMAGE_CPP_IFC_H

#include "gximage_c.h"

#ifdef GX_NAMESPACES
	namespace gx {
#endif

#ifdef GX_DOTNET
  #pragma managed
#endif

/******************************************************************************/
#if !defined(NO_GX_CLASSES)	/*(*/

#ifndef GX_EXT_MODULE	/*(*/

/******************************************************************************/
/** Class for the IMAGE data and some low-level functions.
 * \see gxImage
 */
GX_CLASS gxImageData
#ifdef GX_DOTNET
: public System::IDisposable
#endif
{

#if !defined(GX_SWIGIF) /*(*/

#if !defined(GX_DOTNET) && !defined(GX_MANUAL)
public:
	GX_INDIRECTION_NOGC(gxIMAGE) _image;
#endif

#if defined(GX_DOTNET)
#if !defined(GX_MANUAL)
private:
	GX_INDIRECTION_NOGC(gxIMAGE) _image;	/* Pointer to the allocated gxIMAGE structure. */
#endif

public:
	/** Returns the local image data pointer.
	 *
	 * \attention Do not use this function directly.
	 *
	 * @return Current value. */
	inline gxOutPtr _get_image(void) GX_CONST {
		return GX_PUTPTR(_image);
	}
	/** Modifies the local image data pointer.
	 *
	 * \attention Do not use this function directly.
	 *
	 * @param ptr	New value. */
	inline void _set_image(gxInPtr ptr) {
		_image = (GX_INDIRECTION_NOGC(gxIMAGE))ptr.ToPointer();
	}
#endif

#endif	/*)*/

public:
	/** Class initialization. */
	inline gxImageData() {
		_image = (GX_INDIRECTION_NOGC(gxIMAGE))GX_NULLPTR;
	}
	/** Class termination. */
	inline ~gxImageData() gxFuncThrowsError {
		_Dispose();
	}

#ifdef GX_DOTNET_20
	inline !gxImageData() gxFuncThrowsError {
		_Dispose();
	}
#else
	/** It disposes the class. */
	inline void Dispose(void) gxFuncThrowsError {
#ifdef GX_DOTNET
		System::GC::SuppressFinalize(this);//do not call it from a destructor explicitly!
#endif
		_Dispose();
		/** no base class */
	}
#endif

private:
	inline void _Dispose(void) {
		gxassert(!_image || (_image->ref == 0));
	}
public:


	/** It returns the reference pointer. @see gxIMAGE, gx_refimage, gx_unrefimage
	 * @return Reference pointer of the gxIMAGE structure */
	inline gxi32 ref(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->ref;
	}
#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** It returns the pointer to the image pixels. @see gxIMAGE
	 * @return Pointer to the image pixels. */
	inline void *image(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->image;
	}
#endif
	/** It returns the size of the image in bytes. @see gxIMAGE
	 * @return Size of the image in bytes. */
	inline gxi32 simage(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->simage;
	}
	/** It returns the pixel format of the image. @see gxIMAGE
	 * @return Pixel format of the image (see \ref GX_PIXELFORMATS). */
	inline gxi32 format(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->format;
	}
	/** It returns the horizontal size of the image in pixels. @see gxIMAGE
	 * @return Horizontal size of the image in pixels. */
	inline gxi32 xsize(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->xsize;
	}
	/** It returns the vertical size of the image in pixels. @see gxIMAGE
	 * @return Vertical size of the image in pixels. */
	inline gxi32 ysize(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->ysize;
	}
	/** It returns the size of a row in bytes. @see gxIMAGE
	 * @return Size of a row in bytes. */
	inline gxi32 sline(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->sline;
	}
	/** It returns the horizontal resolution of the image. @see gxIMAGE
	 * @return Horizontal resolution of the image. */
	inline gxi32 xres(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->xres;
	}
	/** It returns the vertical resolution of the image. @see gxIMAGE
	 * @return Vertical resolution of the image. */
	inline gxi32 yres(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->yres;
	}
	/** The function sets the horizontal resolution of the image. @see gxIMAGE
	 *
	 * @param _xres	Value to set. */
	inline void xres(gxi32 _xres) gxFuncThrowsError {
		gxassert(_image);
		_image->xres = _xres;
	}
	/** The function sets the vertical resolution of the image. @see gxIMAGE
	 *
	 * @param _yres	Value to set. */
	inline void yres(gxi32 _yres) gxFuncThrowsError {
		gxassert(_image);
		_image->yres = _yres;
	}

	/** It returns the reference pointer. @see gxIMAGE, gx_refimage, gx_unrefimage
	 * @return Reference pointer of the gxIMAGE structure */
	inline gxi32 GetRef(void) GX_CONST gxFuncThrowsError {
		return ref();
	}

	/** It returns the pointer to the image pixels. @see gxIMAGE
	 * @return Pointer to the image pixels. */
#if defined(GX_DOTNET)
	inline gxOutData GetPixels(void) GX_CONST gxFuncThrowsError {
		return GX_PUTDATA(_image->image, _image->simage);
	}
#elif defined(GX_SWIG)
	inline gxOutData GetPixels(void) GX_CONST gxFuncThrowsError {
		return GX_PUTDATA(image(), simage());
	}
#else
	inline void *GetPixels(void) GX_CONST gxFuncThrowsError {
		return image();
	}
#endif

	/** It returns size of the image in bytes. @see gxIMAGE
	 * @return Size of the image in bytes. */
	inline gxi32 GetSImage(void) GX_CONST gxFuncThrowsError {
		return simage();
	}
	/** It returns the pixel format of the image. @see gxIMAGE
	 * @return Pixel format of the image (see \ref GX_PIXELFORMATS). */
	inline gxi32 GetPixelFormat(void) GX_CONST gxFuncThrowsError {
		return format();
	}
	/** It returns the horizontal size of the image in pixels. @see gxIMAGE
	 * @return Horizontal size of the image in pixels. */
	inline gxi32 GetXSize(void) GX_CONST gxFuncThrowsError {
		return xsize();
	}
	/** It returns the vertical size of the image in pixels. @see gxIMAGE
	 * @return Vertical size of the image in pixels. */
	inline gxi32 GetYSize(void) GX_CONST gxFuncThrowsError {
		return ysize();
	}
	/** It returns the size of a row in bytes. @see gxIMAGE
	 * @return Size of a row in bytes. */
	inline gxi32 GetSLine(void) GX_CONST gxFuncThrowsError {
		return sline();
	}
	/** It returns the horizontal resolution of the image. @see gxIMAGE
	 * @return Horizontal resolution of the image. */
	inline gxi32 GetXRes(void) GX_CONST gxFuncThrowsError {
		return xres();
	}
	/** It returns the vertical resolution of the image. @see gxIMAGE
	 * @return Vertical resolution of the image. */
	inline gxi32 GetYRes(void) GX_CONST gxFuncThrowsError {
		return yres();
	}
	/** The function sets the horizontal resolution of the image. @see gxIMAGE
	 *
	 * @param _xres	Value to set.
	 * @return The function always returns true. */
	inline bool SetXRes(gxi32 _xres) gxFuncThrowsError {
		xres(_xres);
		return true;
	}
	/** The function sets the vertical resolution of the image. @see gxIMAGE
	 *
	 * @param _yres	Value to set.
	 * @return The function always returns true. */
	inline bool SetYRes(gxi32 _yres) gxFuncThrowsError {
		yres(_yres);
		return true;
	}

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Returns a pointer to the gxIMAGE structure.
     * @return Pointer to the gxIMAGE structure. */
	inline operator gxIMAGE *() GX_CONST {
		return _image;
	}
	/** Returns a reference to the gxIMAGE structure.
     * @return Reference to the gxIMAGE structure. */
	inline operator gxIMAGE &() GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return *_image;
	}
#endif
};

/******************************************************************************/
/** Class for the IMAGE functions. */
GX_CLASS gxImageHandler : public gxHandle
{
public:
	/** Class initialization with default properties. */
	inline gxImageHandler() gxFuncThrowsError {
#ifndef GX_DOTNET
		if(!gx_openmodule(this, L"gximage", L"default")) gxthrow;
#else
		GX_GETHANDLE(h);
		if(!gx_openmodule(&h, L"gximage", L"default")) gxthrow;
		_set_handle(h.handle);
#endif
	}

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)	/*(*/
	/** Class initialization with another image handler.
	 * @param src Pointer to another image handler. The reference of
	 *		this image handle will be increased. */
	inline gxImageHandler(const gxHANDLE *src) gxFuncThrowsError : gxHandle() {
		if(src) {
			handle = src->handle;
			if(!gx_refhandle(*this)) gxthrow;
		}
	}
	/** Class initialization with another image handle.
	 * @param src Source image handle. The reference of this
	 *		image handle will be increased. */
	inline gxImageHandler(const gxHANDLE &src) gxFuncThrowsError : gxHandle() {
		handle = src.handle;
		if(!gx_refhandle(*this)) gxthrow;
	}

	/** Class initialization with the place of the properties and name of the module
	 * (Unicode version).
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImageHandler(const wchar_t *propname,
				const wchar_t *modname = (const wchar_t *)0) gxFuncThrowsError {

		if(!gx_openmodule(this, modname ? modname : L"gximage", propname)) gxthrow;
	}

	/** Class initialization with the place of the properties and name of the module
	 * (ASCII version).
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImageHandler(const char *propname,
				const char *modname = (const char *)0) gxFuncThrowsError {

		if(!gx_openmodulea(this, modname ? modname : "gximage", propname)) gxthrow;
	}
#endif	/* GX_SWIGIF GX_DOTNET )*/

#ifdef GX_UNICODE	/*(*/
	/** Class initialization with place of the properties and name of the module.
	 * (Unicode version)
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImageHandler(gxInStr propname, gxInStr modname) gxFuncThrowsError {
		GX_GETSTRING(_propname, propname);
		GX_GETSTRING(_modname, modname);
#ifndef GX_DOTNET
		if(!gx_openmodule(this, _modname, _propname)) gxthrow;
#else
		GX_GETHANDLE(h);
		if(!gx_openmodule(&h, _modname, _propname)) gxthrow;
		_set_handle(h.handle);
#endif
	}
	/** Class initialization with place of the properties.
	 * (Unicode version)
	 * @param propname Place of the properties in the property tree (Name of a property group). */
	inline gxImageHandler(gxInStr propname) gxFuncThrowsError {
		GX_GETSTRING(_propname, propname);
#ifndef GX_DOTNET
		if(!gx_openmodule(this, L"gximage", _propname)) gxthrow;
#else
		GX_GETHANDLE(h);
		if(!gx_openmodule(&h, L"gximage", _propname)) gxthrow;
		_set_handle(h.handle);
#endif
	}
#endif	/* GX_UNICODE )*/

#ifdef GX_ASCII
	/** Class initialization with place of the properties and name of the module.
	 * (ASCII version)
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImageHandler(gxInAStr propname, gxInAStr modname) gxFuncThrowsError {
		GX_GETASTRING(_propname, propname);
		GX_GETASTRING(_modname, modname);
		if(!gx_openmodulea(this, _modname, _propname)) gxthrow;
	}
	/** Class initialization with place of the properties (ASCII version).
	 * @param propname Place of the properties in the property tree (Name of a property group). */
	inline gxImageHandler(gxInAStr propname) gxFuncThrowsError {
		GX_GETASTRING(_propname, propname);
		if(!gx_openmodulea(this, "gximage", _propname)) gxthrow;
	}
#endif

	/** Class initalization with another image handle.
	 * @param src Source image handle. The reference of this
	 *		image handle will be increased. */
	inline gxImageHandler(GX_CONST gxImageHandler GX_REF_CLASS(src)) gxFuncThrowsError : gxHandle() {
#ifndef GX_DOTNET
		handle = src.handle;
		if(!gx_refhandle(*this)) gxthrow;
#else
		gxHANDLE h = { src GX_M _get_handle() };
		if(!gx_refhandle(h)) gxthrow;
		_set_handle(h.handle);
#endif
	}

};

#endif	/* GX_EXT_MODULE )*/
/******************************************************************************/
#if !defined(GX_MANUAL)
#if !defined(GX_DOTNET)
  #define GX_GETIMAGE(name)			gxIMAGE *name = _image
  #define GX_GETIMAGE2(name,src)	gxIMAGE *name = (src)._image
#else
  #define GX_GETIMAGE(name)			gxIMAGE *name = (gxIMAGE *)_get_image().ToPointer()
  #define GX_GETIMAGE2(name,src)	gxIMAGE *name = (gxIMAGE *)((src) GX_M _get_image().ToPointer())
#endif
#endif

/******************************************************************************/

#ifndef GX_EXT_MODULE	/*(*/
/******************************************************************************/
/******************************************************************************/
/** Class for the IMAGE functions. */
#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)	/*(*/
class gxImage : public gxImageHandler, public gxImageData {
#else	/*)(*/
GX_CLASS gxImage : public gxImageHandler
#ifdef GX_DOTNET
, public System::IDisposable
#endif
{

#if !defined(GX_SWIGIF) /*(*/

#if !defined(GX_DOTNET) && !defined(GX_MANUAL)
public:
	GX_INDIRECTION_NOGC(gxIMAGE) _image;	/* Pointer to the allocated gxIMAGE structure. */
#endif

#if defined(GX_DOTNET)
#if !defined(GX_MANUAL)
private:
	GX_INDIRECTION_NOGC(gxIMAGE) _image;	/* Pointer to the allocated gxIMAGE structure. */
#endif

public:
	/** Returns the local image pointer.
	 *
	 * \attention Do not use this function directly.
	 *
	 * @return Current value. */
	inline gxOutPtr _get_image(void) GX_CONST {
		return GX_PUTPTR(_image);
	}
	/** Modifies the local image pointer.
	 *
	 * \attention Do not use this function directly.
	 *
	 * @param ptr	New value. */
	inline void _set_image(gxInPtr ptr) {
		_image = (GX_INDIRECTION_NOGC(gxIMAGE))GX_GETPTR(ptr);
	}
	/** Modifies the local image pointer.
		 * @param ptr	New value. */
private:
	inline void _set_image(GX_INDIRECTION_NOGC(gxIMAGE) ptr) {
		_image = ptr;
	}
public:
#endif
#endif /*)*/

public:
	/** Returns the reference pointer. @see gxIMAGE, gx_refimage, gx_unrefimage
	 * @return Reference pointer of the gxIMAGE structure */
	inline gxi32 ref(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->ref;
	}

	/** Returns the size of the image in bytes. @see gxIMAGE
	 * @return Size of the image in bytes. */
	inline gxi32 simage(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->simage;
	}
	/** Returns the pixel format of the image. @see gxIMAGE
	 * @return Pixel format of the image (see \ref GX_PIXELFORMATS). */
	inline gxi32 format(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->format;
	}
	/** Returns the horizontal size of the image in pixels. @see gxIMAGE
	 * @return Horizontal size of the image in pixels. */
	inline gxi32 xsize(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->xsize;
	}
	/** Returns the vertical size of the image in pixels. @see gxIMAGE
	 * @return Vertical size of the image in pixels. */
	inline gxi32 ysize(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->ysize;
	}
	/** Returns the size of a row in bytes. @see gxIMAGE
	 * @return Size of a row in bytes. */
	inline gxi32 sline(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->sline;
	}
	/** Returns the horizontal resolution of the image. @see gxIMAGE
	 * @return Horizontal resolution of the image. */
	inline gxi32 xres(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->xres;
	}
	/** Returns the vertical resolution of the image. @see gxIMAGE
	 * @return Vertical resolution of the image. */
	inline gxi32 yres(void) GX_CONST gxFuncThrowsError {
		gxassert(_image);
		return _image->yres;
	}
	/** Sets the horizontal resolution of the image. @see gxIMAGE
	 * @param _xres Horizontal resolution to set.*/
	inline void xres(gxi32 _xres) gxFuncThrowsError {
		gxassert(_image);
		_image->xres = _xres;
	}
	/** Sets the vertical resolution of the image. @see gxIMAGE
	 * @param _yres Vertical resolution to set.*/
	inline void yres(gxi32 _yres) gxFuncThrowsError {
		gxassert(_image);
		_image->yres = _yres;
	}

	/** Returns the reference pointer. @see gxIMAGE, gx_refimage, gx_unrefimage
	 * @return Reference pointer of the gxIMAGE structure */
	inline gxi32 GetRef(void) GX_CONST gxFuncThrowsError {
		return ref();
	}

	/** Returns the pointer to the image pixels. @see gxIMAGE
	 * @return Pointer to the image pixels. */
#if defined(GX_DOTNET)
	inline gxOutData GetPixels(void) GX_CONST gxFuncThrowsError {
		return GX_PUTDATA(_image->image, _image->simage);
	}
#elif defined(GX_SWIG)
	inline gxOutData GetPixels(void) GX_CONST gxFuncThrowsError {
		return GX_PUTDATA(image(), simage());
	}
#else
	inline void *GetPixels(void) GX_CONST gxFuncThrowsError {
		return image();
	}
#endif

	/** Returns size of the image in bytes. @see gxIMAGE
	 * @return Size of the image in bytes. */
	inline gxi32 GetSImage(void) GX_CONST gxFuncThrowsError {
		return simage();
	}
	/** Returns the pixel format of the image. @see gxIMAGE
	 * @return Pixel format of the image (see \ref GX_PIXELFORMATS). */
	inline gxi32 GetPixelFormat(void) GX_CONST gxFuncThrowsError {
		return format();
	}
	/** Returns the horizontal size of the image in pixels. @see gxIMAGE
	 * @return Horizontal size of the image in pixels. */
	inline gxi32 GetXSize(void) GX_CONST gxFuncThrowsError {
		return xsize();
	}
	/** Returns the vertical size of the image in pixels. @see gxIMAGE
	 * @return Vertical size of the image in pixels. */
	inline gxi32 GetYSize(void) GX_CONST gxFuncThrowsError {
		return ysize();
	}
	/** Returns the size of a row in bytes. @see gxIMAGE
	 * @return Size of a row in bytes. */
	inline gxi32 GetSLine(void) GX_CONST gxFuncThrowsError {
		return sline();
	}
	/** Returns the horizontal resolution of the image. @see gxIMAGE
	 * @return Horizontal resolution of the image. */
	inline gxi32 GetXRes(void) GX_CONST gxFuncThrowsError {
		return xres();
	}
	/** Returns the vertical resolution of the image. @see gxIMAGE
	 * @return Vertical resolution of the image. */
	inline gxi32 GetYRes(void) GX_CONST gxFuncThrowsError {
		return yres();
	}
	/** Sets the horizontal resolution of the image. @see gxIMAGE
	 * @param _xres Horizontal resolution to set.
	 * @return The function always returns true. */
	inline bool SetXRes(gxi32 _xres) gxFuncThrowsError {
		xres(_xres);
		return true;
	}
	/** Sets the vertical resolution of the image. @see gxIMAGE
	 * @param _yres Vertical resolution to set.
	 * @return The function always returns true. */
	inline bool SetYRes(gxi32 _yres) gxFuncThrowsError {
		yres(_yres);
		return true;
	}
#endif	/*)*/
public:
#if defined(GX_DOTNET) || defined(GX_SWIGIF)	/*(*/
	/** Class initialization with default properties. */
	inline gxImage() gxFuncThrowsError : gxImageHandler() {
		_image = 0;
		if(gxHandle::IsValid()) {
#ifndef GX_DOTNET
			if(!gx_allocimage(*this, &_image)) gxthrow;
#else
			GX_GETHANDLE(h);
			GX_GETIMAGE(i);
			if(!gx_allocimage(h, &i)) gxthrow;
			_set_image(i);
#endif
		}
	}
#endif	/*)*/

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)	/*(*/
	/** Class initialization with default properties. */
	inline gxImage() gxFuncThrowsError : gxImageHandler(), gxImageData() {
		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}

	/** Class initialization with a source image handle.
	 * @param h Source image handle. The reference of this
	 *		image handle will be increased. */
	inline gxImage(const gxHANDLE &h) gxFuncThrowsError : gxImageHandler(h), gxImageData() {
		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}
	/** Class initialization with a source image handle.
	 * @param h Source image handle. The reference of this
	 *		image handle will be increased. */
	inline gxImage(const gxHANDLE *h) gxFuncThrowsError : gxImageHandler(h), gxImageData() {
		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}

	/** Class initialization with the place of the properties and name of the module
	 * (Unicode version).
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImage(const wchar_t *propname,
				const wchar_t *modname = (const wchar_t *)0) gxFuncThrowsError
				: gxImageHandler(propname, modname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}

	/** Class initialization with the place of the properties and name of the module
	 * (ASCII version).
	 * @param propname Place of properties in property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImage(const char *propname,
				const char *modname = (const char *)0) gxFuncThrowsError
				: gxImageHandler(propname, modname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}

	/** Class initialization with another image structure.
	 * @param src Source image. */
	inline gxImage(gxIMAGE &src) gxFuncThrowsError : gxImageHandler(), gxImageData() {
		if(gxHandle::IsValid()) {
			if(gx_allocimage(*this, &_image)) {
				Copy(src);
			} else {
				gxthrow;
			}
		}
	}

	/** Class initialization with another image structure, the place of the properties
	 * and name of the module (Unicode version).
	 * @param src Source image.
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImage(gxIMAGE &src, const wchar_t *propname,
				const wchar_t *modname = (const wchar_t *)0) gxFuncThrowsError
				: gxImageHandler(propname, modname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(gx_allocimage(*this, &_image)) {
				Copy(src);
			} else {
				gxthrow;
			}
		}
	}

	/** Class initialization with another image structure, the place of the properties
	 * and name of the module (ASCII version).
	 * @param src Source image.
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImage(gxIMAGE &src, const char *propname,
				const char *modname = (const char *)0) gxFuncThrowsError
				: gxImageHandler(propname, modname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(gx_allocimage(*this, &_image)) {
				Copy(src);
			} else {
				gxthrow;
			}
		}
	}

	/** Class initialization with another image class, the place of the properties
	 * and name of the module (Unicode version).
	 * @param src Source image.
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImage(gxImage &src, const wchar_t *propname, const wchar_t *modname = (const wchar_t *)0)
				gxFuncThrowsError : gxImageHandler(propname, modname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(gx_allocimage(*this, &_image)) {
				Copy(src);
			} else {
				gxthrow;
			}
		}
	}

	/** Class initialization with another image class, the place of the properties
	 * and name of the module (ASCII version).
	 * @param src Source image.
	 * @param propname Place of the properties in the property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImage(gxImage &src, const char *propname,
				const char *modname = (const char *)0) gxFuncThrowsError
				: gxImageHandler(propname, modname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(gx_allocimage(*this, &_image)) {
				Copy(src);
			} else {
				gxthrow;
			}
		}
	}
#endif	/*)*/

#ifdef GX_UNICODE	/*(*/
	/** Class initialization with the place of the properties (Unicode version).
	 *
	 * @param propname Place of the properties in the property tree (Name of a property group). */
#ifndef GX_DOTNET
	inline gxImage(gxInStr propname) gxFuncThrowsError
				: gxImageHandler(propname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}
#else
	inline gxImage(gxInStr propname) gxFuncThrowsError
				: gxImageHandler(propname) {

		_set_image(0);
		if(gxHandle::IsValid()) {
			GX_GETHANDLE(h);
			GX_GETIMAGE(i);
			if(!gx_allocimage(h, &i)) gxthrow;
			_set_image(i);
		}
	}
#endif

	/** Class initialization with the place of the properties and name of the module (Unicode version).
	 *
	 * @param propname Place of properties in property tree (Name of a property group).
	 * @param modname Name of the module. */
#ifndef GX_DOTNET
	inline gxImage(gxInStr propname, gxInStr modname) gxFuncThrowsError
				: gxImageHandler(propname, modname), gxImageData() {
		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}
#else
	inline gxImage(gxInStr propname, gxInStr modname) gxFuncThrowsError
				: gxImageHandler(propname, modname) {

		_set_image(0);
		if(gxHandle::IsValid()) {
			GX_GETHANDLE(h);
			GX_GETIMAGE(i);
			if(!gx_allocimage(h, &i)) gxthrow;
			_set_image(i);
		}
	}
#endif

#endif	/*)*/

#ifdef GX_ASCII
	/** Class initialization with the place of the properties (ASCII version).
	 *
	 * @param propname Place of the properties in the property tree (Name of a property group). */
	inline gxImage(gxInAStr propname) gxFuncThrowsError
				: gxImageHandler(propname), gxImageData() {

		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}
	/** Class initialization with the place of the properties and name of the module (ASCII version).
	 *
	 * @param propname Place of properties in property tree (Name of a property group).
	 * @param modname Name of the module. */
	inline gxImage(gxInAStr propname, gxInAStr modname) gxFuncThrowsError
				: gxImageHandler(propname, modname), gxImageData() {
		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}
#endif

	/** Class initialization with previously initialized image handler.
	 *
	 * @param ih Previously intialized image handler. */
#ifndef GX_DOTNET
	inline gxImage(gxImageHandler &ih) gxFuncThrowsError : gxImageHandler(ih), gxImageData() {
		if(gxHandle::IsValid()) {
			if(!gx_allocimage(*this, &_image)) gxthrow;
		}
	}
#else

	inline gxImage(gxImageHandler GX_REF_CLASS(ih)) gxFuncThrowsError : gxImageHandler(ih) {

		_set_image(GX_NULLPTR);
		if(gxHandle::IsValid()) {
			GX_GETHANDLE(h);
			GX_GETIMAGE(i);
			if(!gx_allocimage(h, &i)) gxthrow;
			_set_image(i);

		}
	}
#endif

	/** Class initialization with previously initialized image.
	 *
	 * @param src Previously intialized image. */
#ifndef GX_DOTNET
	inline gxImage(gxImage &src) gxFuncThrowsError : gxImageHandler((const gxHANDLE &)src), gxImageData() {
		if(gxHandle::IsValid()) {
			if(gx_allocimage(*this, &_image)) {
				Copy(src);
			} else {
				gxthrow;
			}
		}
	}
#else
	inline gxImage(gxImage GX_REF_CLASS(src)) gxFuncThrowsError : gxImageHandler(src) {
		_set_image(0);
		if(gxHandle::IsValid()) {
			GX_GETHANDLE(h);
			GX_GETIMAGE(i);
			if(gx_allocimage(h, &i)) {
				_set_image(i);
				Copy(src);
			} else {
				gxthrow;
			}
		}
	}
#endif

	/** Class termination. */
	inline ~gxImage() gxFuncThrowsError {
		_Dispose();
	}

#ifdef GX_DOTNET_20
	inline !gxImage() gxFuncThrowsError {
		_Dispose();
	}
#else
	/** It disposes the class. */
	inline void Dispose(void) gxFuncThrowsError {
#ifdef GX_DOTNET
		System::GC::SuppressFinalize(this);//do not call it from a destructor explicitly!
#endif
		_Dispose();
		/** call the base class */
		gxHandle::Dispose();
	}
#endif

private:
inline void _Dispose(void) gxFuncThrowsError {
#ifndef GX_DOTNET
		if(_image) {
			if(!gx_unrefimage(*this, _image)) gxthrow;
		}
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		if(i) {
			if(!gx_unrefimage(h, i)) gxthrow;
		}
#endif
		_image=0;
	}

public:

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)	/*(*/
	/** Copies an image to the class.
	 *
	 * @param src	Source image structure.
	 * @return Returns a reference to the image class. */
	inline gxImage &operator = (gxIMAGE &src) gxFuncThrowsError {
		Copy(src);
		return *this;
	}

	/** Copies an image to the class.
	 *
	 * @param src	Source image class.
	 * @return Returns a reference to the image class. */
	inline gxImage &operator = (gxImage &src) gxFuncThrowsError {
		Copy(src);
		return *this;
	}

	/** Returns a reference to the gxIMAGE structure.
     * @return Reference to the gxIMAGE structure. */
	inline operator gxIMAGE & () gxFuncThrowsError {
		gxassert(_image);
		return *_image;
	}
	/** Returns a pointer to the gxIMAGE structure.
     * @return Pointer to the gxIMAGE structure. */
	inline operator gxIMAGE * () gxFuncThrowsError {
		gxassert(_image);
		return _image;
	}
	/** Returns a pointer to the gxIMAGE structure.
     * @return Pointer to the gxIMAGE structure. */
	inline operator const gxIMAGE * () const gxFuncThrowsError {
		gxassert(_image);
		return _image;
	}

	/** Attaches to an allocated image.
	 * @param src An allocated gxIMAGE structure.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Attach(gxIMAGE *src) gxFuncThrowsError {
		gxassert(_image && src);
		if(src->ref >= 0) {
			gx_seterror(GX_EINVAL, L"(src->ref < 0)");
			gxthrow;
			return false;
		}
		if(!gx_refimage(*this, src)) {
			gxthrow;
			return false;
		}
		if(!gx_unrefimage(*this, _image)) {
			gx_unrefimage(*this, src);
			gxthrow;
			return false;
		}
		_image = src;
		return true;
	}

	/** Detaches the image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Detach(void) gxFuncThrowsError {
		if(!gx_unrefimage(*this, _image)) {
			gxthrow;
			return false;
		}
		if(!gx_allocimage(*this, &_image)) {
			gxthrow;
			return false;
		}
		return true;
	}

	/** Gets the size of a pixel in bytes.
	 * @param size Size of the pixel format.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool GetPixelSize(gxi32 *size) const gxFuncThrowsError {
		if(!gx_getpixelsize(*this, size, format())) {
			throw gxError();
			return false;
		}
		return true;
	}

	/** Returns the pointer of the pixel.
	 * @param x Horizontal position of the pixel.
	 * @param y Vertical position of the pixel.
	 * @return The pointer of the pixel or NULL if an error occurred
	 * or the pixel is not found. */
	inline void *GetPixelPtr(gxi32 x, gxi32 y) gxFuncThrowsError {
		gxassert(_image);
		void *ret = gx_getpixelptr(*this, _image, x, y);
		gxcondthrow(!ret);
		return ret;
	}
#endif	/*)*/
	/** Returns the size of a current format pixel in bytes.
	 * @return Size of the pixel in bytes. */
	inline gxi32 GetPixelSize(void) GX_CONST gxFuncThrowsError {
		gxi32 size = -1;
		GX_GETHANDLE(h);
		if(!gx_getpixelsize(h, &size, format())) gxthrow;
		return size;
	}

	/** Makes an image with proper size and format.
	 * @param format Pixel format (see \ref GX_PIXELFORMATS).
	 * @param xsize Horizontal size of image in pixels.
	 * @param ysize Vertical size of image in pixels.
	 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
	 *         bytes). If zero the system computes the correct value.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Create(gxi32 format, gxi32 xsize, gxi32 ysize, gxi32 sline GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
		bool st = gx_createimage(h, _image, format, xsize, ysize, sline) ? true : false;
		gxcondthrow(!st);
		return st;
	}

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Converts an image to a specific format and size.
	 * @param src Pointer to the source image.
	 * @param format Pixel format of the destination image (see \ref GX_PIXELFORMATS).
	 * @param xsize Horizontal size of the destination image in pixels.
	 * @param ysize Vertical size of the destination image in pixels.
	 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
	 *         bytes). If zero the system computes the correct value.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Convert(gxIMAGE *src, gxi32 format, gxi32 xsize, gxi32 ysize, gxi32 sline GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_convertimage(*this, _image, src, format, xsize, ysize, sline) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif
	/** Converts an image to a specific format and size.
	 * @param src Reference to the source image data.
	 * @param format Pixel format of the destination image (see \ref GX_PIXELFORMATS).
	 * @param xsize Horizontal size of the destination image in pixels.
	 * @param ysize Vertical size of the destination image in pixels.
	 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
	 *         bytes). If zero the system computes the correct value.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Convert(gxImageData GX_REF_CLASS(src), gxi32 format, gxi32 xsize, gxi32 ysize, gxi32 sline GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
#ifndef GX_DOTNET
		bool st = gx_convertimage(h, _image, src._image, format, xsize, ysize, sline) ? true : false;
#else
		bool st = gx_convertimage(h, _image, (gxIMAGE *)src GX_M _get_image().ToPointer(), format, xsize, ysize, sline) ? true : false;
#endif
		gxcondthrow(!st);
		return st;
	}

	/** Converts an image to a specific format and size.
	 * @param src Pointer to the source image.
	 * @param format Pixel format of the destination image (see \ref GX_PIXELFORMATS).
	 * @param xsize Horizontal size of the destination image in pixels.
	 * @param ysize Vertical size of the destination image in pixels.
	 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
	 *         bytes). If zero the system computes the correct value.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Convert(gxImage GX_REF_CLASS(src), gxi32 format, gxi32 xsize, gxi32 ysize, gxi32 sline GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
		bool st = gx_convertimage(h, _image, src GX_M _image, format, xsize, ysize, sline) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Converts an image to a specific format and size.
	 * @param format Pixel format of the destination image (see \ref GX_PIXELFORMATS).
	 * @param xsize Horizontal size of the destination image in pixels.
	 * @param ysize Vertical size of the destination image in pixels.
	 * @param sline Length of a row in bytes (min.: xsize * size of a pixel in
	 *         bytes). If zero the system computes the correct value.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Convert(gxi32 format, gxi32 xsize, gxi32 ysize, gxi32 sline GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
		bool st = gx_convertimage(h, _image, _image, format, xsize, ysize, sline) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Converts an image to a specific format.
	 * @param format Pixel format of the destination image (see \ref GX_PIXELFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Convert(gxi32 format) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
		bool st = gx_convertimage(h, _image, _image, format, xsize(), ysize(), 0) ? true : false;
		gxcondthrow(!st);
		return st;
	}

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Copies an image to the class.
	 * @param src Pointer of the source image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Copy(gxIMAGE *src) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_copyimage(*this, _image, src) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Copies an image to the class.
	 * @param src Pointer of the source image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Copy(gxIMAGE &src) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_copyimage(*this, _image, &src) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Copies an image to the class.
	 * @param src Pointer of the source image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Copy(gxImage *src) gxFuncThrowsError {
		gxassert(src);
		return Copy((gxIMAGE *)(*src));
	}
#endif

	/** Copies an image to the class.
	 * @param src Reference of the source image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Copy(gxImageData GX_REF_CLASS(src)) gxFuncThrowsError {
#ifndef GX_DOTNET
		return Copy((gxIMAGE &)src);
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_copyimage(h, i, (gxIMAGE *)src GX_M _get_image().ToPointer()) ? true : false;
		gxcondthrow(!st);
		return st;
#endif
	}

	/** Copies an image to the class.
	 * @param src Pointer of the source image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Copy(gxImage GX_REF_CLASS(src)) gxFuncThrowsError {
#ifndef GX_DOTNET
		return Copy((gxIMAGE &)src);
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_copyimage(h, i, (gxIMAGE *)src GX_M _get_image().ToPointer()) ? true : false;
		gxcondthrow(!st);
		return st;
#endif
	}

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Zooms an image to a specified size.
	 * @param src   Pointer to the source image.
	 * @param xsize Horizontal size of the destination area.
	 * @param ysize Vertical size of the destination area.
	 * @param sline Size of a row of the destination area.
	 * @param sx1_8 Left coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sy1_8 Top coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sx2_8 Right coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sy2_8 Bottom coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Zoom(gxIMAGE *src, gxi32 xsize, gxi32 ysize, gxi32 sline,
					gxi32 sx1_8 GX_DEFARG(0), gxi32 sy1_8 GX_DEFARG(0),
					gxi32 sx2_8 GX_DEFARG(0), gxi32 sy2_8 GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_zoomimage(*this, _image, xsize, ysize, sline, src,
					sx1_8, sy1_8, sx2_8, sy2_8) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif

	/** Zooms an image to a specified size.
	 * @param src   Reference to the source image data.
	 * @param xsize Horizontal size of the destination area.
	 * @param ysize Vertical size of the destination area.
	 * @param sline Size of a row of the destination area.
	 * @param sx1_8 Left coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sy1_8 Top coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sx2_8 Right coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sy2_8 Bottom coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Zoom(gxImageData GX_REF_CLASS(src), gxi32 xsize, gxi32 ysize, gxi32 sline,
					gxi32 sx1_8 GX_DEFARG(0), gxi32 sy1_8 GX_DEFARG(0),
					gxi32 sx2_8 GX_DEFARG(0), gxi32 sy2_8 GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
#ifndef GX_DOTNET
		bool st = gx_zoomimage(*this, _image, xsize, ysize, sline, src,
					sx1_8, sy1_8, sx2_8, sy2_8) ? true : false;
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_zoomimage(h, i, xsize, ysize, sline,
					(gxIMAGE *)src GX_M _get_image().ToPointer(),
					sx1_8, sy1_8, sx2_8, sy2_8) ? true : false;
#endif
		gxcondthrow(!st);
		return st;
	}
#ifdef GX_DOTNET
	/** Zooms an image to a specified size.
	 * @param src   Reference to the source image data.
	 * @param xsize Horizontal size of the destination area.
	 * @param ysize Vertical size of the destination area.
	 * @param sline Size of a row of the destination area.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Zoom(gxImageData GX_REF_CLASS(src), gxi32 xsize, gxi32 ysize, gxi32 sline) gxFuncThrowsError {
			return false;
		//return Zoom(src, xsize, ysize, sline, 0,0,0,0);
	}
#endif

	/** Zooms an image to a specified size.
	 * @param src   Reference to the source image.
	 * @param xsize Horizontal size of the destination area.
	 * @param ysize Vertical size of the destination area.
	 * @param sline Size of a row of the destination area.
	 * @param sx1_8 Left coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sy1_8 Top coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sx2_8 Right coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @param sy2_8 Bottom coordinate of the zoomed area (shifted left with 8 bit = 256)
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Zoom(gxImage GX_REF_CLASS(src), gxi32 xsize, gxi32 ysize, gxi32 sline,
					gxi32 sx1_8 GX_DEFARG(0), gxi32 sy1_8 GX_DEFARG(0),
					gxi32 sx2_8 GX_DEFARG(0), gxi32 sy2_8 GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
#ifndef GX_DOTNET
		bool st = gx_zoomimage(*this, _image, xsize, ysize, sline, src,
					sx1_8, sy1_8, sx2_8, sy2_8) ? true : false;
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_zoomimage(h, i, xsize, ysize, sline,
					(gxIMAGE *)src GX_M _get_image().ToPointer(),
					sx1_8, sy1_8, sx2_8, sy2_8) ? true : false;
#endif
		gxcondthrow(!st);
		return st;
	}
#ifdef GX_DOTNET
	/** Zooms an image to a specified size.
	 * @param src   Reference to the source image.
	 * @param xsize Horizontal size of the destination area.
	 * @param ysize Vertical size of the destination area.
	 * @param sline Size of a row of the destination area.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Zoom(gxImage GX_REF_CLASS(src), gxi32 xsize, gxi32 ysize, gxi32 sline) gxFuncThrowsError {
		return Zoom(src, xsize, ysize, sline, 0,0,0,0);
	}
#endif

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Rotates an image to a specified degree.
	 * @param src  Pointer to the source image.
	 * @param degree Degree (Must be 0, 90, 180, 270).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Rotate(gxIMAGE *src, gxi32 degree) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_rotateimage(*this, _image, src, degree) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif

	/** Rotates an image to a specified degree.
	 * @param src  Reference to the source image data.
	 * @param degree Degree (Must be 0, 90, 180, 270).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Rotate(gxImageData GX_REF_CLASS(src), gxi32 degree) gxFuncThrowsError {
		gxassert(_image);
#ifndef GX_DOTNET
		bool st = gx_rotateimage(*this, _image, src, degree) ? true : false;
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_rotateimage(h, i, (gxIMAGE *)src GX_M _get_image().ToPointer(), degree) ? true : false;
#endif
		gxcondthrow(!st);
		return st;
	}

	/** Rotates an image to a specified degree.
	 * @param src  Reference to the source image.
	 * @param degree Degree (Must be 0, 90, 180, 270).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Rotate(gxImage GX_REF_CLASS(src), gxi32 degree) gxFuncThrowsError {
		gxassert(_image);
#ifndef GX_DOTNET
		bool st = gx_rotateimage(*this, _image, src, degree) ? true : false;
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_rotateimage(h, i, (gxIMAGE *)src GX_M _get_image().ToPointer(), degree) ? true : false;
#endif
		gxcondthrow(!st);
		return st;
	}

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Mirrors an image to a specified mode.
	 * @param src Pointer to an image.
	 * @param flags Mirror flags. It can be \ref GX_MIRROR_HORIZONTAL or/and \ref GX_MIRROR_VERTICAL.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Mirror(gxIMAGE *src, gxi32 flags) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_mirrorimage(*this, _image, src, flags) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif
	/** Mirrors an image to a specified mode.
	 * @param src Reference to an image data.
	 * @param flags Mirror flags. It can be \ref GX_MIRROR_HORIZONTAL or/and \ref GX_MIRROR_VERTICAL.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Mirror(gxImageData GX_REF_CLASS(src), gxi32 flags) {
		gxassert(_image);
#ifndef GX_DOTNET
		bool st = gx_mirrorimage(*this, _image, src, flags) ? true : false;
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_mirrorimage(h, i, (gxIMAGE *)src GX_M _get_image().ToPointer(), flags) ? true : false;
#endif
		gxcondthrow(!st);
		return st;
	}

	/** Mirrors an image to a specified mode.
	 * @param src Reference to an image.
	 * @param flags Mirror flags. It can be \ref GX_MIRROR_HORIZONTAL or/and \ref GX_MIRROR_VERTICAL.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Mirror(gxImage GX_REF_CLASS(src), gxi32 flags) {
		gxassert(_image);
#ifndef GX_DOTNET
		bool st = gx_mirrorimage(*this, _image, src, flags) ? true : false;
#else
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_mirrorimage(h, i, (gxIMAGE *)src GX_M _get_image().ToPointer(), flags) ? true : false;
#endif
		gxcondthrow(!st);
		return st;
	}

	/** Mirrors an image to a specified mode.
	 * @param flags Mirror flags. It can be \ref GX_MIRROR_HORIZONTAL or/and \ref GX_MIRROR_VERTICAL.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Mirror(gxi32 flags) {
		gxassert(_image);
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_mirrorimage(h, i, flags) ? true : false;
		gxcondthrow(!st);
		return st;
	}

#if !defined(GX_SWIG) && !defined(GX_DOTNET)
	/** Loads an image from a mass storage device and converts it to a specific pixel
	 * format (Unicode version).
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::Load() must be -1 for RAW images.
	 *
	 * \code
	   gxImage image;
	   image.Create(GX_GRAY, 1024, 768, 1024);
	   image.Load("test.raw", -1);
	   ...
	   \endcode
	 *
	 * @param filename Name of the image file.
	 * @param pixelformat The specific pixel format. Giving \ref GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @param properties Properties for the image loader. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Load(const wchar_t *filename, gxi32 pixelformat GX_DEFARG(GX_UNDEF),
					gxVARIANT properties GX_DEFARG(0),
					gxVARIANT comments GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_loadimagefromfile(_image, filename, pixelformat,
						properties, comments) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Loads an image from a mass storage device and converts it to a specific pixel
	 * format (ASCII version).
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::Load() must be -1 for RAW images.
	 *
	 * \code
	   gxImage image;
	   image.Create(GX_GRAY, 1024, 768, 1024);
	   image.Load("test.raw", -1);
	   ...
	   \endcode
	 *
	 * @param filename Name of the image file.
	 * @param pixelformat The specific pixel format. Giving \ref GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @param properties Properties for the image loader. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Load(const char *filename, gxi32 pixelformat GX_DEFARG(GX_UNDEF),
					gxVARIANT properties GX_DEFARG(0),
					gxVARIANT comments GX_DEFARG(0)) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_loadimagefromfile(_image, filename, pixelformat,
						properties, comments) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif
#ifdef GX_UNICODE
	/** Loads an image with comments from a mass storage device and converts it
	 * to a specific pixel format (Unicode version).
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::Load() must be -1 for RAW images.
	 *
	 * \ifnot NET
	 * \code
	   gxImage image;
	   image.Create(GX_GRAY, 1024, 768, 1024);
	   image.Load("test.raw", -1);
	   ...
	   \endcode
	 * \endif
	 *
	 * @param filename Name of the image file.
	 * @param pixelformat The specific pixel format. Giving \ref GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @param properties Properties for the image loader.
	 * @param comments Comments of the image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Load(gxInStr filename, gxi32 pixelformat,
#ifndef GX_DOTNET
					gxVariant &properties, gxVariant &comments
#else
	#ifndef GX_DOTNET_20
					gxVariant *properties, gxVariant *comments
	#else
					gxVariant ^properties, gxVariant ^comments
	#endif
#endif
					) gxFuncThrowsError {

		GX_GETSTRING(_filename, filename);
		GX_GETIMAGE(i);
		bool st = gx_loadimagefromfile(i, _filename, pixelformat,
#ifndef GX_DOTNET
					properties._variant, comments._variant
#else
					properties ? properties->_variant : 0,
					comments ? comments->_variant : 0
#endif
					) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Loads an image from a mass storage device and converts it
	 * to a specific pixel format (Unicode version).
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::Load() must be -1 for RAW images.
	 *
	 * \ifnot NET
	 * \code
	   gxImage image;
	   image.Create(GX_GRAY, 1024, 768, 1024);
	   image.Load("test.raw", -1);
	   ...
	   \endcode
	 * \endif
	 *
	 * @param filename Name of the image file.
	 * @param pixelformat The specific pixel format. Giving \ref GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Load(gxInStr filename, gxi32 pixelformat) gxFuncThrowsError {

		GX_GETSTRING(_filename, filename);
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_loadimage(h, i, _filename, pixelformat) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Loads an image from a mass storage device (Unicode version).
	 * @param filename Name of the image file.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Load(gxInStr filename) gxFuncThrowsError {
		GX_GETSTRING(_filename, filename);
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_loadimage(h, i, _filename, (int)GX_ENUM_PATH(GX_PIXELFORMATS) GX_UNDEF) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif
#ifdef GX_ASCII
	/** Loads an image with comments from a mass storage device and converts it
	 * to a specific pixel format (ASCII version).
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::Load() must be -1 for RAW images.
	 *
	 * \code
	   gxImage image;
	   image.Create(GX_GRAY, 1024, 768, 1024);
	   image.Load("test.raw", -1);
	   ...
	   \endcode
	 *
	 * @param filename Name of the image file.
	 * @param pixelformat The specific pixel format. Giving \ref GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @param properties Properties for the image loader.
	 * @param comments Comments of the image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Load(gxInAStr filename, gxi32 pixelformat,
				gxVariant &properties, gxVariant &comments) gxFuncThrowsError {

		GX_GETASTRING(_filename, filename);
		bool st = gx_loadimagefromfile(_image, _filename, pixelformat,
					properties._variant, comments._variant) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Loads an image from a mass storage device and converts it to a specific pixel
	 * format (ASCII version).
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::Load() must be -1 for RAW images.
	 *
	 * \code
	   gxImage image;
	   image.Create(GX_GRAY, 1024, 768, 1024);
	   image.Load("test.raw", -1);
	   ...
	   \endcode
	 *
	 * @param filename Name of the image file.
	 * @param pixelformat The specific pixel format. Giving \ref GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Load(gxInAStr filename, gxi32 pixelformat GX_DEFARG(GX_UNDEF)) gxFuncThrowsError {
		GX_GETASTRING(_filename, filename);
		bool st = gx_loadimage(*this, _image, _filename, pixelformat) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Saves an image to a mass storage device in the specific file format
	 * (Unicode version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(const wchar_t *filename, gxi32 fileformat) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_saveimage(*this, _image, filename, fileformat) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Saves an image to a mass storage device in the specific file format
	 * (ASCII version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(const char *filename, gxi32 fileformat) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_saveimage(*this, _image, filename, fileformat) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Saves an image with comments to a mass storage device in the specific
	 * file format (Unicode version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @param properties Properties for the image writer. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(const wchar_t *filename, gxi32 fileformat,
					gxVariant &properties, gxVariant &comments) gxFuncThrowsError {

		gxassert(_image);
		bool st = gx_saveimagetofile(_image, filename, fileformat,
						properties._variant, comments._variant) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Saves an image with comments to a mass storage device in the specific
	 * file format (ASCII version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @param properties Properties for the image writer. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(const char *filename, gxi32 fileformat,
					gxVariant &properties, gxVariant &comments) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_saveimagetofile(_image, filename, fileformat,
					properties._variant, comments._variant) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif
#ifdef GX_UNICODE
	/** Saves an image to a mass storage device in the specific file format
	 * (Unicode version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @param properties Properties for the image writer. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(gxInStr filename, gxi32 fileformat,
				gxVariant GX_REF_CLASS(properties), gxVariant GX_REF_CLASS(comments)
				) gxFuncThrowsError {

		GX_GETSTRING(_filename, filename);
		GX_GETIMAGE(i);
		bool st = gx_saveimagetofile(i, _filename, fileformat,
#ifndef GX_DOTNET
				properties._variant, comments._variant
#else
				properties ? properties->_variant : 0,
				comments ? comments->_variant : 0
#endif
				) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Saves an image to a mass storage device in the specific file format
	 * (Unicode version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(gxInStr filename, gxi32 fileformat) gxFuncThrowsError {
		GX_GETSTRING(_filename, filename);
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_saveimage(h, i, _filename, fileformat) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif
#ifdef GX_ASCII
	/** Saves an image with comments to a mass storage device in the specific
	 * file format (ASCII version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @param properties Properties for the image writer. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(gxInAStr filename, gxi32 fileformat,
				gxVariant &properties, gxVariant &comments) gxFuncThrowsError {
		GX_GETASTRING(_filename, filename);
		return Save(_filename, fileformat, properties, comments);
	}

	/** Saves an image to a mass storage device in the specific file format
	 * (ASCII version).
	 * @param filename Name of the image file.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Save(gxInAStr filename, gxi32 fileformat) gxFuncThrowsError {
		GX_GETASTRING(_filename, filename);
		return Save(_filename, fileformat);
	}
#endif

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)
	/** Loads an image with comments from the memory and converts it to
	 * a specific pixel format.
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::LoadFromMem() must be -1 for RAW images.
	 *
	 * @param buffer Pointer to the image data.
	 * @param buflen Length of image data in bytes.
	 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @param properties Properties for the image loader.
	 * @param comments Comments of the image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool LoadFromMem(const void *buffer, gxi32 buflen, gxi32 pixelformat,
					gxVariant &properties, gxVariant &comments) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_loadimagefrommem(_image, buffer, buflen, pixelformat,
					properties._variant, comments._variant) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Saves an image with comments in a specific file format to a
	 * specific memory area.
	 * @param buffer Pointer to the memory area pointer that is large enough or NULL.
	 * @param buflen Size of the memory area.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @param properties Properties for the image writer. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool SaveToMem(void **buffer, gxi32 *buflen, gxi32 fileformat,
					gxVariant &properties, gxVariant &comments) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_saveimagetomem(_image, buffer, buflen, fileformat,
					properties._variant, comments._variant) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Loads an image from the memory and converts it to a specific pixel format.
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::LoadFromMem() must be -1 for RAW images.
	 *
	 * @param buffer Pointer to the image data.
	 * @param buflen Length of image data in bytes.
	 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool LoadFromMem(const void *buffer, gxi32 buflen, gxi32 pixelformat GX_DEFARG(GX_UNDEF)) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_loadfrommem(*this, _image, buffer, buflen, pixelformat) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Saves an image in a specific file format to a specific memory area.
	 * @param buffer Pointer to the memory area pointer that is large enough or NULL.
	 * @param buflen Size of the memory area.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool SaveToMem(void **buffer, gxi32 *buflen, gxi32 fileformat) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_savetomem(*this, _image, buffer, buflen, fileformat) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif

#if defined(GX_SWIG) || defined(GX_DOTNET)	/*(*/
	/** Loads an image with comments from the memory and converts it to
	 * a specific pixel format.
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::LoadFromMem() must be -1 for RAW images.
	 *
	 * @param buffer The image data.
	 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @param properties Properties for the image loader.
	 * @param comments Comments of the image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool LoadFromMem(gxInData buffer, gxi32 pixelformat,
				gxVariant GX_REF_CLASS(properties), gxVariant GX_REF_CLASS(comments)
				) gxFuncThrowsError {

		GX_GETDATA(buffer);
		GX_GETIMAGE(i);
		bool st = gx_loadimagefrommem(i, GX_DATAPTR(buffer), GX_DATALEN(buffer), pixelformat,
#ifndef GX_DOTNET
					properties._variant, comments._variant
#else
					properties ? properties->_variant : 0,
					comments ? comments->_variant : 0
#endif
					) ? true : false;
		GX_FREEDATA(buffer);
		if(!st) gxthrow;
		return st;
	}

	/** Saves an image with comments in a specific file format to a specific memory area.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @param properties Properties for the image writer. It can be empty.
	 * @param comments Comments of the image. It can be empty.
	 * @return Image data. An exception is thrown or null is returned in case of error. */
	inline gxOutData SaveToMem(gxi32 fileformat,
					gxVariant GX_REF_CLASS(properties), gxVariant GX_REF_CLASS(comments)
					) gxFuncThrowsError {

		void *buffer = (void *)0;
		gxi32 buflen = 0;
		GX_GETIMAGE(i);
		if(!gx_saveimagetomem(i, &buffer, &buflen, fileformat,
#ifndef GX_DOTNET
					properties._variant, comments._variant
#else
					properties ? properties->_variant : 0,
					comments ? comments->_variant : 0
#endif
					)) {
			gxthrow;
			return GX_PUTDATA_NONE;
		}
		gxOutData ret = GX_PUTDATA(buffer, buflen);
		if(buffer) gx_globalfree(buffer);
		return ret;
	}

	/** Loads an image from the memory and converts it to a specific pixel format.
	 *
	 * On loading a RAW image, the image must be created with correct
	 * parameters (sline parameter also included). The format parameter of the
	 * gxImage::LoadFromMem() must be -1 for RAW images.
	 *
	 * @param buffer The image data.
	 * @param pixelformat The specific pixel format. Giving GX_GRAY converts a color
	 *               image to grayscale (see \ref GX_PIXELFORMATS).
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool LoadFromMem(gxInData buffer, gxi32 pixelformat) gxFuncThrowsError {
		GX_GETDATA(buffer);
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_loadfrommem(h, i, GX_DATAPTR(buffer), GX_DATALEN(buffer), pixelformat) ? true : false;
		GX_FREEDATA(buffer);
		if(!st) gxthrow;
		return st;
	}

	/** Saves an image in a specific file format to a specific memory area.
	 * @param fileformat Format of the image file (see \ref GX_IMGFILEFORMATS).
	 * @return Image data. An exception is thrown or null is returned in case of error. */
	inline gxOutData SaveToMem(gxi32 fileformat) gxFuncThrowsError
	{
		void *buffer = (void *)0;
		gxi32 buflen = 0;
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		if(!gx_savetomem(h, i, &buffer, &buflen, fileformat)) {
			gxthrow;
			return GX_PUTDATA_NONE;
		}
		gxOutData ret = GX_PUTDATA(buffer, buflen);
		if(buffer) gx_globalfree(buffer);
		return ret;
	}
#endif	/*)*/

	/** Checks if the image structure and the handle are valid.
	 * @return In case of valid image the return value is non-zero. */
	inline bool IsValid(void) GX_CONST {
		if(!gxHandle::IsValid()) return false;
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		return gx_isvalidimage(h, i) ? true : false;
	}

#ifdef WIN32	/*(*/
#if !defined(GX_SWIGIF)
	/** Displays an image.
	 * @param hdc Handle of device context.
	 * @param x   X1 coordinate of display.
	 * @param y   Y1 coordinate of display.
	 * @param xsize Displayed horizontal size of image.
	 * @param ysize Displayed vertical size of image.
	 * @return An exception is thrown or false is returned in case of error. */
#ifndef GX_DOTNET
	inline bool DisplayImage(HDC hdc, gxi32 x, gxi32 y, gxi32 xsize, gxi32 ysize) gxFuncThrowsError {
		gxassert(_image);
		bool st = gx_displayimage(*this, hdc, x, y, xsize, ysize, _image) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#else
	inline bool DisplayImage(gxInPtr hdc, gxi32 x, gxi32 y, gxi32 xsize, gxi32 ysize) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
		bool st = gx_displayimage(h, hdc.ToPointer(), x, y, xsize, ysize, _image) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif
#endif

	/** Copies the image to the clipboard.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Copy(void) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_imgtoclipboard(h, i) ? true : false;
		gxcondthrow(!st);
		return st;
	}

	/** Inserts an image from the clipboard.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool Paste(void) gxFuncThrowsError {
		gxassert(_image);
		GX_GETHANDLE(h);
		GX_GETIMAGE(i);
		bool st = gx_pasteimage(h, i) ? true : false;
		gxcondthrow(!st);
		return st;
	}
#endif	/*)*/

	/** Loads an embedded image from a variant.
	 *
	 * @param variant	Variant with an embedded image.
	 * @return An exception is thrown or false is returned in case of error. */
	inline bool FromVariant(
		gxVariant GX_REF_CLASS(variant)
		) gxFuncThrowsError {

		GX_GETIMAGE(i);
#ifndef GX_DOTNET
		bool st = gx_imagefromvariant(&i, variant._variant) ? true : false;
		if(st) {
			gx_unrefimage(_image);
			_image=i;
		}
#else
		bool st = gx_imagefromvariant(&i,
			variant ? variant->_variant : 0 ) ? true : false;
		if(st) {
			gx_unrefimage(_image);
			_set_image(i);
		}
#endif
		gxcondthrow(!st);
		return st;
	}

	/** Embedds an image into a variant.
	 *
	 * @param varid		The id for the created variant.
	 * @return An exception is thrown or false is returned in case of error. */
	inline GX_INDIRECTION1(gxVariant) ToVariant(gxi32 varid) gxFuncThrowsError {

		GX_GETIMAGE(i);
		gxVARIANT vari;
		bool st = gx_imagetovariant(i, varid, &vari) != 0;
		gxcondthrow(!st);
		if(!st) return GX_NULLPTR;
		GX_INDIRECTION(gxVariant,vr) = gxnew gxVariant(vari);
		gx_unrefvariant(vari);
		if(!vr){gxSystem::SetError((int)GX_ENUM_PATH(GX_ERROR_CODES) GX_ENOMEM);gxthrow;}
		return vr;
	}

#if !defined(GX_SWIG) && !defined(GX_DOTNET)
	/** Tests if a file is an image file.
	 *
	 * @param filename Name of the file to test.

	 * @return An exception is thrown or 0 is returned in case of error, -1 is returned
	 * for unknown file type. */
	inline gxi32 TestImageFileFormat(const wchar_t *filename) gxFuncThrowsError {
		gxassert(filename);
		gxi32 fileformat;
		bool st = gx_testimagefileformat(filename, &fileformat) ? true : false;
		if(!st) { gxthrow; return 0; }
		return fileformat;
	}

	/** Tests if a file is an image file.
	 *
	 * @param filename Name of the file to test.

	 * @return An exception is thrown or 0 is returned in case of error, -1 is returned
	 * for unknown file type. */
	inline gxi32 TestImageFileFormat(const char *filename) gxFuncThrowsError {
		gxassert(filename);
		gxi32 fileformat;
		bool st = gx_testimagefileformat(filename, &fileformat) ? true : false;
		if(!st) { gxthrow; return 0; }
		return fileformat;
	}

	/** Tests if a memory area contains an image file.
	 *
	 * @param buffer Pointer to the memory area.
	 * @param buflen Size of the memory area.
	 * @return An exception is thrown or 0 is returned in case of error, -1 is returned
	 * for unknown file type. */
	inline gxi32 TestImageFileFormat(const void *buffer, gxi32 buflen) gxFuncThrowsError {
		gxassert(buffer && buflen>0);
		gxi32 fileformat;
		bool st = gx_testimagefileformat(buffer, buflen, &fileformat) ? true : false;
		if(!st) { gxthrow; return 0; }
		return fileformat;
	}
#else
	/** Tests if a memory area contains an image file.
	 *
	 * @param buffer The file data.
	 * @return An exception is thrown or 0 is returned in case of error, -1 is returned
	 * for unknown file type. */
	inline gxi32 TestImageFileFormat(gxInData buffer) gxFuncThrowsError {
		GX_GETDATA(buffer);
		gxi32 fileformat;
		bool st = gx_testimagefileformat(GX_DATAPTR(buffer), GX_DATALEN(buffer), &fileformat) ? true : false;
		GX_FREEDATA(buffer);
		if(!st) { gxthrow; return 0; }
		return fileformat;
	}
#endif

#ifdef GX_UNICODE
	/** Tests if a file is an image file.
	 *
	 * @param filename Name of the file to test.

	 * @return An exception is thrown or 0 is returned in case of error, -1 is returned
	 * for unknown file type. */
	inline gxi32 TestImageFileFormat(gxInStr filename) gxFuncThrowsError {
		GX_GETSTRING(_filename, filename);
		gxi32 fileformat;
		bool st = gx_testimagefileformat(_filename, &fileformat) ? true : false;
		if(!st) { gxthrow; return 0; }
		return fileformat;
	}
#endif

#ifdef GX_ASCII
	/** Tests if a file is an image file.
	 *
	 * @param filename Name of the file to test.

	 * @return An exception is thrown or 0 is returned in case of error, -1 is returned
	 * for unknown file type. */
	inline gxi32 TestImageFileFormat(gxInAStr filename) gxFuncThrowsError {
		GX_GETASTRING(_filename, filename);
		gxi32 fileformat;
		bool st = gx_testimagefileformat(_filename, &fileformat) ? true : false;
		if(!st) { gxthrow; return 0; }
		return fileformat;
	}
#endif

};

#if !defined(GX_SWIGIF) && !defined(GX_DOTNET)

/* gxImageRef class
 * Wrapper class for the gxIMAGE structure.
 * Purpose: Using a !read-only!, (automatically) referenced image object in multi-threaded environment.
 * Problem with gxImage class: Copy constructor always makes a copy of the image.
 * Problem with gxIMAGE structure: Needs manual referencing.
 *
 * If a gxImageRef object is a member of another class then the class's default copy constructor / =operator
 * takes care of the referencing (indirectly).
*/

class gxImageRef
{
    gxIMAGE* gximg;

protected:
    inline void Ref() {if(gximg) gx_refimage(gximg);}
    inline void Unref() {if(gximg) gx_unrefimage(gximg);}

public:
    inline gxImageRef();
    inline gxImageRef(gxIMAGE* img);
    inline ~gxImageRef();
    inline gxImageRef(const gxImageRef& other);
    inline gxImageRef& operator=(const gxImageRef& other);

    inline bool IsNull() {return 0==gximg;}
    inline const gxIMAGE* Image() const {return gximg;}
    inline void Release() {Unref(); gximg=0;}

	inline bool Touch();
    inline gxImageRef& CopyFrom(gxImageRef &src);

#if 0
    inline bool Load(const char* filename,gxi32 pixelformat=GX_UNDEF);
    inline bool Load(const wchar_t* filename,gxi32 pixelformat=GX_UNDEF);
    inline bool LoadFromMem(void* buffer, gxi32 buflen, gxi32 pixelformat=GX_UNDEF);
    inline bool Save(const char* filename,gxi32 fileformat);
    inline bool Save(const wchar_t* filename,gxi32 fileformat);
    inline bool SaveToMem(void** buffer,gxi32* buflen,gxi32 fileformat,gxVARIANT properties=0,gxVARIANT comments=0);
#endif
};

// gxImageRef method definitions

inline gxImageRef::gxImageRef()
{
    gximg=0;
}

inline gxImageRef::gxImageRef(gxIMAGE* img)
{
    this->gximg=img;
    Ref();
}

inline gxImageRef::~gxImageRef()
{
    Release();
}

inline gxImageRef::gxImageRef(const gxImageRef& other)
{
    gximg=other.gximg;
    if(gximg) Ref();
}

inline gxImageRef& gxImageRef::operator=(const gxImageRef& other)
{
    if(gximg!=other.gximg)
    {
        Unref();
        gximg=other.gximg;
        Ref();
    }

    return *this;
}
#if 0
inline bool gxImageRef::Load(const char *filename, gxi32 pixelformat)
{
    if(gximg) Release();
    gx_allocimage(&gximg);
    int ret = gx_loadimage(gximg,filename,pixelformat);
    if(0==ret) {Release();}
    return ret!=0;
}

inline bool gxImageRef::Load(const wchar_t *filename, gxi32 pixelformat)
{
    if(gximg) Release();
    gx_allocimage(&gximg);
    int ret = gx_loadimage(gximg,filename,pixelformat);
    if(0==ret) {Release();}
    return ret!=0;
}

inline bool gxImageRef::LoadFromMem(void *buffer, gxi32 buflen, gxi32 pixelformat)
{
    if(gximg) Release();
    gx_allocimage(&gximg);
    int ret = gx_loadfrommem(gximg,buffer,buflen,pixelformat);
    if(0==ret) {Release();}
    return ret!=0;
}

inline bool gxImageRef::Save(const char* filename,gxi32 fileformat)
{
    if(!gximg) return false;
    int ret=gx_saveimage(gximg,filename,fileformat);
    return 0!=ret;
}

inline bool gxImageRef::Save(const wchar_t* filename,gxi32 fileformat)
{
    if(!gximg) return false;
    int ret=gx_saveimage(gximg,filename,fileformat);
    return 0!=ret;
}

inline bool gxImageRef::SaveToMem(void** buffer,gxi32* buflen,gxi32 fileformat,gxVARIANT properties,gxVARIANT comments)
{
    if(!gximg) return false;
    int ret=gx_saveimagetomem(gximg,buffer,buflen,fileformat,properties,comments);
    return 0!=ret;
}

#endif

inline bool gxImageRef::Touch()
{
    if(!gximg) return true;
    if(1==gximg->ref) return true;
    gxIMAGE* new_img=0;
    int ret=0;
    ret = gx_allocimage(&new_img);
    if(0==ret) {Release(); return false;}
    ret = gx_copyimage(new_img,gximg);
    Release();
    if(0!=ret)
    {
        gximg=new_img;
        return true;
    } else {
        if(new_img) gx_unrefimage(new_img);
        return false;
    }
    return false;
}

inline gxImageRef& gxImageRef::CopyFrom(gxImageRef& src)
{
    *this = src;
    this->Touch();
    return *this;
}
// end of gxImageRef method definitions

#endif

#endif	/* GX_EXT_MODULE )*/

#endif /* NO_GX_CLASSES )*/
/******************************************************************************/
#ifdef GX_NAMESPACES
	}
#endif
#ifdef __BORLANDC__
	#pragma warn .rch
#endif
/******************************************************************************/

#endif
