
#ifndef GXIMAGE_TYPES_AND_CONSTANTS_H
#define GXIMAGE_TYPES_AND_CONSTANTS_H
/******************************************************************************/
#include "gxsd.h"
#if defined(WIN32) && !defined(GX_DOTNET)
	#include <windows.h>
#endif
#include <assert.h>
#include "gxerror.h"
#include "gxvariant.h"
/******************************************************************************/
#ifdef __BORLANDC__
	#pragma warn -rch
#endif
/******************************************************************************/
#ifdef GX_NAMESPACES
	namespace gx {
#endif
/******************************************************************************/
#ifdef GX_DOTNET
	#pragma managed
#endif
/******************************************************************************/
#ifndef GX_EXT_MODULE	/*(*/
/******************************************************************************/
/** Enumerations for pixel format */
GX_ENUM GX_PIXELFORMATS {
	GX_UNDEF	= 0x0000,		/**< Undefined format (for gx_loadimage() functions) */

	GX_GRAY		= 0x0001,		/**<  8 bit: 8 bit (grayscale) */
	GX_RGB555	= 0x0002,		/**< 16 bit: 1-5-5-5 bit (0,R,G,B) */
	GX_BGR555	= 0x0003,		/**< 16 bit: 1-5-5-5 bit (0,B,G,R) */
	GX_RGB565	= 0x0004,		/**< 16 bit: 5-6-5 bit (R,G,B) */
	GX_BGR565	= 0x0005,		/**< 16 bit: 5-6-5 bit (B,G,R) */
	GX_RGB		= 0x0006,		/**< 24 bit: 8-8-8 bit (R,G,B) */
	GX_BGR		= 0x0007,		/**< 24 bit: 8-8-8 bit (B,G,R) */
	GX_RGBA		= 0x0008,		/**< 32 bit: 8-8-8-8 bit (R,G,B,0) */
	GX_BGRA		= 0x0009,		/**< 32 bit: 8-8-8-8 bit (B,G,R,0) */
	GX_YUV422	= 0x000A,		/**< 32 bit/2 pixel: 8-8-8-8 bit (U,Y1,V,Y2) */
	GX_GRAY12	= 0x000B,		/**< 16 bit: 4-12 bit (0,grayscale) */
	GX_RGB12	= 0x000C,		/**< 48 bit: 4-12-4-12-4-12 bit (0,R,0,G,0,B) */
	GX_BGR12	= 0x000D,		/**< 48 bit: 4-12-4-12-4-12 bit (0,B,0,G,0,R) */
	GX_GBRG		= 0x000E,		/**< 32 bit/4 pixel: 8-8-8-8 bit Bayern-pattern (G,B,R,G) */
	GX_BGGR		= 0x000F,		/**< 32 bit/4 pixel: 8-8-8-8 bit Bayern-pattern (B,G,G,R) */
	GX_RGGB		= 0x0010,		/**< 32 bit/4 pixel: 8-8-8-8 bit Bayern-pattern (R,G,G,B) */
	GX_GRBG		= 0x0011,		/**< 32 bit/4 pixel: 8-8-8-8 bit Bayern-pattern (G,R,B,G) */

	GX_IMGFILE	= 0xFFFF,		/**< Special gxIMAGE: it contains a memory image file(Available formats: GX_BMP,GX_JPEG,GX_PNG). */

	GX_FORMAT_FIRST	= GX_GRAY,	/**< First available pixel format */
	GX_FORMAT_LAST	= GX_GRBG	/**< Last available pixel format */
};

/** Enumerations for image file format */
GX_ENUM GX_IMGFILEFORMATS {
	GX_BMP			= 1,			/**< BMP format */
	GX_JPEG			= 2,			/**< JPEG format (ISO/IEC 10918-1) */
	GX_JPEG2K_JPC	= 3,			/**< JPEG-2000 Code stream syntax (ISO/IEC 15444-1) */
	GX_JPEG2K_JP2	= 4,			/**< JPEG-2000 JP2 format syntax (ISO/IEC 15444-1) */
	GX_JPEG2K		= GX_JPEG2K_JP2,/**< JPEG-2000 JP2 format syntax (ISO/IEC 15444-1) */
	GX_RAW			= 5,			/**< RAW format (uncompressed pixel data without header) */
	GX_PNG			= 6,			/**< PNG format (Portable Network Graphics) */
	GX_WSQ			= 7				/**< WSQ format (Wavelet Scalar Quantization) */
};

/** Enumerations for mirror operations */
GX_ENUM GX_MIRRORFLAGS {
	GX_MIRROR_NOMIRROR	 = 0,	/**< No mirroring */
	GX_MIRROR_HORIZONTAL = 1,	/**< Horizontal mirroring */
	GX_MIRROR_VERTICAL	 = 2	/**< Vertical mirroring */
};

/** Enumerations for zoom modes */
GX_ENUM GX_ZOOMMODES {
	GX_ZOOM_NEAREST	= 0,		/**< Nearest algorithm (faster than linear) */
	GX_ZOOM_LINEAR	= 1			/**< Linear algorithm (better than nearest) */
};

/** Enumerations for quick convert */
GX_ENUM GX_QUICKCONVERT {
	GX_CONVERT_NORMAL	= 0,	/**< Normal algorithm */
	GX_CONVERT_QUICK	= 1		/**< Quick algorithm */
};

/** Enumerations for quick display */
GX_ENUM GX_QUICKDISPLAY {
	GX_DISPLAY_NORMAL	= 0,	/**< Normal algorithm */
	GX_DISPLAY_QUICK	= 1		/**< Quick algorithm */
};

/** Enumerations for image comments */
GX_ENUM GX_IMAGECOMMENTS {
	GX_IMGCOMM_GENERAL = 0,		/**< Type of the general image comment. */
	GX_IMGCOMM_JPEG_APP0,		/**< APP0 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP1,		/**< APP1 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP2,		/**< APP2 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP3,		/**< APP3 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP4,		/**< APP4 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP5,		/**< APP5 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP6,		/**< APP6 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP7,		/**< APP7 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP8,		/**< APP8 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP9,		/**< APP9 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP10,		/**< APP10 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP11,		/**< APP11 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP12,		/**< APP12 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP13,		/**< APP13 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP14,		/**< APP14 comment for jpeg applications. */
	GX_IMGCOMM_JPEG_APP15,		/**< APP15 comment for jpeg applications. */
	GX_IMGCOMM_INVALIDFORMAT	/**< Invalid format signal at image loading. */
};

/** Enumerations for image properties */
GX_ENUM GX_IMAGEPROPERTIES1 {
	GX_IMGPROP_BMP_FORCE24BPP = GX_CALL_GROUP_GX_BMP,		/**< Identifier for bmp/force24bpp property.
															 *
															 * The BMP writer forces the 24 bit per pixel mode.
															 * It can be:
															 *	- 0 : the indexed mode is used instead (default)
															 *	- 1 : the writer uses 24 bit per pixel (RGB/BGR) mode (faster than indexed) */

	GX_IMGPROP_BMP_ENABLERLE,								/**< Identifier for bmp/enablerle property.
															 *
															 * The BMP writer can use the RLE compression.
															 * It can be:
															 *	- 0 : disables the RLE compression (faster, default)
															 *	- 1 : enables the RLE compression */

	GX_IMGPROP_JPEG_SCALE_NUM = GX_CALL_GROUP_GX_JPEG,		/**< Identifier for jpeg/decompress/scale_num property. */
	GX_IMGPROP_JPEG_SCALE_DENOM,							/**< Identifier for jpeg/decompress/scale_denom property. */
	GX_IMGPROP_JPEG_DCT_METHOD,								/**< Identifier for jpeg/decompress/dct_method property. */
	GX_IMGPROP_JPEG_DO_FANCY_UPSAMPLING,					/**< Identifier for jpeg/decompress/do_fancy_upsampling property. */
	GX_IMGPROP_JPEG_DO_BLOCK_SMOOTHING,						/**< Identifier for jpeg/decompress/do_block_smoothing property. */

	GX_IMGPROP_JPEG_QUALITY,								/**< Identifier for jpeg/compress/quality property. */
	GX_IMGPROP_JPEG_PROGRESSIVE,							/**< Identifier for jpeg/compress/progressive property. */
	GX_IMGPROP_JPEG_OPTIMIZE_CODING,						/**< Identifier for jpeg/compress/optimize_coding property. */
	GX_IMGPROP_JPEG_RESTART_INTERVAL,						/**< Identifier for jpeg/compress/restart_interval property. */
	GX_IMGPROP_JPEG_RESTART_IN_ROWS,						/**< Identifier for jpeg/compress/restart_in_rows property. */
	GX_IMGPROP_JPEG_SMOOTHING_FACTOR,						/**< Identifier for jpeg/compress/smoothing_factor property. */

	GX_IMGPROP_JPEG_EXT_DO_ZEROMEM = GX_CALL_GROUP_GX_JPEG + 0x0100,	/**< Identifier for jpeg/decompress/do_zeromem property. */

	GX_IMGPROP_JPEG2K_IMGAREATLX = GX_CALL_GROUP_GX_JPEG2K,	/**< Identifier for jpeg2k/imgareatlx property. */
	GX_IMGPROP_JPEG2K_IMGAREATLY,							/**< Identifier for jpeg2k/imgareatly property. */
	GX_IMGPROP_JPEG2K_TILEGRDTLX,							/**< Identifier for jpeg2k/tilegrdtlx property. */
	GX_IMGPROP_JPEG2K_TILEGRDTLY,							/**< Identifier for jpeg2k/tilegrdtly property. */
	GX_IMGPROP_JPEG2K_TILEWIDTH,							/**< Identifier for jpeg2k/tilewidth property. */
	GX_IMGPROP_JPEG2K_TILEHEIGHT,							/**< Identifier for jpeg2k/tileheight property. */
	GX_IMGPROP_JPEG2K_PRCWIDTH,								/**< Identifier for jpeg2k/prcwidth property. */
	GX_IMGPROP_JPEG2K_PRCHEIGHT,							/**< Identifier for jpeg2k/prcheight property. */
	GX_IMGPROP_JPEG2K_CBLKWIDTH,							/**< Identifier for jpeg2k/cblkwidth property. */
	GX_IMGPROP_JPEG2K_CBLKHEIGHT,							/**< Identifier for jpeg2k/cblkheight property. */
	GX_IMGPROP_JPEG2K_MODE,									/**< Identifier for jpeg2k/mode property. */
	GX_IMGPROP_JPEG2K_RATE,									/**< Identifier for jpeg2k/rate property. */
	GX_IMGPROP_JPEG2K_ILYRRATES,							/**< Identifier for jpeg2k/ilyrrates property. */
	GX_IMGPROP_JPEG2K_PRG,									/**< Identifier for jpeg2k/prg property. */
	GX_IMGPROP_JPEG2K_NOMCT,								/**< Identifier for jpeg2k/nomct property. */
	GX_IMGPROP_JPEG2K_NUMRLVLS,								/**< Identifier for jpeg2k/numrlvls property. */
	GX_IMGPROP_JPEG2K_SOP,									/**< Identifier for jpeg2k/sop property. */
	GX_IMGPROP_JPEG2K_EPH,									/**< Identifier for jpeg2k/eph property. */
	GX_IMGPROP_JPEG2K_LAZY,									/**< Identifier for jpeg2k/lazy property. */
	GX_IMGPROP_JPEG2K_TERMALL,								/**< Identifier for jpeg2k/termall property. */
	GX_IMGPROP_JPEG2K_SEGSYM,								/**< Identifier for jpeg2k/segsym property. */
	GX_IMGPROP_JPEG2K_VCAUSAL,								/**< Identifier for jpeg2k/vcausal property. */
	GX_IMGPROP_JPEG2K_PTERM,								/**< Identifier for jpeg2k/pterm property. */
	GX_IMGPROP_JPEG2K_RESETPROB,							/**< Identifier for jpeg2k/resetprob property. */
	GX_IMGPROP_JPEG2K_NUMGBITS,								/**< Identifier for jpeg2k/numgbits property. */

	GX_IMGPROP_WSQ_BITRATE	= GX_CALL_GROUP_GX_WSQ					/**< Identifier for WSQ bitrate property. */
};
/******************************************************************************/
#endif	/*)*/
/******************************************************************************/
#if defined(GX_DOTNET) && !defined(GX_MANUAL)
	#pragma unmanaged
	#define gxIMAGE _gxIMAGE
	#define HDC void*
#endif
/******************************************************************************/
#if !defined(GX_SWIGIF) && !(defined(GX_MANUAL) && defined(GX_DOTNET))	/*(*/
/** Structure which stores data of an image */
struct gxIMAGE {
	gxi32				gxid;	/**< Structure identifier. */
	gxi32				ref;	/**< Reference counter. */
	void*				image;	/**< Pointer to pixel data. */
	gxi32				simage;	/**< Size of an image area in bytes (allocated space = ysize*sline) */
	gxi32				format;	/**< Pixel format (see \ref GX_PIXELFORMATS) */
	gxi32				xsize;	/**< Horizontal size of the image in pixels. */
	gxi32				ysize;	/**< Vertical size of the image in pixels. */
	gxi32				sline;	/**< Size of a row in bytes. */

	/** Horizontal resolution.
	 * \par
	 * On a free air camera system such as number plate or face capturing
	 * this value is related to a PAL or NTSC image in percentage.\n
	 * For example:
	 *	- in case of PAL image 768 pixel = 100 %, 384 pixel = 50 %
	 *	- in case of NTSC image 640 pixel = 100 %, 320 pixel = 50 %
	 * \par
	 * On a fixed object windowed camera system like document capturing
	 * this value is definied in pixel-per-meter (for example 300 DPI = 11811 pix/m).
	 * \par
	 * (0 value = not defined) */
	gxi32				xres;
	/** Vertical resolution.
	 * \par
	 * On a free air camera system such as number plate or face capturing
	 * this value is related to a PAL or NTSC image in percentage.\n
	 * For example:
	 *	- in case of PAL image 576 pixel = 100 %, 288 pixel = 50 %
	 *	- in case of NTSC image 480 pixel = 100 %, 240 pixel = 50 %
	 * \par
	 * On a fixed object windowed camera system like document capturing
	 * this value is definied in pixel-per-meter (for example 300 DPI = 11811 pix/m).
	 * \par
	 * (0 value = not defined) */
	gxi32				yres;
};
/******************************************************************************/
#ifndef __cplusplus
	typedef struct gxIMAGE gxIMAGE;	/**< Workaround for ANSI C */
#endif
/******************************************************************************/
#endif	/* GX_SWIGIF GX_MANUAL GX_DOTNET )*/
#if !defined(GX_SWIGIF) && !defined(GX_MANUAL)	/*(*/
/******************************************************************************/
/* Call codes for image operations. */
enum {
	GX_CALL_GETPIXELSIZE		= GX_CALL_GROUP_GX_IMAGE,	/**< Call code for the gx_getpixelsize() function. */
	GX_CALL_CONVERTPIXELS,		/**< Call code for the gx_convertpixels() function. */
	GX_CALL_FASTRECONVERT,		/**< Call code for the gx_fastreconvert() function. */
	GX_CALL_ZOOMPIXELS,			/**< Call code for the gx_zoompixels() function. */
	GX_CALL_ROTATEPIXELS,		/**< Call code for the gx_rotatepixels() function. */
	GX_CALL_MIRRORPIXELS,		/**< Call code for the gx_mirrorpixels() function. */
	GX_CALL_INITIMAGE,			/**< Call code for the gx_initimage() function. */
	GX_CALL_ALLOCIMAGE,			/**< Call code for the gx_allocimage() function. */
	GX_CALL_FREEIMAGE,			/**< Call code for the gx_freeimage() function. */
	GX_CALL_REFIMAGE,			/**< Call code for the gx_refimage() function. */
	GX_CALL_UNREFIMAGE,			/**< Call code for the gx_unrefimage() function. */
	GX_CALL_CREATEIMAGE,		/**< Call code for the gx_createimage() function. */
	GX_CALL_CONVERTIMAGE,		/**< Call code for the gx_convertimage() function. */
	GX_CALL_ZOOMIMAGE,			/**< Call code for the gx_zoomimage() function. */
	GX_CALL_ROTATEIMAGE,		/**< Call code for the gx_rotateimage() function. */
	GX_CALL_MIRRORIMAGE,		/**< Call code for the gx_mirrorimage() function. */
	GX_CALL_LOADIMAGE,			/**< Call code for the gx_loadimage() function (Unicode version). */
	GX_CALL_LOADIMAGEA,			/**< Call code for the gx_loadimage() function (ASCII version). */
	GX_CALL_SAVEIMAGE,			/**< Call code for the gx_saveimage() function (Unicode version). */
	GX_CALL_SAVEIMAGEA,			/**< Call code for the gx_saveimage() function (ASCII version). */
	GX_CALL_LOADFROMMEM,		/**< Call code for the gx_loadfrommem() function. */
	GX_CALL_SAVETOMEM,			/**< Call code for the gx_savetomem() function. */
	GX_CALL_ISVALIDIMAGE,		/**< Call code for the gx_isvalidimage() function. */
	GX_CALL_DISPLAYIMAGE,		/**< Call code for the gx_displayimage() function. */
	GX_CALL_GETPIXELPTR,		/**< Call code for the gx_getpixelptr() function. */
	GX_CALL_IMGTOCLIPBOARD,		/**< Call code for the gx_imgtoclipboard() function. */
	GX_CALL_PASTEIMAGE,			/**< Call code for the gx_pasteimage() function. */

	GX_CALL_LOADIMAGEFROMFILE,	/**< Call code for the gx_loadimagefromfile() function (Unicode version). */
	GX_CALL_LOADIMAGEFROMFILEA,	/**< Call code for the gx_loadimagefromfile() function (ASCII version). */
	GX_CALL_LOADIMAGEFROMMEM,	/**< Call code for the gx_loadfrommem() function. */
	GX_CALL_SAVEIMAGETOFILE,	/**< Call code for the gx_saveimagetofile() function (Unicode version). */
	GX_CALL_SAVEIMAGETOFILEA,	/**< Call code for the gx_saveimagetofile() function (ASCII version). */
	GX_CALL_SAVEIMAGETOMEM,		/**< Call code for the gx_saveimagetomem() function. */

	GX_CALL_COPYIMAGE,			/**< Call code for the gx_copyimage() function. */
	GX_CALL_IMAGETOVARIANT,		/**< Call code for the gx_imagetovariant() function. */
	GX_CALL_IMAGEFROMVARIANT,	/**< Call code for the gx_imagefromvariant() function. */

	GX_CALL_TESTIMAGEFILEFORMAT,	/**< Call code for the gx_testimagefileformat() function. */
	GX_CALL_TESTIMAGEFILEFORMATA,	/**< Call code for the gx_testimagefileformata() function. */
	GX_CALL_TESTIMAGEFILEFORMATMEM,	/**< Call code for the gx_testimagefileformatmem() function. */

	GX_CALL_GETIMAGEFILEFORMATINFO,	/**< Call code for the gx_getimagefileformatinfo() function. */
	GX_CALL_SAVEIMAGETOFILE2,	/**< Call code for the gx_saveimagetofile2() function (Unicode version). */
	GX_CALL_SAVEIMAGETOMEM2,	/**< Call code for the gx_saveimagetomem2() function. */

	GX_CALL_TESTIMAGEFILEFORMATID,	/**< Call code for the gx_testimagefileformatid() function. */
	GX_CALL_TESTIMAGEFILEFORMATIDMEM,	/**< Call code for the gx_testimagefileformatidmem() function. */
};

#ifdef GX_NAMESPACES
	}
#endif

#endif
#endif
