/** \file *********************************************************************
 *
 * pr_calib - Passport Reader sample program
 *
 * 2006-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
 ******************************************************************************/

/**
 * Calibration example
 *
 * Purpose:
 *		Show how to calibrate the device.
 *
 * Description:
 *		Connect to the device, put the calibration image onto the document window.
 *		Type 'yes' to calibrate the device or 'no' to cancel.
 *
 */

#include "prapi.h"
#include "gxsdldr.c"

#include "../__lib__.h"

/**
 * Main function
 */
int main() {
	const char *s = NULL;

	/* Opening the PR system */
	gxHANDLE hpr;	/* handle for the PR system */
	lib_function_start("Opening system files");
	if (gx_openmodulea(&hpr,"prapi","default")) lib_function_end();
	else {
		lib_displ_err();
		return 0;
	}

	/* Connecting to the first device */
	lib_function_start("Connecting to the first device");
	if (pr_usedevicen(hpr,0,PR_UMODE_FULL_CONTROL)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	lib_write_line("The device is %scalibrated.", pr_iscalibrated(hpr,0)?"":"not ");
	lib_write_line("Put the calibration image on the document window.");

	for (;;) {
		s = lib_read_line("Do you want to recalibrate? (yes/no) ");

		if (!strcmp(s, "no")) break;
		if (strcmp(s, "yes")) {
			lib_write_line(" !!! Type 'yes' or 'no' please!");
			continue;
		}

		lib_function_start("Calibrating device");
		if (pr_calibrate(hpr,0)) {
			lib_function_end();
		} else {
			lib_displ_err();
			continue;
		}
	}

	/* Overall result */
	lib_write_line("The device is %scalibrated.", pr_iscalibrated(hpr,0)?"":"not ");

	/* Closing the device */
	lib_function_start("Closing the device");
	if (pr_closedevice(hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the PR system */
	lib_function_start("Closing the PR system");
	if (gx_closehandle(&hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	return 1;
}
