/** \file *********************************************************************
 *
 * pr_capture_simple - Passport Reader sample program
 *
 * 2006-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
 ******************************************************************************/

/**
 * Simple image capturing
 *
 * Purpose:
 *		Show how to capture image.
 *
 * Description:
 *		Capture images after successful startup.
 *
 */

#include "prapi.h"
#include "gxsdldr.c"

#include "../__lib__.h"

/**
 * Main function
 */
int main() {

	/* Opening the PR system */
	gxHANDLE hpr;	/* handle for the PR system */
	lib_function_start("Opening system files");
	if (gx_openmodulea(&hpr,"prapi","default")) lib_function_end();
	else {
		lib_displ_err();
		return 0;
	}

	/* Connecting to the first device */
	lib_function_start("Connecting to the first device");
	if (pr_usedevicen(hpr,0,PR_UMODE_FULL_CONTROL)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	while(!lib_kbhit()) {

		lib_process_start("Processing document");

		/* Capturing images */
		lib_function_start("Capturing images");
		if (pr_capture(hpr)) {
			lib_function_end();
		} else {
			lib_displ_err();
			break;
		}

		/* Saving to file */
		lib_function_start("Saving image");
		if (pr_saveimagea(hpr,0,PR_LIGHT_WHITE,PR_IT_ORIGINAL,"white.bmp",GX_BMP))
			lib_function_end();
		else
			lib_displ_err();

		lib_process_end();
		lib_wait_for_sec(3);
	}

	/* Closing the device */
	lib_function_start("Closing the device");
	if (pr_closedevice(hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the PR system */
	lib_function_start("Closing the PR system");
	if (gx_closehandle(&hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	return lib_print_stat();
}
