/** \file *********************************************************************
 *
 * pr_mrz - Passport Reader sample program
 *
 * 2006-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
 ******************************************************************************/

/**
 * Simple image capturing and processing
 *
 * Purpose:
 *		Shows how to capture image and to read MRZ data.
 *
 * Description:
 *
 *		Capture images after successful startup.
 *		Read the MRZ data, the calculated checksum, and display the result.
 *
 */

#include "prapi.h"
#include "gxsdldr.c"

#include "../__lib__.h"

/**
 * Main function
 */
int main() {
	gxVARIANT pdoc;

	/* Opening the PR system */
	gxHANDLE hpr;	/* handle for the PR system */
	lib_function_start("Opening system files");
	if (gx_openmodulea(&hpr,"prapi","default")) lib_function_end();
	else {
		lib_displ_err();
		return 0;
	}

	/* Connecting to the first device */
	lib_function_start("Connecting to the first device");
	if (pr_usedevicen(hpr,0,PR_UMODE_FULL_CONTROL)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	/* Using the device */
	while(!lib_kbhit()) {
		char text[100];
		int stext;
		int status;
		char *ec[]={"Ok","Warning","Error"};

		lib_process_start("MRZ reading");

		/* Capturing images */
		lib_function_start("Capturing images");
		if (!pr_capture(hpr)) {
			lib_displ_err();
		}
		else
		{
			lib_function_end();

			/* Getting the MRZ data */
			lib_function_start("Reading MRZ");
			if (pr_getmrz(hpr,0,PR_LIGHT_INFRA,PR_IT_ORIGINAL,&pdoc)) {
				lib_function_end();
			} else {
				lib_displ_err();
			}

			if (!pdoc) {
				lib_write_line("No MRZ data found!");
			} else {
				/* Searching for MRZ fields and displaying them */
				stext = sizeof(text);
				if (pr_getfieldfromdoca(hpr, pdoc, PR_DF_MRZ1, text, &stext, &status, 0)) {
					if (*text) lib_write_line("MRZ1 \"%s\" [%s]", text, ec[status/100]);
				} else {
					lib_displ_err();
				}

				stext = sizeof(text);
				if (pr_getfieldfromdoca(hpr, pdoc, PR_DF_MRZ2, text, &stext, &status, 0)) {
					if (*text) lib_write_line("MRZ2 \"%s\" [%s]", text, ec[status/100]);
				} else {
					lib_displ_err();
				}

				stext = sizeof(text);
				if (pr_getfieldfromdoca(hpr, pdoc, PR_DF_MRZ3, text, &stext, &status, 0)) {
					if(*text) lib_write_line("MRZ3 \"%s\" [%s]", text, ec[status/100]);
				} else {
					lib_displ_err();
				}
				gx_disposevariant(&pdoc);
			}
		}

		lib_process_end();
		lib_wait_for_sec(3);
	}

	/* Closing the device */
	lib_function_start("Closing the device");
	if (pr_closedevice(hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the PR system */
	lib_function_start("Closing the PR system");
	if (gx_closehandle(&hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	return lib_print_stat();
}
