unit USample1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, AxCtrls, GXIMAGE_OCXLib_TLB, PRAPI_OCXLib_TLB, OleCtrls,
  ExtCtrls, Menus, ComCtrls;

type
  TFSample1 = class(TForm)
	prDoc: TPrDocOCX;
	prApi: TPrApiOCX;
    GroupBox2: TGroupBox;
	Img: TGxImageOCX;
    MainMenu1: TMainMenu;
    About1: TMenuItem;
    Splitter1: TSplitter;
	StatusBar: TStatusBar;
	Panel1: TPanel;
    Capture: TButton;
    Recognize: TButton;
    ReadMRZ: TButton;
    ReadBarcode: TButton;
    GroupBox1: TGroupBox;
	Data: TMemo;
    Panel2: TPanel;
    Views: TRadioGroup;
    Lights: TRadioGroup;
    Splitter2: TSplitter;
    Calibrate: TButton;
    SaveDocument: TButton;
    LoadDocument: TButton;
    SaveDialog: TSaveDialog;
    OpenDialog: TOpenDialog;
	procedure FormCreate(Sender: TObject);
	procedure About1Click(Sender: TObject);
	procedure FormClose(Sender: TObject; var Action: TCloseAction);
	procedure prApiPREvent(Sender: TObject; PREvent, param1,
	  param2: Integer);
	procedure prApiTestDocEvent(Sender: TObject; status: Integer);
	procedure CaptureClick(Sender: TObject);
    procedure ReadMRZClick(Sender: TObject);
    procedure ReadBarcodeClick(Sender: TObject);
    procedure RecognizeClick(Sender: TObject);
    procedure LightsClick(Sender: TObject);
    procedure ViewsClick(Sender: TObject);
    procedure CalibrateClick(Sender: TObject);
    procedure SaveDocumentClick(Sender: TObject);
    procedure LoadDocumentClick(Sender: TObject);
  private
	{ Private declarations }
	AStatus: array[0..2] of string;
	ImgView: string;
	Light: string;
	function ShowError(const str: string): Integer;
	function ShowDocError(const str: string): Integer;
  public
	{ Public declarations }
  end;

var
  FSample1: TFSample1;

implementation

{$R *.DFM}

uses
	{$IFDEF VER150}
	Variants,
	{$ENDIF}
	prtypes;

procedure TFSample1.FormCreate(Sender: TObject);
var
	i: Integer;
begin
	if (not prApi.UseDeviceN(0, 2)) then begin
		ShowError('<use device>');
		Application.Terminate;
		Exit;
	end;

	AStatus[0] := 'Ok';
	AStatus[1] := 'Warning';
	AStatus[2] := 'Error';

	Views.Items.Add('Original');
	Views.Items.Add('Document');
	Views.Items.Add('Preview');
	Views.ItemIndex := 0;

	ImgView := Views.Items[Views.ItemIndex];

	for i := 0 to prApi.Lights-1 do begin
		case prApi.LightListItem(i) of
			1: Lights.Items.Add('White');
			2: Lights.Items.Add('Infra');
			3: Lights.Items.Add('UV');
			4: Lights.Items.Add('Coaxial');
			5: Lights.Items.Add('Kinegram');
			6: Lights.Items.Add('Photo');
		else
			Lights.Items.Add('Unknown');
		end;
	end;
	Lights.ItemIndex := 0;
	Light := Lights.Items[Lights.ItemIndex];

	Img.ImgSrcParam := ImgView + ',' + Light;

	if (Img.Connect(prApi.GetImageSource, prApi.GetSourceHandle) = 0) then begin
		ShowError('<connect>');
		Application.Terminate;
		Exit;
	end;

	prApi.Freerunmode := PR_FRMODE_TESTDOCUMENT;
end;

procedure TFSample1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	if (not prApi.CloseDevice) then ShowError('<close device>');
end;

function TFSample1.ShowError(const str: string): Integer;
begin
	if (prApi.GetErrorCode <> 0) then MessageDlg(str + ' ' + prApi.GetErrorString, mtError, [mbOK], 0);
	Result := prApi.GetErrorCode;
end;

function TFSample1.ShowDocError(const str: string): Integer;
begin
	if (prDoc.GetErrorCode <> 0) then MessageDlg(str + ' ' + prDoc.GetErrorString, mtError, [mbOK], 0);
	Result := prDoc.GetErrorCode;
end;

procedure TFSample1.About1Click(Sender: TObject);
begin
	ShowMessage('This is a sample program for showing the functionality of the prApi and prDoc ActiveX controls.');
end;

procedure TFSample1.prApiPREvent(Sender: TObject; PREvent, param1,
  param2: Integer);
begin
	case PREvent of
		PR_EV_CAPTURESTARTED: begin
			Data.Clear;
			Data.Lines.Add('<Capture>');
			Data.Lines.Add('');
			Application.ProcessMessages;
		end;
		PR_EV_IMAGECAPTURED: begin
			Data.Lines.Add('Image captured: ' +
				'param1 = ' + IntToStr(param1) + ', ' +
				'param2 = ' + IntToStr(param2)
			);
		end;
		PR_EV_PAGECAPTURED: begin
			Data.Lines.Add('Page captured: ' +
				'param1 = ' + IntToStr(param1) + ', ' +
				'param2 = ' + IntToStr(param2)
			);
		end;
	else
		Data.Lines.Add('Event occured: PREvent = ' + IntToStr(PREvent) + ', ' +
			'param1 = ' + IntToStr(param1) + ', ' +
			'param2 = ' + IntToStr(param2)
		);
	end;
end;

procedure TFSample1.prApiTestDocEvent(Sender: TObject; status: Integer);
begin
	case status of
		PR_TD_OUT: StatusBar.SimpleText := 'No document ...';
		PR_TD_MOVE: StatusBar.SimpleText := 'Document is moving ...';
		PR_TD_NOMOVE: StatusBar.SimpleText := 'Document stopped ...';
		PR_TD_IN: StatusBar.SimpleText := 'Ready to capture ...';
	else
		StatusBar.SimpleText := 'Status: ' + IntToStr(status);
	end;
end;

procedure TFSample1.CaptureClick(Sender: TObject);
begin
	if (not prApi.Capture) then ShowError('Capture failed!');
end;

procedure TFSample1.ReadMRZClick(Sender: TObject);
var
	i: Integer;
begin
	Data.Clear;
	Data.Lines.Add('<MRZ>');
	Data.Lines.Add('');
	Application.ProcessMessages;

	prDoc.Document := prApi.GetMrz(0, prapi.LightListItem(Lights.ItemIndex), 0);

	if not prDoc.Valid then begin
		Data.Lines.Add('No MRZ found!');
	end else begin
		for i := 1 to 3 do begin
			if (prDoc.Field(1000 + i) <> '') then begin
				Data.Lines.Append('Line ' + IntToStr(i) + ': ' + prDoc.Field(1000 + i) +
					#09'[' + AStatus[prDoc.FieldStatus(1000 + i) div 100] + ']');
			end;
		end;
	end;
	Data.Lines.Add('');
	Data.Lines.Add('Done.');
end;

procedure TFSample1.ReadBarcodeClick(Sender: TObject);
var
	bc: string;
	i: Integer;
	bcd: OleVariant;
	p: PChar;
begin
	Data.Clear;
	Data.Lines.Add('<Barcode>');
	Data.Lines.Add('');
	Application.ProcessMessages;

	prDoc.Document := prApi.GetBarcode(0, prapi.LightListItem(Lights.ItemIndex), 0, 0, 0);

	if not prDoc.Valid then begin
		Data.Lines.Add('No barcode found!');
	end else begin
		prDoc.FieldBinary(3001, bcd);

		if VarIsEmpty(bcd) or (VarType(bcd) <> (varArray or varByte)) then begin
			Data.Lines.Add('Data type error!');
		end else if VarIsNull(bcd) then begin
			Data.Lines.Add('No barcode data found!');
		end else begin

			// Creating a string from the Variant data
			SetLength(bc, VarArrayHighBound(bcd, 1) - VarArrayLowBound(bcd, 1));
			p := VarArrayLock(bcd);
			for i := 1 to Length(bc) do begin
				bc[i] := p^;
				inc(p);
			end;
			VarArrayUnlock(bcd);

			if bc <> '' then begin
				Data.Lines.Add('Barcode: length = ' + IntToStr(Length(bc)) + ', ' +
					'status = ' + IntToStr(prDoc.FieldStatus(3001)));
				Data.Lines.Add('');
				Data.Lines.Add(bc);
			end;
		end;
	end;
	Data.Lines.Add('');
	Data.Lines.Add('Done.');
end;

procedure TFSample1.RecognizeClick(Sender: TObject);
var
	i: Integer;
	field: WideString;
begin
	Data.Clear;
	Data.Lines.Add('<Document data>');
	Data.Lines.Add('');
	Application.ProcessMessages;

	prDoc.Document := prApi.Recognize(0);

	if not prDoc.Valid then begin
		Data.Lines.Add('No data found!');
	end else begin
		for i := 0 to 20000 do begin
			field := prDoc.Field(i);
			if field <> '' then
				Data.Lines.Add('Field (' + IntToStr(i) + '): ' + field +
					#09'[' + AStatus[prDoc.FieldStatus(i) div 100] + ']');
		end;
	end;
	Data.Lines.Add('');
	Data.Lines.Add('Done.');
end;

procedure TFSample1.LightsClick(Sender: TObject);
begin
	prApi.PreviewLight := prapi.LightListItem(Lights.ItemIndex);
	Light := Lights.Items[Lights.ItemIndex];
	Img.ImgSrcParam := ImgView + ',' + Light;
	Img.UpdateImage;
end;

procedure TFSample1.ViewsClick(Sender: TObject);
begin
	ImgView := Views.Items[Views.ItemIndex];
	if ImgView = 'Preview' then begin
		prApi.FreerunMode := PR_FRMODE_PREVIEWLIGHT;
		StatusBar.SimpleText := 'Preview ...';
	end else begin
		prApi.FreerunMode := PR_FRMODE_TESTDOCUMENT;
	end;

	Img.ImgSrcParam := ImgView + ',' + Light;
	Img.UpdateImage;
end;

procedure TFSample1.CalibrateClick(Sender: TObject);
begin
	StatusBar.SimpleText := 'Calibrating ...';
	if not prApi.Calibrate(0) then ShowError('<Calibration>');
	StatusBar.SimpleText := '';
end;

procedure TFSample1.SaveDocumentClick(Sender: TObject);
begin
	prDoc.Document := prApi.GetDocumentRoot;
	if not prDoc.Valid then
		ShowError('Getting document root failed!');

	if SaveDialog.Execute then begin
		if prDoc.Save(SaveDialog.FileName, PR_DFF_ZIPPED) then
			ShowMessage('Document saved to ' + SaveDialog.FileName + '.')
		else
			ShowDocError('Saving document to ' + SaveDialog.FileName + ' failed!');
	end;
end;

procedure TFSample1.LoadDocumentClick(Sender: TObject);
begin
	if OpenDialog.Execute then begin
		prDoc.Document := prApi.LoadDocument(OpenDialog.FileName);
		if prDoc.Valid then begin
			ShowMessage('Document loaded from ' + OpenDialog.FileName + '.');
			LightsClick(Lights);
		end else
			ShowError('Loading document from ' + OpenDialog.FileName + ' failed!');
	end;
end;

end.
