unit USample3;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, GXIMAGE_OCXLib_TLB, PRAPI_OCXLib_TLB, OleCtrls,
  GXVARIANT_OCXLib_TLB;

type
  TFSample3 = class(TForm)
    prApi: TPrApiOCX;
    prDoc: TPrDocOCX;
    imviz: TGxImageOCX;
    imrf: TGxImageOCX;
    Start: TButton;
    Result: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    gxVariant: TGxVariantOCX;
    procedure FormCreate(Sender: TObject);
    procedure StartClick(Sender: TObject);
    procedure prApiTestDocEvent(ASender: TObject; status: Integer);
  private
    { Private declarations }
    function ShowError(const str: string): Integer;
    function ShowDocError(const str: string): Integer;
    procedure SetResult(const str: string; vok: Integer);
  public
    { Public declarations }
  end;

var
  FSample3: TFSample3;

implementation

{$R *.dfm}

uses
    prtypes, prdoctypes, gxerror;


procedure TFSample3.FormCreate(Sender: TObject);
var
    i: Integer;
begin
    //connect to the device
    if (not prApi.UseDeviceN(0, 2)) then begin
        ShowError('<use device>');
        Application.Terminate;
        Exit;
    end;

    //connect imviz and prDoc controls and set interested field id
    imviz.ImgSrcParam := IntToStr(PR_DF_VIZ_FACE);
    if (imviz.Connect(prDoc.GetImageSource, prDoc.GetSourceHandle) = 0) then begin
        ShowDocError('<connect>');
        Application.Terminate;
        Exit;
    end;

    //connect imrf and prDoc controls and set interested field id
    imrf.ImgSrcParam := IntToStr(PR_DF_RFID_FACE+1);
    if (imrf.Connect(prDoc.GetImageSource, prDoc.GetSourceHandle) = 0) then begin
        ShowDocError('<connect>');
        Application.Terminate;
        Exit;
    end;

    //set used lights
    i := prApi.GetLightMask(PR_LIGHT_WHITE, 0) + prApi.GetLightMask(PR_LIGHT_INFRA, 0);
    prApi.SetPageLight(1, i);

    //enable document detection
    prApi.FreerunMode := PR_FRMODE_TESTDOCUMENT;

end;

procedure TFSample3.prApiTestDocEvent(ASender: TObject; status: Integer);
begin
    //autostart the process if a document is placed in
    if status = PR_TD_IN then StartClick(ASender);
end;

function TFSample3.ShowError(const str: string): Integer;
begin
    if (prApi.GetErrorCode <> 0) then MessageDlg(str + ' ' + prApi.GetErrorString, mtError, [mbOK], 0);
    Result := prApi.GetErrorCode;
end;

function TFSample3.ShowDocError(const str: string): Integer;
begin
    if (prDoc.GetErrorCode <> 0) then MessageDlg(str + ' ' + prDoc.GetErrorString, mtError, [mbOK], 0);
    Result := prDoc.GetErrorCode;
end;

procedure TFSample3.StartClick(Sender: TObject);
var
    i: Integer;
    id: WideString;
    mrz: WideString;
    data: OleVariant;
begin
    //clear form
    Screen.Cursor := crHourGlass;
    result.Font.Color := clBlack;
    result.Caption := 'Working';
    Application.ProcessMessages;
    imviz.ClearImage;
    imrf.ClearImage;

    //capture images, find VIZ data and display VIZ photo
    if not prApi.Capture then begin
        ShowError('<capture>');
        SetResult('Error', -1);
        Exit;
    end;
    prDoc.Document := prApi.Recognize(0);
    imviz.UpdateImage; (* Connection and set of image source parameter was done
                          at form create. Now prDoc contains the result of
                          recognition, and we have to update the image manually.
                        *)

    if not prDoc.Valid or (imviz.XSize = 0) then begin
        SetResult('No Passport', -1);
        Exit;
    end;

    //search for RFID card
    i := prApi.GetRfidCardList;
    if i = 0 then begin
        SetResult('No RF', -1);
        Exit;
    end;

    //connect to the RFID card
    if not prApi.GetRfidCard(0, id) or not prApi.ConnectRfidCard(id) then begin
        ShowError('<connect RFID>');
        SetResult('Error', -1);
        Exit;
    end;

    //handle BAC if needed
    if not prApi.GetRfidFileB(id, PR_RFID_EF_COM, data) then begin
        if not (prApi.GetErrorCode = GX_EACCES) then begin
            ShowError('<read file>');
            SetResult('Error', -1);
            Exit;
        end;
        mrz := prDoc.Field(PR_DF_MRZ1) + #13#10 + prDoc.Field(PR_DF_MRZ2) + #13#10 + prDoc.Field(PR_DF_MRZ3);
                      (* prDoc still contains the result of recognition. *)

        if not prApi.MakeBAC(id, mrz) or not prApi.GetRfidFileB(id, PR_RFID_EF_COM, data) then begin
            ShowError('<read file>');
            SetResult('Error', -1);
            Exit;
        end;
    end;

    //parse DG2 and display RFID photo
    prDoc.Document := prApi.GetRfidFile(id, PR_RFID_EF_DG2);
    imrf.UpdateImage;  (* Connection and set of image source parameter was done
                          at form create. Now prDoc contains the parsed EF.DG2
                          data, and we have to update the image manually.
                        *)

    if not prDoc.Valid or (imrf.XSize = 0) then begin
        SetResult('No RF photo', -1);
        Exit;
    end;

    //compare photos
    prDoc.Document := prApi.CompareFace(0, 0);
    gxVariant.gxvar := prDoc.ToGxVariant;
    gxVariant.SetRefPath('C,D=' + IntToStr(PRV_FIELDLIST) + '/L/C,D=' + IntToStr(PRV_CONFIDENCE));

    SetResult(IntToStr(gxVariant.RefInt), gxVariant.RefInt);
end;

procedure TFSample3.SetResult(const str: string; vok: Integer);
begin
    if vok >= 670 then result.Font.Color := clGreen
    else if vok >= 330 then result.Font.Color := clYellow
    else result.Font.Color := clRed;
    result.Caption := str;
    Screen.Cursor := crArrow;
end;

end.
