#ifndef PR22_ENUMINFO_INCL
#define PR22_ENUMINFO_INCL
/******************************************************************************/
/** \file
 *	Enum information classes.
 */
/******************************************************************************/
#define PR22_VERSION 2.2.0.20
#include "Util/EnumInfo.inl"
#include "Util/Types.hpp"
#include <stdexcept>
//------------------------------------------------------------------------------
namespace Pr22 { namespace Util {

/// Enumeration info storage data type
struct ENUM_INFO {
	const wchar_t *name;		///< Name of an enum value
	int value;					///< Integer representation of an enum value
};
//------------------------------------------------------------------------------
/// Returns information about an enumeration
template<typename ENUM>
class EnumInfo {
public:
	/// Returns the number of values in the enum.
	static int Count() { return int(ENUM::gi(-1)-ENUM::gi(0)); }

	/// Returns a value of the enum by ordinal number.
	static ENUM GetValue(int ix) {
		ENUM_INFO *info = ENUM::gi(ix);
		if(!info->name) throw std::out_of_range("[pr22] Enum index");
		return (typename ENUM::values)info->value;
	}

	/// Returns the string representation of a value of the enum by ordinal number.
	static PrOutputString GetName(int ix) {
		ENUM_INFO *info = ENUM::gi(ix);
		if(!info->name) throw std::out_of_range("[pr22] Enum index");
		return (ToPrOutputString)info->name;
	}

	/// Converts enum value from its string representation.
	static ENUM Parse(const InputString &str) {
		return str.GetAString() ? Parse(str.GetAString()) : Parse(str.GetString());
	}
#if defined(QSTRING_H) || defined(GX_MANUAL)
	/// Converts enum value from its string representation.
	static ENUM Parse(const QString &str) { return Parse((const ushort*)str.constData()); }
#endif

	/// Converts enum value from its string representation.
	template<class CHAR>
	static ENUM Parse(const CHAR *str) {
		ENUM_INFO *info;
		if(!str) throw std::invalid_argument("[pr22] Null");
		for(int ix=0; (info=ENUM::gi(ix))->name; ix++)
			if(!cmp(info->name, str)) return ENUM(info->value);
		int ln = cmp(ENUM().ClassName(), str);
		if(ln>0) return ENUM(toint(str+ln));
		throw std::invalid_argument("[pr22] Enum string not found!");
	}

	/// Returns a forward compatibile string representation of the enum.
	static PrOutputString ToString(const ENUM &en) {
		const wchar_t *nam = en.ToStringN();
		if(nam) return (ToPrOutputString)nam;
		wchar_t buff[64];
		PrIns::snwprintf(buff, gx_arraycnt(buff), L"%ls%i", en.ClassName(), en);
		return (ToPrOutputString)buff;
	}

private:

	template<class T1, class T2>
	static int cmp(const T1 *name, const T2 *str) {
		const T2 *st0 = str;
		while(*name==*str && *name) name++, str++;
		return *name==*str ? 0 : *name ? -1 : int(str-st0);
	}

	template<class T>
	static int toint(const T *str) {
		bool mn = *str=='-';
		if(mn) str++;
		int val = 0;
		for(; *str>='0' && *str<='9'; str++) val = val*10 + *str-'0';
		if(*str) throw std::invalid_argument("[pr22] Enum string not found!");
		return mn ? -val : val;
	}
};

} // namespace Util
} // namespace Pr22
//------------------------------------------------------------------------------
#define GX_ENUM_BEGIN		GXM_CNC(GX_ENUM_DECLARE_, ENUMSTEP)		///< Macro for enum class generator
#define GXEV				GXM_CNC(GXEV_, ENUMSTEP)				///< Macro for enum class generator
#define GX_ENUM_END			GXM_CNC(GX_ENUM_END_DECL_, ENUMSTEP)	///< Macro for enum class generator
//------------------------------------------------------------------------------
#endif //PR22_ENUMINFO_INCL
