#ifndef PR22_MUTEX_INCL
#define PR22_MUTEX_INCL
/******************************************************************************/
/** \file
 *	Mutual exclusion.
 */
 /******************************************************************************/
#ifndef __cplusplus
#error C++ compiler required.
#endif
#define PR22_VERSION 2.2.0.20
#include "PrIns/gxtypes.hpp"
#if defined WIN32
#include <windows.h>
extern "C" WINBASEAPI BOOL WINAPI TryEnterCriticalSection(LPCRITICAL_SECTION lpCriticalSection);
#elif defined LINUX
#include <pthread.h>
#endif
//------------------------------------------------------------------------------
namespace Pr22 { namespace PrIns {

/// Mutex object
class Mutex {
public:
#if defined WIN32
	Mutex() { InitializeCriticalSection(&ml); }
	~Mutex() { DeleteCriticalSection(&ml); }
	void Lock(void) { EnterCriticalSection(&ml); }
	void Unlock(void) { LeaveCriticalSection(&ml); }
	bool TryLock(void) { return TryEnterCriticalSection(&ml) != 0; }
#elif defined LINUX
	Mutex() { pthread_mutex_init(&ml, 0); }
	~Mutex() { pthread_mutex_destroy(&ml); }
	void Lock(void) { pthread_mutex_lock(&ml); }
	void Unlock(void) { pthread_mutex_unlock(&ml); }
	bool TryLock(void) { return pthread_mutex_trylock(&ml) == 0; }
#endif

private:
	Mutex(const Mutex&);
	Mutex& operator=(const Mutex&);
#if defined WIN32
	CRITICAL_SECTION ml;
#elif defined LINUX
	pthread_mutex_t ml;
#endif
};

class MxLocker {
	Mutex *mx;
public:
	MxLocker(Mutex &mutex) : mx(&mutex) { mx->Lock(); }
	~MxLocker() { mx->Unlock(); }
};

} // namespace PrIns
} // namespace Pr22
//------------------------------------------------------------------------------
#endif //PR22_MUTEX_INCL
