#ifndef PR22_FINGERCOLLECTION_INCL
#define PR22_FINGERCOLLECTION_INCL
/******************************************************************************/
/**	\file
 *	Set of previously scanned fingers to save.
 */
/******************************************************************************/
#define PR22_VERSION 2.2.0.20
#include "Processing/BinData.hpp"
#include "Task/FingerTask.hpp"
#include "Util/Variant.hpp"
//------------------------------------------------------------------------------
namespace Pr22 { namespace Processing {

/// Set of previously scanned fingers to save.
class FingerCollection {
public:

	#define ENUMSTEP 1
	/// Fingerprint data file formats.
	GX_ENUM_BEGIN(FileFormat)
	#include "Processing/fingerfiles.inl"
	GX_ENUM_END(FileFormat)
	#undef ENUMSTEP

	inline BinData Save(const FileFormat & format, Util::Variant properties) const;	// Saves the set of fingers to the memory.

private:

	PrIns::gxModule apiHandle;			// Api handler.
	int fingers[11];					// Finger presence map.

	friend class Pr22::PrIns::Invoker;
	FingerCollection(const PrIns::gxModule & handle, const Task::FingerTask & task) :
		apiHandle(handle) { memcpy(fingers, task.fingers, sizeof(fingers)); }	// Constructor for internal usage only.
};

/******************************************************************************
 * This is the end of the FingerCollection interface. Now the implementation follows
 ******************************************************************************/

/** Saves the set of fingers to the memory in a specific FingerCollection::FileFormat.
 *
 * @param format		The file format.
 * @param properties	Properties for saving. The properties are described in
 *						the reference manual.
 * @return	Output raw memory data.
 */
BinData FingerCollection::Save(const FileFormat & format, Util::Variant properties) const
{
	struct fpsFingerCollectionToMem
	{
		void			**buffer;
		int				*buflen;
		PrIns::gxVARIANT fields;
	} op;
	if(properties.Type() == Util::Variant::Types::Null)
		properties = Util::Variant(0, Util::Variant::List);

	int val = 0;
	for(int ix=1; ix<(int)gx_arraycnt(fingers); ++ix) {
		if(fingers[ix] == FingerCondition::Available) val |= 1 << ix;
	}

	properties(1) = val;
	op.fields = properties._variant;
	PrIns::GlobalData buffer;
	op.buffer = &buffer.buffer;
	op.buflen = &buffer.buflen;

	if(format == FileFormat::Nist) {
		if(apiHandle.Call(PrIns::FpsApiCallCodes::FingersToNistMem, &op))
			return BinData();
	}
	else if(format == FileFormat::FingerMinutiaRecord) {
		if(apiHandle.Call(PrIns::FpsApiCallCodes::FingerMinutiaToMem, &op))
			return BinData();
	}
	return buffer.BinData();
}

#define ENUMSTEP 2
GX_ENUM_BEGIN(FingerCollection::FileFormat)
#include "Processing/fingerfiles.inl"
GX_ENUM_END(FingerCollection::FileFormat)
#undef ENUMSTEP

} // namespace Processing
} // namespace Pr22
//------------------------------------------------------------------------------
#endif //PR22_FINGERCOLLECTION_INCL
