﻿' This example shows how to get general information about the device capabilities.

Option Explicit On
Imports System.Collections.Generic
Imports Microsoft.VisualBasic
Imports Pr22
Namespace tutorial

    Class MainClass

        Private fps As FingerprintScannerDevice = Nothing

        '----------------------------------------------------------------------
        ''' <summary>
        ''' Opens the first document reader device.
        ''' </summary>
        ''' <returns></returns>
        Public Function Open() As Integer

            System.Console.WriteLine("Opening a device")
            System.Console.WriteLine()
            fps = New FingerprintScannerDevice()

            AddHandler fps.Connection, AddressOf onDeviceConnected
            AddHandler fps.DeviceUpdate, AddressOf onDeviceUpdate

            Try
                fps.UseDevice(0)
            Catch ex As Pr22.Exceptions.NoSuchDevice
                System.Console.WriteLine("No device found!")
                Return 1
            End Try

            System.Console.WriteLine("The device " + fps.DeviceName + " is opened.")
            System.Console.WriteLine()
            Return 0
        End Function
        '----------------------------------------------------------------------

        Public Function Program() As Integer

            'Devices can be manipulated only after opening.
            If Open() <> 0 Then Return 1

            System.Console.WriteLine("SDK versions:")
            System.Console.WriteLine(vbTab & "Assembly: " & fps.GetType().Assembly.GetName().Version.ToString())
            System.Console.WriteLine(vbTab & "Interface: " & fps.GetVersion("A"c))
            System.Console.WriteLine(vbTab & "System: " & fps.GetVersion("S"c))
            System.Console.WriteLine()

            Dim scannerinfo As Pr22.FingerScanner.Information = fps.Scanner.Info

            'Devices provide proper image quality only if they are calibrated.
            'Devices are calibrated by default. If you receive the message "not calibrated"
            'then please contact your hardware supplier.
            System.Console.WriteLine("Calibration state of the device:")
            If scannerinfo.IsCalibrated() Then
                System.Console.WriteLine(vbTab & "calibrated")
            Else
                System.Console.WriteLine(vbTab & "not calibrated")
            End If
            System.Console.WriteLine()

            Dim frame As System.Drawing.Rectangle = scannerinfo.GetSize(0)
            System.Console.WriteLine("Window size: " & frame.Width / 1000 & " x " & frame.Height / 1000 & " mm")
            System.Console.WriteLine()

            System.Console.WriteLine("Scanner component versions:")
            System.Console.WriteLine(vbTab & "Firmware: " & scannerinfo.GetVersion("F"c))
            System.Console.WriteLine(vbTab & "Hardware: " & scannerinfo.GetVersion("H"c))
            System.Console.WriteLine(vbTab & "Software: " & scannerinfo.GetVersion("S"c))
            System.Console.WriteLine()

            System.Console.WriteLine("Available status LEDs:")
            Dim leds As List(Of Pr22.Control.StatusLed) = fps.Peripherals.StatusLeds
            For i As Integer = 0 To leds.Count - 1
                System.Console.WriteLine(vbTab & i & ": color " & leds(i).Light.ToString())
            Next
            System.Console.WriteLine()

            System.Console.WriteLine("Closing the device.")
            fps.Close()
            Return 0
        End Function
        '----------------------------------------------------------------------
        ' Event handlers
        '----------------------------------------------------------------------

        Private Sub onDeviceConnected(ByVal a As Object, ByVal e As Pr22.Events.ConnectionEventArgs)

            System.Console.WriteLine("Connection event. Device number: " & e.DeviceNumber)
        End Sub
        '----------------------------------------------------------------------

        Private Sub onDeviceUpdate(ByVal a As Object, ByVal e As Pr22.Events.UpdateEventArgs)

            System.Console.WriteLine("Update event.")
            Select Case e.part
                Case 1
                    System.Console.WriteLine("  Reading calibration file from device.")
                Case 2
                    System.Console.WriteLine("  Scanner firmware update.")
                Case 4
                    System.Console.WriteLine("  RFID reader firmware update.")
                Case 5
                    System.Console.WriteLine("  License update.")
            End Select
        End Sub
        '----------------------------------------------------------------------

        Public Shared Function Main(ByVal args As String()) As Integer

            Try
                Dim prog As New MainClass()
                prog.Program()
            Catch e As Pr22.Exceptions.General
                System.Console.Error.WriteLine(e.Message)
            End Try
            System.Console.WriteLine("Press any key to exit!")
            System.Console.ReadKey(True)
            Return 0
        End Function
        '----------------------------------------------------------------------
    End Class
End Namespace
