﻿' This example shows how to parametrize the rolled scanning process.

Option Explicit On
Imports Microsoft.VisualBasic
Imports System.Collections.Generic
Imports Pr22
Imports Pr22.Exceptions
Imports Pr22.Imaging
Imports Pr22.Processing
Imports Pr22.Task
Imports Pr22.Util
Namespace tutorial

    Class MainClass

        Private fps As FingerprintScannerDevice = Nothing
        'progress status values
        Private Resolution As Integer = 0
        Private ScanState As Integer = 0
        Private Progress As Integer = 0
        Private Message As String = ""

        '----------------------------------------------------------------------
        ''' <summary>
        ''' Opens the first document reader device.
        ''' </summary>
        ''' <returns></returns>
        Public Function Open() As Integer

            System.Console.WriteLine("Opening a device")
            System.Console.WriteLine()
            fps = New FingerprintScannerDevice()

            AddHandler fps.Connection, AddressOf onDeviceConnected
            AddHandler fps.DeviceUpdate, AddressOf onDeviceUpdate

            Try
                fps.UseDevice(0)
            Catch ex As Pr22.Exceptions.NoSuchDevice
                System.Console.WriteLine("No device found!")
                Return 1
            End Try

            System.Console.WriteLine("The device " + fps.DeviceName + " is opened.")
            System.Console.WriteLine()
            Return 0
        End Function
        '----------------------------------------------------------------------

        Public Function Program() As Integer

            'Devices can be manipulated only after opening.
            If Open() <> 0 Then Return 1

            'Subscribing to scan events
            AddHandler fps.PreviewCaptured, AddressOf PreviewCaptured
            AddHandler fps.FingerImageUpdated, AddressOf FingerImageUpdated

            Dim Scanner As FingerScanner = fps.Scanner

            System.Console.WriteLine("Roll your left index finger on the scanner.")
            System.Console.WriteLine()

            'starting rolling scan finger collection
            Dim RollTask As FingerTask = FingerTask.RolledScan(FingerPosition.LeftIndex)
            Dim ScanTask As TaskControl = Scanner.StartTask(RollTask)

            While Progress < 100
                Progress = ScanTask.GetState()
                System.Threading.Thread.Sleep(100)
            End While
            ScanTask.Wait()
            Message = "Done"
            PrintState()

            System.Console.WriteLine()
            System.Console.WriteLine()

            'saving images
            Scanner.GetFinger(FingerPosition.LeftIndex, ImpressionType.Rolled). _
                GetImage().Save(RawImage.FileFormat.Bmp).Save("index.bmp")

            'removing the finger data before a possible additional scan process
            Scanner.CleanUpData()

            System.Console.WriteLine("Scanning processes are finished.")
            fps.Close()
            Return 0
        End Function
        '----------------------------------------------------------------------

        Private Sub PrintState()

            Dim States As String = "|/-\"
            Dim Prog As String = "##########"

            System.Console.Write(" {0} {1,3} DPI [{2,-10}] [{3,-21}]" & vbCr, _
                States(ScanState Mod 4), Resolution, Prog.Substring(10 - Progress \ 10), Message)
        End Sub
        '----------------------------------------------------------------------
        ' Event handlers
        '----------------------------------------------------------------------

        Private Sub onDeviceConnected(ByVal a As Object, ByVal e As Pr22.Events.ConnectionEventArgs)

            System.Console.WriteLine("Connection event. Device number: " & e.DeviceNumber)
        End Sub
        '----------------------------------------------------------------------

        Private Sub onDeviceUpdate(ByVal a As Object, ByVal e As Pr22.Events.UpdateEventArgs)

            System.Console.WriteLine("Update event.")
            Select Case e.part
                Case 1
                    System.Console.WriteLine("  Reading calibration file from device.")
                Case 2
                    System.Console.WriteLine("  Scanner firmware update.")
                Case 4
                    System.Console.WriteLine("  RFID reader firmware update.")
                Case 5
                    System.Console.WriteLine("  License update.")
            End Select
        End Sub
        '----------------------------------------------------------------------

        Private Sub PreviewCaptured(ByVal a As Object, ByVal e As Pr22.Events.PreviewEventArgs)

            Resolution = CInt((fps.Scanner.GetLiveImage().HRes / 39.37))
            ScanState += 1
            PrintState()
        End Sub
        '----------------------------------------------------------------------

        Private Sub FingerImageUpdated(ByVal a As Object, ByVal e As Pr22.Events.FingerUpdateEventArgs)

            fps.Scanner.GetFinger(e.position, ImpressionType.Rolled).GetImage(). _
                Save(RawImage.FileFormat.Bmp).Save(ScanState.ToString("d2") + ".bmp")
        End Sub
        '----------------------------------------------------------------------

        Public Shared Function Main(ByVal args As String()) As Integer

            Try
                Dim prog As New MainClass()
                prog.Program()
            Catch e As Pr22.Exceptions.General
                System.Console.Error.WriteLine(e.Message)
            End Try
            System.Console.WriteLine("Press any key to exit!")
            System.Console.ReadKey(True)
            Return 0
        End Function
        '----------------------------------------------------------------------
    End Class
End Namespace
